Package: TwoStepSDFM
Title: Estimate a Sparse Mixed Frequency Gaussian Factor Model Using a
        Two-Step Procedure
Version: 0.2.2
Authors@R: person(
    given = "Domenic",
    family = "Franjic",
    email = "franjic@uni-hohenheim.de",
    role = c("aut", "cre")
	)
Maintainer: Domenic Franjic <franjic@uni-hohenheim.de>
Description: Estimate a sparse Gaussian state-space model with mixed frequency
    data via sparse principal components analysis and the Kalman filter
    and smoother. For more details see Franjic and Schweikert (2024) <doi:10.2139/ssrn.4733872>.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: Rcpp (>= 1.0.8), zoo, xts, lubridate, ggplot2, patchwork,
        doSNOW, doParallel, foreach, parallel, Rdpack, grDevices, withr
RdMacros: Rdpack
LinkingTo: Rcpp, RcppEigen
NeedsCompilation: yes
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
Depends: R (>= 4.0)
LazyData: TRUE
Packaged: 2026-05-19 08:26:16 UTC; 520K-User
Author: Domenic Franjic [aut, cre]
Repository: CRAN
Date/Publication: 2026-05-19 08:50:08 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-19 11:18:22 UTC; unix
Archs: TwoStepSDFM.so.dSYM

Package: TwoStepSDFM
Title: Estimate a Sparse Mixed Frequency Gaussian Factor Model Using a
        Two-Step Procedure
Version: 0.2.2
Authors@R: person(
    given = "Domenic",
    family = "Franjic",
    email = "franjic@uni-hohenheim.de",
    role = c("aut", "cre")
	)
Maintainer: Domenic Franjic <franjic@uni-hohenheim.de>
Description: Estimate a sparse Gaussian state-space model with mixed frequency
    data via sparse principal components analysis and the Kalman filter
    and smoother. For more details see Franjic and Schweikert (2024) <doi:10.2139/ssrn.4733872>.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: Rcpp (>= 1.0.8), zoo, xts, lubridate, ggplot2, patchwork,
        doSNOW, doParallel, foreach, parallel, Rdpack, grDevices, withr
RdMacros: Rdpack
LinkingTo: Rcpp, RcppEigen
NeedsCompilation: yes
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
Depends: R (>= 4.0)
LazyData: TRUE
Packaged: 2026-05-19 08:26:16 UTC; 520K-User
Author: Domenic Franjic [aut, cre]
Repository: CRAN
Date/Publication: 2026-05-19 08:50:08 UTC

# Generated by roxygen2: do not edit by hand

S3method(plot,KFSFit)
S3method(plot,NoOfFactorsFit)
S3method(plot,SDFMFit)
S3method(plot,SDFMcrossVal)
S3method(plot,SDFMnowcast)
S3method(plot,SPCAFit)
S3method(plot,SimulData)
S3method(predict,SDFMFit)
S3method(print,KFSFit)
S3method(print,NoOfFactorsFit)
S3method(print,SDFMFit)
S3method(print,SDFMcrossVal)
S3method(print,SDFMnowcast)
S3method(print,SPCAFit)
S3method(print,SimulData)
export(crossVal)
export(kalmanFilterSmoother)
export(noOfFactors)
export(nowcast)
export(simFM)
export(sparsePCA)
export(twoStepDenseDFM)
export(twoStepSDFM)
import(doParallel)
import(doSNOW)
import(foreach)
import(ggplot2)
import(grDevices)
import(lubridate)
import(parallel)
import(patchwork)
import(stats)
import(utils)
import(withr)
import(xts)
import(zoo)
importFrom(Rcpp,sourceCpp)
importFrom(Rdpack,reprompt)
useDynLib(TwoStepSDFM, .registration=TRUE)

# TwoStepSDFM
A ``C++``-based ``R`` implementation of the two-step estimation procedure for a (linear Gaussian) Sparse Dynamic Factor Model (SDFM) as outlined in Franjic and Schweikert (2024).

## Introduction

The ``TwoStepSDFM`` package provides a fast implementation of the Kalman Filter and Smoother (hereinafter KFS, see Koopman and Durbin, 2000) to estimate factors in a mixed-frequency SDFM framework, explicitly accounting for cross-sectional correlation in the measurement error. The KFS is initialized using results from Sparse Principal Components Analysis (SPCA) by Zou and Hastie (2006) in a preliminary step. This approach generalizes the two-step estimator for approximate dynamic factor models by Giannone, Reichlin, and Small (2008) and Doz, Giannone, and Reichlin (2011). For more details see Franjic and Schweikert (2024).

## Main Features

- **Fast Model Simulation**: The ``simFM()`` function provides a flexible framework to simulate mixed-frequency data with ragged edges from an approximate DFM.
- **Estimation of the Number of Factors**: The ``noOfFactors()`` function uses the Onatski (2009) procedure to estimate the number of factors efficiently while providing good finite sample performance.
- **Fast Model Estimation**: The ``twoStepSDFM()`` function provides a fast, memory-efficient, and convenient implementation of the two-step estimator outlined in Franjic and Schweikert (2024).
- **Fast Hyper-Parameter Cross-Validation**: The ``crossVal()`` function provides a fast and parallel cross-validation wrapper to retrieve the optimal hyper-parameters using time-series cross-validation (Hyndman and Athanasopoulos 2018) with random hyper-parameter search (Bergstra and Bengio 2012).
- **Fast Model Prediction**: The ``nowcast()`` function is a highly convenient prediction function for backcasts, nowcasts, and forecasts of multiple targets. It automatically takes care of all issues arising with mixed-frequency data and ragged edges.
- **Compatibility**: All functions take advantage of C++ for enhanced speed and memory-efficiency.

## Side Features

- **Fast dense DFM estimation and prediction**: The ``nowcast()`` function is also able to produce predictions of a dense DFM according to Giannone, Reichlin, and Small (2008). The function ``twoStepDenseDFM()`` additionally exposes an estimation procedure for the dense two-step estimator.
- **Fast SPCA**: ``sparsePCA()`` exposes the internal ``C++``-backed SPCA routine in ``R``. This provides access to a fast and memory-efficient SPCA estimation routine as implemented by Zou and Hastie (2020) in pure R.
- **Fast Kalman Filter and Smoother**: The ``kalmanFilterSmoother()`` function exposes the internal ``C++``-backed KFS routine.

## Installation

The package is available on CRAN and can be installed via ``install.packages("TwoStepSDFM")``. If this turns out to be no longer possible, run the PackageBuilder.R file.

## Prerequisites

For the installation from source via the PackageBuilder.R file, the following is required:

- **Rcpp**: A package for integrating `C++` code into `R` (Eddelbuettel and François, 2011). [Rcpp CRAN repository](https://CRAN.R-project.org/package=Rcpp)
- **RcppEigen**: A package for integrating the `Eigen` linear algebra library into `R` (Bates and Eddelbuettel, 2013). [RcppEigen CRAN repository](https://CRAN.R-project.org/package=RcppEigen)
- **GCC compiler** (version 5.0 or later) [GCC Website](https://gcc.gnu.org/).

## Usage

For a quick step-by-step user guide of the main features, see the package vignette.

## License

License: GPL v3

(C) 2024-2026 Domenic Franjic

This project is licensed under the **GNU General Public License v3.0**. See the LICENSE file for details.

## Contributing

Contributions are welcome! Please open an issue or submit a pull request for any improvements or bug fixes.

**To Contribute:**

1. Fork the repository.
2. Create a new branch for your feature or bug fix.
3. Commit your changes with descriptive messages.
4. Push to your fork and submit a pull request.

## Support

If you have any questions or need assistance, please open an issue on the GitHub repository or contact us via email.

## Contact

- **Name**: Domenic Franjic
- **Institution**: University of Hohenheim
- **Department**: Econometrics and Statistics, Core Facility Hohenheim
- **E-Mail**: franjic@uni-hohenheim.de

## References

### Papers
- Bai, J., & Ng, S. (2002). Determining the number of factors in approximate factor models. Econometrica, 70(1), 191-221.
- Bergstra, James, and Yoshua Bengio (2012). [Random Search for Hyper-Parameter Optimization](http://www.jmlr.org/papers/volume13/bergstra12a/bergstra12a.pdf). *Journal of Machine Learning Research*, 13(2).
- Doz, Catherine, Domenico Giannone, and Lucrezia Reichlin (2011). [A two-step estimator for large approximate dynamic factor models based on Kalman filtering](https://doi.org/10.1016/j.jeconom.2011.02.012). *Journal of Econometrics*, 164(1), 188–205.
- Franjic, Domenic, and Karsten Schweikert (2024). [Nowcasting Macroeconomic Variables with a Sparse Mixed Frequency Dynamic Factor Model](https://ssrn.com/abstract=4733872). SSRN 4733872.
- Giannone, Domenico, Lucrezia Reichlin, and David Small (2008). [Nowcasting: The Real-Time Informational Content of Macroeconomic Data](https://doi.org/10.1016/j.jmoneco.2008.05.010). *Journal of Monetary Economics*, 55(4), 665–76.
- Koopman, Siem Jan, and James Durbin (2000). [Fast Filtering and Smoothing for Multivariate State Space Models](https://doi.org/10.1111/1467-9892.00186). *Journal of Time Series Analysis*, 21(3), 281–96.
- Marcellino, Massimiliano, and Christian Schumacher (2010). [Factor MIDAS for nowcasting and forecasting with ragged-edge data: A model comparison for German GDP](https://doi.org/10.1111/j.1468-0084.2010.00591.x). *Oxford Bulletin of Economics and Statistics*, 72(4), 518–50.
- Mariano, Roberto S., and Yasutomo Murasawa (2003). [A New Coincident Index of Business Cycles Based on Monthly and Quarterly Series](https://doi.org/10.1002/jae.695). *Journal of Applied Econometrics*, 18(4), 427–43.
- Onatski, Alexei (2009). [Testing Hypotheses about the Number of Factors in Large Factor Models](https://doi.org/10.3982/ECTA6964). *Econometrica*, 77(5), 1447–79.
- Zou, Hui, Trevor Hastie, and Robert Tibshirani (2006). [Sparse Principal Component Analysis](https://doi.org/10.1198/106186006X113430). *Journal of Computational and Graphical Statistics*, 15(2), 265–86.

### Books
- Hyndman, Rob J., and George Athanasopoulos (2018). *Forecasting: Principles and Practice* (3rd ed.). [OTexts Melbourne](https://otexts.com/fpp3/).

### Software / Packages
- Zou, Hui, and Trevor Hastie (2020). [Elasticnet: Elastic-Net for Sparse Estimation and Sparse PCA (R package)](https://CRAN.R-project.org/package=elasticnet)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE,
                      comment = "#>",
                      fig.width = 8,
                      fig.height = 5,
                      dev = "png",
                      dpi = 150,
                      out.width = "100%"
)

## ----setup, include = FALSE---------------------------------------------------
library(TwoStepSDFM)

## ----simulation, eval = TRUE, message = FALSE---------------------------------

seed <- 02102025
set.seed(seed)
no_of_obs <- 300
no_of_vars <- 50
no_of_factors <- 3
trans_error_var_cov <- diag(1, no_of_factors)
loading_matrix <- matrix(rnorm(no_of_vars * no_of_factors), no_of_vars, no_of_factors)
meas_error_mean <- rep(0, no_of_vars)
meas_error_var_cov <- diag(1, no_of_vars)
trans_var_coeff <- cbind(diag(0.5, no_of_factors), -diag(0.25, no_of_factors))
factor_lag_order <- 2
delay <- c(2, 1, floor(rexp(no_of_vars - 2, 1)))
quarterfy <- TRUE
quarterly_variable_ratio <- 0.1
corr <- TRUE
beta_param <- 2
burn_in <- 999
starting_date <- "1970-01-01"
rescale <- FALSE
check_stationarity <- TRUE
stationarity_check_threshold <- 1e-10
factor_model <- simFM(no_of_obs = no_of_obs, no_of_vars = no_of_vars,
                      no_of_factors = no_of_factors, loading_matrix = loading_matrix,
                      meas_error_mean = meas_error_mean, meas_error_var_cov = meas_error_var_cov,
                      trans_error_var_cov = trans_error_var_cov, trans_var_coeff = trans_var_coeff,
                      factor_lag_order = factor_lag_order, delay = delay, quarterfy = quarterfy,
                      quarterly_variable_ratio  = quarterly_variable_ratio, corr = corr,
                      beta_param = beta_param, seed = seed, burn_in = burn_in, 
                      starting_date = starting_date, rescale = rescale, 
                      check_stationarity = check_stationarity, 
                      stationarity_check_threshold = stationarity_check_threshold)
spca_plots <- plot(factor_model, axis_text_size = 10, legend_title_text_size = 5)
spca_plots$`Data Plots`
spca_plots$`Factor Time Series Plots`
spca_plots$`Loading Matrix Heatmap`

## ----no_of_fact_test, eval = TRUE, message = FALSE----------------------------
mtly_data <- scale(factor_model$data[, which(factor_model$frequency == 12)])
no_of_fact_test <- noOfFactors(mtly_data, min_no_factors = 1, max_no_factors = 7,
                                  confidence_threshold = 0.2)
print(no_of_fact_test)

## ----cross_val, eval = TRUE, message = FALSE----------------------------------
no_of_mtly_vars <- sum(factor_model$frequency == 12)
all_data <- scale(factor_model$data)
cross_val_res <- crossVal(data = all_data, variable_of_interest = 1, fcast_horizon = 0,
                                 delay = factor_model$delay, frequency = factor_model$frequency, 
                                 no_of_factors = no_of_fact_test$test$no_of_factors, seed = seed, 
                                 min_ridge_penalty = 1e-6, max_ridge_penalty = 1e+5, cv_repetitions = 5,
                                 cv_size = 100, lasso_penalty_type = "selected", min_max_penalty = c(5, no_of_mtly_vars),
                                 verbose = FALSE)
print(cross_val_res)
cross_val_plot <- plot(cross_val_res)
cross_val_plot$`CV Results`

## ----estim_sparse_dfm, eval = TRUE, message = FALSE---------------------------
mtly_data <- scale(factor_model$data[, which(factor_model$frequency == 12)])
mtly_delay <- factor_model$delay[which(factor_model$frequency == 12)]
inferred_no_of_selected <- cross_val_res$CV$`Min. CV`[3:(3 + no_of_fact_test$test$no_of_factors - 1)]
inferred_ridge_penalty <- cross_val_res$CV$`Min. CV`[1]
sdfm_fit <- twoStepSDFM(data = mtly_data, delay = mtly_delay, selected = inferred_no_of_selected, 
                        no_of_fact_test$test$no_of_factors, ridge_penalty = inferred_ridge_penalty)
print(sdfm_fit)
sdfm_fit_plot <- plot(sdfm_fit)
sdfm_fit_plot$`Factor Time Series Plots`
sdfm_fit_plot$`Loading Matrix Heatmap`

## ----predict, eval = TRUE, message = FALSE------------------------------------
all_data <- scale(factor_model$data)
inferred_no_of_selected <- cross_val_res$CV$`Min. CV`[3:(3 + no_of_fact_test$test$no_of_factors - 1)]
inferred_ridge_penalty <- cross_val_res$CV$`Min. CV`[1]
nc_results <- nowcast(data = all_data, variables_of_interest = c(1, 2), max_fcast_horizon = 2, 
                      delay = factor_model$delay, ridge_penalty = inferred_ridge_penalty, selected = inferred_no_of_selected, frequency = factor_model$frequency, no_of_factors = no_of_fact_test$test$no_of_factors)
nc_plot <- plot(nc_results)
nc_plot$`Single Pred. Fcast Density Plots Series 1`
nc_plot$`Single Pred. Fcast Density Plots Series 2`

#' @useDynLib TwoStepSDFM, .registration=TRUE
#' @importFrom Rcpp sourceCpp
#' @import zoo
#' @import xts
#' @import lubridate
#' @import ggplot2
#' @import stats
#' @import utils
NULL

# SPDX-License-Identifier: GPL-3.0-or-later
#
#  Copyright (C) 2024-2026 Domenic Franjic
#
#  This file is part of TwoStepSDFM.
#
#  TwoStepSDFM is free software: you can redistribute
#  it and/or modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TwoStepSDFM is distributed in the hope that it
#  will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with TwoStepSDFM. If not, see <https://www.gnu.org/licenses/>.

#' @name simFM
#' @title Simulate Dynamic Factor Models.
#' @description
#' Simulate data from a linear Gaussian state-space model (latent factor model),
#' with measurement equation
#' \deqn{
#'   \bm{x}_t = \bm{\Lambda} \bm{f}_{t} + \bm{\xi}_t,\quad \bm{\xi}_t \sim \mathcal{N}(\bm{\mu}, \bm{\Sigma}_{\xi}),
#' }
#' and transition equation
#' \deqn{
#'   \bm{f}_t = \sum_{p=1}^P\bm{\Phi}_p \bm{f}_{t-p} + \bm{\epsilon}_t,\quad \bm{\epsilon}_t \sim \mathcal{N}(\bm{0}, \bm{\Sigma}_{f}).
#' }
#' for t = 1, \dots, T, as is used in, among others, 
#' \insertRef{franjic2024nowcasting}{TwoStepSDFM}.
#' 
#' @param no_of_obs Integer number of observations.
#' @param no_of_vars Integer number of Variables.
#' @param no_of_factors Integer number of factors.
#' @param loading_matrix Numeric (`no_of_vars` \eqn{\times}{x} `no_of_factors`) 
#' loading matrix.
#' @param meas_error_mean Numeric vector of the means of the measurement errors.
#' @param meas_error_var_cov Numeric (`no_of_vars` \eqn{\times}{x} 
#' `no_of_vars`) variance-covariance matrix of the measurement errors.
#' @param trans_error_var_cov Numeric (`no_of_factors` \eqn{\times}{x} 
#' `no_of_factors`) variance-covariance matrix of the transition errors.
#' @param trans_var_coeff Either a list of length `max_factor_lag_order` with 
#' each entry a numeric (`no_of_factors` \eqn{\times}{x} `no_of_factors`) VAR 
#' coefficient matrix or a matrix of dimensions (`no_of_factors` \eqn{\times}{x}
#' (`no_of_factors * max_factor_lag_order`)) holding the VAR coefficients of the
#' factor VAR process in each (`no_of_factors` \eqn{\times}{x} `no_of_factors`) 
#' block.
#' @param factor_lag_order Integer order of the VAR process in the transition 
#' equation.
#' @param delay Integer vector of delays imposed onto the end of the data 
#' (ragged edges).
#' @param quarterfy Logical, whether or not some of the data should be 
#' aggregated to quarterly representations.
#' @param quarterly_variable_ratio Ratio of variables ought to be quarterfied.
#' @param corr Logical, whether or not the measurement error should be randomly 
#' correlated inside the function using a random correlation matrix with 
#' off-diagonal elements governed by a beta-distribution.
#' @param beta_param Parameter of the beta-distribution governing the 
#' off-diagonal elements of the variance-covariance matrix of the measurement 
#' error.
#' @param seed 32-bit unsigned integer seed for all random processes inside the 
#' function.
#' @param burn_in Integer burn-in period of the simulated data ought to be 
#' discarded at the beginning of the sample.
#' @param rescale Logical, whether or not the variance of the measurement error 
#' should be rescaled by the common component to equalise the 
#' signal-to-noise ratio.
#' @param starting_date A date type object indicating the start of the dataset. 
#' If NULL (default), the function returns matrices with observations along the 
#' second dimension (i.e., time in columns). If specified, the function treats 
#' the data as a time series and returns a `zoo` object.
#' @param check_stationarity Logical, whether or not the stationarity properties
#'  of the factor VAR process should be checked.
#' @param stationarity_check_threshold Threshold of the stationarity check for 
#' when to deem an eigenvalue numerically negative.
#' @param parallel Logical, make use of Eigen internal parallel matrix 
#' operations.
#' 
#' @details
#' The `delay` vector indicates the number of observations at the end of the 
#' sample that will be set to `NA` for each variable. Here, `delay` refers to 
#' the number of months for monthly data and the number of quarters for 
#' quarterly data. For example, consider `delay <- c(1, 1)` and assume the 
#' variable with index `1` will be quarterfied. In that case, the variable with 
#' index `1` will be delayed by 1 quarter, i.e., it will be missing 3 
#' observations at the end of the panel. The variable with index `2` will be 
#' delayed by 1 month, i.e., it will be missing 1 observation at the end of the 
#' panel. This convention differs from the `delay` object of the `SimulData` 
#' class this function returns. There, `delay` represents the number of months 
#' since the most recent publication. For monthly variables, these values 
#' coincide, but for quarterly variables they are inherently different.
#' 
#' If `quarterfy = TRUE`, `floor(quarterly_variable_ratio * no_of_vars)` 
#' variables will be aggregated to a quarterly representation using the 
#' geometric mean according to 
#' \insertRef{Mariano2003new_coincident}{TwoStepSDFM}.
#' 
#' If `corr = TRUE`, the matrix `meas_error_var_cov` is internally replaced by a
#' random variance-covariance matrix: 
#' \eqn{\tilde{\bm{\Sigma}}:=\bm{S}\bm{R}\bm{S}}{new_meas_var_cov = diag_variance_sqrt * corr_matrix * diag_variance_sqrt},
#' where \eqn{\bm{S}}{diag_variance_sqrt} is a diagonal matrix with entries 
#' equal to `sqrt(diag(meas_error_var_cov))` and \eqn{\bm{R}}{corr_matrix} is a
#' random correlation matrix. \eqn{\bm{R}}{corr_matrix} is drawn according to 
#' \insertRef{lewandowski2009generating}{TwoStepSDFM} (see also
#' \url{https://stats.stackexchange.com/questions/2746/how-to-efficiently-generate-random-positive-semidefinite-correlation-matrices}).
#' The parameter `beta_param` governs the degree of cross-correlation of the 
#' off-diagonal elements. For more information see the literature cited above.
#' 
#' The random draws of the fundamental error terms are drawn within the `C++` 
#' backend. Therefore, `seed` must be provided and `set.seed()` will not 
#' guarantee reproduceability.
#' 
#' @return Returns a `SimulData` containing the following elements:
#' \describe{
#' \item{data}{If `starting_date` is provided, a `zoo` object, else, 
#' a (`no_of_vars` \eqn{\times}{x} `no_of_obs`) numeric matrix holding the 
#' simulated data.}
#' \item{factors}{If `starting_date` is provided, a `zoo` object, else a
#'  (`no_of_factors` \eqn{\times}{x} `no_of_obs`) numeric matrix holding  the 
#'  simulated latent factors.}
#' \item{trans_var_coeff}{Numeric (`no_of_factors` \eqn{\times}{x} 
#' (`no_of_factors` * `factor_lag_order`)) factor VAR coefficient matrix.}
#' \item{loading_matrix}{Numeric factor loading matrix.}
#' \item{meas_error}{If `starting_date` is provided, a `zoo` object, else 
#' a (`no_of_vars` \eqn{\times}{x} `no_of_obs`) numeric matrix holding the 
#' fundamental measurement errors.}
#' \item{meas_error_var_cov}{Numeric measurement error 
#' variance-covariance matrix.}
#' \item{trans_error_var_cov}{Numeric transition error 
#' variance-covariance matrix.}
#' \item{frequency}{Integer vector of variable frequencies.}
#' \item{delay}{Integer vector of variable delays, measured as the number of 
#' months since the latest available observation.}
#' }
#'
#' @author
#' Domenic Franjic
#' 
#' @references
#' \insertRef{Mariano2003new_coincident}{TwoStepSDFM}
#' 
#' \insertRef{lewandowski2009generating}{TwoStepSDFM}
#' 
#' \insertRef{franjic2024nowcasting}{TwoStepSDFM}
#'
#' @examples
#' seed <- 02102025
#' set.seed(seed)
#' no_of_obs <- 100
#' no_of_vars <- 50
#' no_of_factors <- 3
#' trans_error_var_cov <- diag(1, no_of_factors)
#' loading_matrix <- matrix(round(rnorm(no_of_vars * no_of_factors)), no_of_vars, no_of_factors)
#' meas_error_mean <- rep(0, no_of_vars)
#' meas_error_var_cov <- diag(1, no_of_vars)
#' trans_var_coeff <- cbind(diag(0.5, no_of_factors), -diag(0.25, no_of_factors))
#' factor_lag_order <- 2
#' delay <- c(floor(rexp(no_of_vars, 1)))
#' quarterfy <- FALSE
#' quarterly_variable_ratio  <- 0
#' corr <- TRUE
#' beta_param <- 2
#' burn_in <- 999
#' starting_date <- "1970-01-01"
#' rescale <- TRUE
#' check_stationarity <- TRUE
#' stationarity_check_threshold <- 1e-10
#' factor_model <- simFM(no_of_obs = no_of_obs, no_of_vars = no_of_vars,
#'                       no_of_factors = no_of_factors, loading_matrix = loading_matrix,
#'                       meas_error_mean = meas_error_mean, 
#'                       meas_error_var_cov = meas_error_var_cov,
#'                       trans_error_var_cov = trans_error_var_cov, 
#'                       trans_var_coeff = trans_var_coeff,
#'                       factor_lag_order = factor_lag_order, delay = delay, 
#'                       quarterfy = quarterfy, 
#'                       quarterly_variable_ratio  = quarterly_variable_ratio, corr = corr,
#'                       beta_param = beta_param, seed = seed, burn_in = burn_in, 
#'                       starting_date = starting_date, rescale = rescale, 
#'                       check_stationarity = check_stationarity,
#'                       stationarity_check_threshold = stationarity_check_threshold)
#' print(factor_model)
#' spca_plots <- plot(factor_model)
#' spca_plots$`Factor Time Series Plots`
#' spca_plots$`Loading Matrix Heatmap`
#' spca_plots$`Meas. Error Var.-Cov. Matrix Heatmap`
#' spca_plots$`Meas. Error Var.-Cov. Eigenvalue Plot`
#' spca_plots$`Data Var.-Cov. Matrix Heatmap`
#' spca_plots$`Data Var.-Cov. Eigenvalue Plot`
#'
#' @export
simFM <- function(no_of_obs, no_of_vars, no_of_factors, loading_matrix, 
                  meas_error_mean, meas_error_var_cov, trans_error_var_cov, trans_var_coeff, 
                  factor_lag_order, delay = NULL, quarterfy = FALSE, quarterly_variable_ratio = 0, 
                  corr = FALSE, beta_param = Inf, seed = 20022024, burn_in = 1000, 
                  rescale = TRUE, starting_date = NULL,
                  check_stationarity = FALSE, stationarity_check_threshold = 1e-5,
                  parallel = FALSE) {
  func_call <- match.call()
  
  # Mishandling of dimensionalities
  no_of_obs <- checkPositiveSignedInteger(no_of_obs, "no_of_obs")
  if(no_of_obs == 0){
    stop("no_of_obs must be strictly positive.")
  }
  
  no_of_vars <- checkPositiveSignedInteger(no_of_vars, "no_of_vars")
  if(no_of_vars == 0){
    stop("no_of_vars must be strictly positive.")
  }
  
  no_of_factors <- checkPositiveSignedInteger(no_of_factors, "no_of_factors")
  if(no_of_factors == 0){
    stop("no_of_factors must be strictly positive.")
  }
  
  if(no_of_factors > no_of_vars){
    stop(paste0("no_of_factors must be smaller than no_of_vars."))
  }
  if(no_of_obs <= no_of_vars){
    warning(paste0("no_of_vars is bigger than no_of_obs."))
  }
  
  # Mishandling of the loading matrix
  LambdaR <- checkParameterMatrix(loading_matrix, "loading_matrix", no_of_vars, no_of_factors)
  
  # Mishandling of the measurement error mean
  muR <- checkParameterMatrix(meas_error_mean, "meas_error_mean", no_of_vars, 1)
  
  
  # Mishandling of the measurement error covariance
  SigmaR <- checkParameterMatrix(meas_error_var_cov, "meas_error_var_cov", no_of_vars, no_of_vars)
  if(!isSymmetric(SigmaR)) {
    stop(paste0("meas_error_var_cov must be symmetric."))
  }
  if(any(diag(SigmaR)< 0)){
    stop(paste0("meas_error_var_cov must not have negative values along the diagonal."))
  }
  eig <- eigen(SigmaR, only.values = TRUE)$values
  if(any(eig < -1e-8)){
    warning("meas_error_var_cov may not be positive semi-definite.")
  }
  
  # Mishandling of the transition error variance
  SR <- checkParameterMatrix(trans_error_var_cov, "trans_error_var_cov", no_of_factors, no_of_factors)
  if(!isSymmetric(SR)) {
    stop(paste0("trans_error_var_cov must be symmetric."))
  }
  if(any(diag(SR)< 0)){
    stop(paste0("trans_error_var_cov must not have negative values along the diagonal."))
  }
  eig <- eigen(SR, only.values = TRUE)$values
  if(any(eig < -1e-8)){
    warning("trans_error_var_cov may not be positive semi-definite.")
  }
  
  # Mishandling of the factor VAR lag order
  factor_lag_order <- checkPositiveSignedInteger(factor_lag_order, "factor_lag_order")
  if(factor_lag_order == 0){
    stop("factor_lag_order must be strictly positive.")
  }
  
  # Misshandling of the VAR coefficient matrix
  if (is.list(trans_var_coeff) && !is.data.frame(trans_var_coeff)) { 
    # If trans_var_coeff is provided as list: Check whether each element in 
    #   trans_var_coeff has the correct dimensions and whether there is the 
    #   correct number of matrices provided.
    
    ind <- c()
    for (i in 1:length(trans_var_coeff)) {
      s <- sum(dim(trans_var_coeff[[i]]) == c(no_of_factors, no_of_factors))
      if (s != 2) {
        ind[i] <- i
      }
    }
    if (!is.null(ind)) {
      stop("The VAR coefficient matrices in trans_var_coeff must be of dimensions (no_of_factors x no_of_factors) = (", no_of_factors, "x", no_of_factors, "). The matrices with index ", paste(ind, collapse = ", "), " are of different dimensions.")
    }
    if (length(trans_var_coeff) != factor_lag_order) {
      stop("The number of VAR coefficient matrices in trans_var_coeff must equal factor_lag_order = ", factor_lag_order, " but is ", length(trans_var_coeff), ".")
    }
    
    # Store the list elements as matrix
    PhiR <- matrix(0, no_of_factors, no_of_factors * factor_lag_order)
    for (o in 1:factor_lag_order) {
      PhiR[1:no_of_factors, ((o - 1) * no_of_factors + 1):((o - 1) * no_of_factors + no_of_factors)] <- trans_var_coeff[[o]]
    }
    PhiR <- checkParameterMatrix(PhiR, "trans_var_coeff", no_of_factors, no_of_factors * factor_lag_order)
    
  } else {
    # If trans_var_coeff is provided as matrix: Do regular dimensionality checks
    PhiR <- checkParameterMatrix(trans_var_coeff, "trans_var_coeff", no_of_factors, no_of_factors * factor_lag_order)
  }
  
  # Mishandling of dealy
  if(is.null(delay)){
    delay <- matrix(rep(0, no_of_vars), ncol = 1)
  }else{
    delay <- checkPositiveSignedParameterVector(delay, "delay", no_of_vars)
  }
  # Mishandling of quarterfy
  quarterfy <- checkBoolean(quarterfy, "quarterfy")
  
  # Mishandling the ratio of variables ought to be quarterfied
  quarterly_variable_ratio <- checkPositiveDouble(quarterly_variable_ratio, "quarterly_variable_ratio")
  if(quarterly_variable_ratio == 0 && quarterfy){
    warning("quarterfy is set to TRUE but quarterly_variable_ratio = 0. No variables will be aggregated to quarterly data.")
  }
  if(quarterly_variable_ratio != 0 && !quarterfy){
    warning("quarterfy is set to FALSE but quarterly_variable_ratio != 0. No variables will be aggregated to quarterly data.")
  }
  if(quarterly_variable_ratio > 1){
    warning("quarterly_variable_ratio is bigger than 1. It will be set to 1 before further use. All variables will be aggregated to quarterly data.")
    quarterly_variable_ratio <- 1
  }
  
  # Mishandling of corr
  corr <- checkBoolean(corr, "corr")
  
  # Check for mishandling of the beta parameter governing the distribution of the off-diagonal elements of the meas. var.cov.
  if(is.infinite(beta_param)){
    beta_param <- .Machine$double.xmax
  }
  beta_param <- checkPositiveDouble(beta_param, "beta_param")
  if((beta_param == .Machine$double.xmax || is.infinite(beta_param)) && corr){
    warning("corr is set to TRUE but beta_param = Inf. Measurement errors will not be cross-correlated.")
    corr <- FALSE
  }
  if((beta_param < .Machine$double.xmax || !is.infinite(beta_param)) && !corr){
    warning("corr is set to FALSE but beta_param != Inf. Measurement errors will not be cross-correlated.")
  }
  if(beta_param == 0){
    warning("beta_param cannot be exactly 0. It will be jittered before further use.")
    beta_param <- 1e-15
  }
  
  # Mishandling of seed
  seed <- checkPositiveSignedInteger(seed, "seed", 33)
  
  # Mishandling of burn-in
  burn_in <- checkPositiveSignedInteger(burn_in, "burn_in")
  
  # Mishandling of rescale
  rescale <- checkBoolean(rescale, "rescale")
  
  # Misshandling of the starting date
  if(!is.null(starting_date)){
    if (length(starting_date) != 1) {
      stop("starting_date must be NULL or a single string date object or single string character object convertible to a date object.")
    }
    if(!is.Date(starting_date)){
      starting_date <- try(as.Date(starting_date), silent = TRUE)
      if (inherits(starting_date, "try-error")) {
        stop(paste0("The (pseudo) starting date for the data set must be a date object or convertible to a date object."))
      }
    }
    if(quarterfy && (month(starting_date) %% 3 != 1)){
      stop(paste0("If any observations will be quarterfied, `starting_month` must refer to a date that lies in the first month of its corresponding quarter."))
    }
  }
  
  # Mishandling of check_stationarity
  check_stationarity <- checkBoolean(check_stationarity, "check_stationarity")
  
  # Check for mishandling of stationarity_check_threshold
  stationarity_check_threshold <- checkPositiveDouble(stationarity_check_threshold, "stationarity_check_threshold")
  if(stationarity_check_threshold == 0){
    warning("stationarity_check_threshold should not be exactly set to zero. It will be jittered before further use.")
    stationarity_check_threshold <- 1e-15
  }
  
  # Mishandling of parallel
  parallel <- checkBoolean(parallel, "parallel")
  
  # Check whether the process will result in a stationary process
  if (check_stationarity) {
    
    if(factor_lag_order > 1){
      Comp <- matrix(0, no_of_factors * factor_lag_order, no_of_factors * factor_lag_order)
      Comp[1:no_of_factors, 1:(factor_lag_order * no_of_factors)] <- PhiR
      Comp[(no_of_factors + 1):(factor_lag_order * no_of_factors), 1:(no_of_factors * (factor_lag_order - 1))] <- diag(1, no_of_factors * (factor_lag_order - 1))
    }else{
      Comp <- PhiR
    }
    
    # Compute eigenvalues
    e <- eigen(Comp)$values
    test <- any(abs(abs(e) - 1) < stationarity_check_threshold)
    if (test) {
      warning("At least one eigenvalue very close to one detected. Process might have random walk properties.\n")
    }
    test2 <- all(abs(e) < 1 + stationarity_check_threshold)
    if (!test2) {
      warning("At least one eigenvalue lies outside the complex unit-circle. Process is not stationary.")
    }
    
  }
  
  # Generate the data
  FM <- runStaticFM(T = no_of_obs, N = no_of_vars, S = SR, Lambda = LambdaR, mu_e = muR, 
                    Sigma_e = SigmaR, A = PhiR, order = factor_lag_order, quarterfy = quarterfy, 
                    corr = corr, beta_param = beta_param, m = quarterly_variable_ratio, 
                    seed = seed, R = no_of_factors, burn_in = burn_in, rescale = rescale,
                    parallel = parallel
                    )
  
  # Clean up results
  names(FM) <- c("factors", "trans_var_coeff", "loading_matrix", "meas_error_var_cov",
                 "trans_error_var_cov", "meas_error", "data", "frequency")
  
  # Imposing the delay
  if(any(delay != 0)){
    for( n in 1:length(delay)){
      if(FM$frequency[n] == 12){
        if(delay[n] > 0){
          FM$data[n, (no_of_obs - delay[n] + 1):no_of_obs] <- NaN
          FM$meas_error[n, (no_of_obs - delay[n] + 1):no_of_obs] <- NaN
        }
      }else if(FM$frequency[n] == 4){
        # Check which month the latest observation corresponds to
        overhang <- 0
        if(FM$data[n, no_of_obs] != FM$data[n, no_of_obs - 1] &&
           FM$data[n, no_of_obs - 1] == FM$data[n, no_of_obs - 2]){
          overhang <- 1
        }
        if(FM$data[n, no_of_obs] != FM$data[n, no_of_obs - 1] &&
           FM$data[n, no_of_obs - 1] != FM$data[n, no_of_obs - 2]){
          overhang <- 2
        }
        delay[n] <- delay[n] * 3 + overhang
        if(delay[n] > 0){
          FM$data[n, (no_of_obs - delay[n] + 1):no_of_obs] <- NaN
          FM$meas_error[n, (no_of_obs - delay[n] + 1):no_of_obs] <- NaN
        }
      }
    }
  }
  FM$delay <- delay
  
  # Reorder the results 
  FM <- FM[c("data", "factors", "trans_var_coeff", "loading_matrix", "meas_error",
             "meas_error_var_cov", "trans_error_var_cov", "frequency", "delay")]
  
  # Create pseudo names
  rownames(FM$data) <- paste0("Series ", 1:no_of_vars)
  rownames(FM$factors) <- paste0("Factor ", 1:no_of_factors)
  rownames(FM$meas_error) <- paste0("Series ", 1:no_of_vars)
  
  if(is.Date(starting_date)){ # Turn all data to time series if stat_date is provided
    start_vector <-  c(year(starting_date), month(starting_date))
    FM$data <- as.zoo(ts(t(FM$data), start = start_vector, frequency = 12))
    FM$factors <- as.zoo(ts(t(FM$factors), start = start_vector, frequency = 12))
    FM$meas_error <- as.zoo(ts(t(FM$meas_error), start = start_vector, frequency = 12))
  }
  
  FM$call <- func_call
  class(FM) <- "SimulData"
  return(FM)
  
}

#' @name print.SimulData
#' @title Generic printing function for SimulData S3 objects
#' @description
#' Print a compact summary of an `SimulData` object.
#'
#' @param x `SimulData` object.
#' @param ... Additional parameters for the plotting functions.
#'
#' @return
#' No return value; Prints a summary to the console.
#'
#' @author
#' Domenic Franjic
#' 
#' @export
print.SimulData <- function(x, ...) {
  simulated_time_series <- is.zoo(x$data)
  no_of_factors <- ifelse(simulated_time_series, dim(x$factors)[2], dim(x$factors)[1])
  no_of_obs <- ifelse(simulated_time_series, dim(x$data)[1], dim(x$data)[2])
  cat("Simulated Dynamic Factor Model\n")
  cat("=========================================================================\n")
  cat("No. of Observations        :", ifelse(simulated_time_series, dim(x$data)[1], dim(x$data)[2]), "\n")
  cat("No. of Variables           :", ifelse(simulated_time_series, dim(x$data)[2], dim(x$data)[1]), "\n")
  cat("No. of Factors             :", no_of_factors, "\n")
  cat("Factor Lag Order           :", dim(x$trans_var_coeff)[2] / no_of_factors, "\n")
  max_print <- min(5, no_of_obs)
  if(any(x$frequency == 4)){
    cat("No. of Quarterly Variables :", sum(x$frequency == 4), "\n")
  }
  cat("=========================================================================\n")
  cat("Head of the factors :\n")
  if(simulated_time_series){
    print(head(x$factors, max_print))
  }else{
    print(x$factors[, 1:max_print])
  }
  cat("Tail of the factors :\n")
  if(simulated_time_series){
    print(tail(x$factors, max_print))
  }else{
    print(x$factors[, (dim(x$factors)[2] - (max_print - 1)):(dim(x$factors)[2])])
  }
  cat("Head of the observations :\n")
  if(simulated_time_series){
    print(head(x$data, max_print))
  }else{
    print(x$data[, 1:max_print])
  }
  cat("Tail of the observations :\n")
  if(simulated_time_series){
    print(tail(x$data, max_print))
  }else{
    print(x$data[, (dim(x$data)[2] - (max_print - 1)):(dim(x$data)[2])])
  }
  cat("=========================================================================\n")
  invisible(x)
}

#' @name plot.SimulData
#' @title Generic plotting function for SimulData S3 objects
#' @description
#' Create diagnostic plots for an `SimulData` object.
#'
#' @param x `SimulData` object.
#' @param axis_text_size Numeric size of x- and y-axis labels. Prased to ggplot2
#' `theme(..., text = element_text(size = axis_text_size))`.
#' @param legend_title_text_size Numeric size of x- and y-axis labels. Prased to
#' ggplot2 
#' `theme(..., legend.title = element_text(size = legend_title_text_size))`.
#' @param ... Additional parameters for the plotting functions.
#'
#' @return
#' A named list of plot objects:
#' \describe{
#'   \item{`Factor Time Series Plots`}{`patchwork`/`ggplot` object showing the 
#'   simulated factors over time.}
#'   \item{`Loading Matrix Heatmap`}{`ggplot` object showing a heatmap of the 
#'   simulated factor loadings. Zeros are highlighted in black.}
#'   \item{`Meas. Error Var.-Cov. Matrix Heatmap`}{`ggplot` object showing a 
#'   heatmap of the measurement error  variance-covariance matrix.}
#'   \item{`Meas. Error Var.-Cov. Eigenvalue Plot`}{`ggplot`object showing a bar 
#'   plot of the eigenvalues of the measurement error variance-covariance 
#'   matrix.}
#'   \item{`Data Var.-Cov. Matrix Heatmap`}{`ggplot` object showing a heatmap of 
#'   the data variance-covariance matrix.}
#'   \item{`Data Var.-Cov. Eigenvalue Plot`}{`ggplot` object showing a bar plot 
#'   of the eigenvalues of the data variance-covariance matrix.}
#'   }
#' @export
plot.SimulData <- function(x, 
                           axis_text_size = 20, 
                           legend_title_text_size = 20,
                           ...) 
{
  out_list <- list()
  if (is.zoo(x$data)) {
    series_names <- colnames(x$data)
    no_of_factors <- dim(x$factors)[2]
    no_of_obs <- dim(x$factors)[1]
    time_vector <- as.Date(time(x$data))
    factors <- x$factors
    data <- x$data
  }
  else {
    series_names <- rownames(x$data)
    no_of_factors <- dim(x$factors)[1]
    time_vector <- 1:dim(x$factors)[2]
    no_of_obs <- dim(x$factors)[2]
    factors <- t(x$factors)
    data <- t(x$data)
  }
  
  mtly_data <- x$data[, which(x$frequency == 12)]
  is_ther_qtly_data <- any(x$frequency == 4)
  if (is_ther_qtly_data) {
    qtly_data <- x$data[, which(x$frequency == 4)]
  }
  mtly_data_df <- fortify.zoo(mtly_data, names = "Date")
  mtly_data_long <- stack(mtly_data_df[, -1])
  names(mtly_data_long) <- c("Value", "Series")
  mtly_data_long$Series <- factor(mtly_data_long$Series)
  mtly_data_long$Date <- rep(mtly_data_df$Date, times = dim(mtly_data_df)[2] - 1)
  mtly_data_plot <- ggplot(mtly_data_long, aes(x = Date, y = Value, colour = Series)) + 
    geom_line() + 
    labs(title = ifelse(is_ther_qtly_data, "Monthly Data", ""), y = "") + 
    theme_minimal() + 
    theme(axis.text.x = element_text(angle = 0, vjust = 1, hjust = 1), 
          text = element_text(size = axis_text_size), 
          legend.position = "none")
  if (is_ther_qtly_data) {
    qtly_data_df <- fortify.zoo(qtly_data, names = "Date")
    qtly_data_long <- stack(qtly_data_df[, -1])
    names(qtly_data_long) <- c("Value", "Series")
    qtly_data_long$Series <- factor(qtly_data_long$Series)
    qtly_data_long$Date <- rep(qtly_data_df$Date, times = dim(qtly_data_df)[2] - 1)
    qtly_data_plot <- ggplot(qtly_data_long, aes(x = Date, y = Value, colour = Series)) + 
      geom_line() + labs(title = "Quarterly Data", y = "") + 
      theme_minimal() + 
      theme(axis.text.x = element_text(angle = 0, vjust = 1, hjust = 1), 
            text = element_text(size = axis_text_size), 
            legend.position = "none")
    data_plots <- list()
    data_plots$qtly_data_plot <- qtly_data_plot
    data_plots$mtly_data_plot <- mtly_data_plot
    out_list$`Data Plots` <- patchwork::wrap_plots(data_plots, ncol = 1)
  }
  else {
    out_list$`Data Plots` <- mtly_data_plot
  }
  
  out_list$`Factor Time Series Plots` <- plotFactorEstimates(factors, matrix(0, no_of_factors, no_of_factors * no_of_obs), 
                                                             no_of_factors, axis_text_size)
  
  out_list$`Loading Matrix Heatmap` <- plotLoadingHeatMap(x$loading_matrix, series_names, 
                                                          no_of_factors, axis_text_size, 
                                                          legend_title_text_size)
  
  if (is.zoo(x$data)) {
    residuals <- coredata(na.omit(x$meas_error))
  }
  else {
    residuals <- na.omit(t(x$meas_error))
  }
  measurement_error_var_cov_df <- as.data.frame(t(residuals) %*% residuals * 1/(dim(residuals)[1] - 1))
  out_list$`Meas. Error Var.-Cov. Matrix Heatmap` <- plotMeasVarCovHeatmap(measurement_error_var_cov_df, 
                                                                           series_names, 
                                                                           axis_text_size, 
                                                                           legend_title_text_size)
  
  out_list$`Meas. Error Var.-Cov. Eigenvalue Plot` <- plotMeasVarCovEigenvalues(eigen(measurement_error_var_cov_df)$values, 
                                                                                no_of_factors, axis_text_size, legend_title_text_size)
  
  if (is.zoo(x$data)) {
    data <- coredata(na.omit(x$meas_error))
  }
  else {
    data <- na.omit(t(x$meas_error))
  }
  data_var_cov_df <- as.data.frame(t(data) %*% data * 1/(dim(data)[1] - 1))
  out_list$`Data Var.-Cov. Matrix Heatmap` <- plotMeasVarCovHeatmap(data_var_cov_df, 
                                                                    series_names, 
                                                                    axis_text_size, 
                                                                    legend_title_text_size)
  
  out_list$`Data Var.-Cov. Eigenvalue Plot` <- plotMeasVarCovEigenvalues(eigen(data_var_cov_df)$values, 
                                                                         no_of_factors, axis_text_size, legend_title_text_size)
  return(out_list)
}

test_that("", {
  
  seed <- 01102025
  no_of_obs <- 102
  no_of_vars <- 50
  no_of_factors <- 3
  trans_error_var_cov <- diag(1, no_of_factors)
  loading_matrix <- matrix(round(rnorm(no_of_vars * no_of_factors)), no_of_vars, no_of_factors)
  meas_error_mean <- rep(0, no_of_vars)
  meas_error_var_cov <- diag(1, no_of_vars)
  trans_var_coeff <- cbind(diag(0.5, no_of_factors), -diag(0.25, no_of_factors))
  factor_lag_order <- 2
  simul_delay <- c(floor(rexp(no_of_vars, 1)))
  quarterfy <- TRUE
  quarterly_variable_ratio  <- 0.10
  corr <- TRUE
  beta_param <- 2
  set.seed(seed)
  burn_in <- 999
  starting_date <- "1970-01-01"
  rescale <- TRUE
  check_stationarity <- TRUE
  stationarity_check_threshold <- 1e-10
  
  # Generate pseudo-date data
  expect_silent(data_date <- simFM(no_of_obs = no_of_obs, no_of_vars = no_of_vars,
                                   no_of_factors = no_of_factors, loading_matrix = loading_matrix,
                                   meas_error_mean = meas_error_mean, meas_error_var_cov = meas_error_var_cov,
                                   trans_error_var_cov = trans_error_var_cov, trans_var_coeff = trans_var_coeff,
                                   factor_lag_order = factor_lag_order, delay = simul_delay, quarterfy = quarterfy,
                                   quarterly_variable_ratio  = quarterly_variable_ratio, corr = corr,
                                   beta_param = beta_param, seed = seed, burn_in = burn_in, starting_date = starting_date,
                                   rescale = rescale, check_stationarity = check_stationarity,
                                   stationarity_check_threshold = stationarity_check_threshold))
  
  # Generate numeric data
  expect_silent(data_mat <- simFM(no_of_obs = no_of_obs, no_of_vars = no_of_vars,
                                  no_of_factors = no_of_factors, loading_matrix = loading_matrix,
                                  meas_error_mean = meas_error_mean, meas_error_var_cov = meas_error_var_cov,
                                  trans_error_var_cov = trans_error_var_cov, trans_var_coeff = trans_var_coeff,
                                  factor_lag_order = factor_lag_order, delay = simul_delay, quarterfy = quarterfy,
                                  quarterly_variable_ratio  = quarterly_variable_ratio, corr = corr,
                                  beta_param = beta_param, seed = seed, burn_in = burn_in, starting_date = NULL,
                                  rescale = rescale, check_stationarity = check_stationarity,
                                  stationarity_check_threshold = stationarity_check_threshold))
  
  # Basic checks
  expect_equal(t(coredata(data_date$data)), data_mat$data)
  expect_equal(t(coredata(data_date$factors)), data_mat$factors)
  expect_equal(data_date$trans_var_coeff, data_mat$trans_var_coeff)
  expect_equal(data_date$loading_matrix, data_mat$loading_matrix)
  expect_equal(t(coredata(data_date$meas_error)), data_mat$meas_error)
  expect_equal(data_date$meas_error_var_cov, data_mat$meas_error_var_cov)
  expect_equal(data_date$trans_error_var_cov, data_mat$trans_error_var_cov)
  expect_equal(data_date$frequency, data_mat$frequency)
  expect_equal(data_date$delay, data_mat$delay)
  
})

#' @useDynLib TwoStepSDFM, .registration=TRUE
#' @import utils
NULL

utils::globalVariables(c(
  "Value", "Lower 95%-CI", "Upper 95%-CI",
  "Factor", "Variable", "Loading", "(Co-)Variable", "(Co-)Variance",
  "Ridge Penalty", "CV Errors", "Avg. Lasso Penalty",
  "Sparsity Ratio", "# of LARS Steps", "Series",
  "Variance Explained", "Component", "radial_position", "pct_label",
  "Eigen Value", "cut_off"
))

#' @useDynLib TwoStepSDFM, .registration=TRUE
#' @importFrom Rcpp sourceCpp
#' @importFrom Rdpack reprompt
#' @import zoo
#' @import xts
#' @import lubridate
#' @import ggplot2
#' @import stats
#' @import utils
NULL

# SPDX-License-Identifier: GPL-3.0-or-later
#
#  Copyright (C) 2024-2026 Domenic Franjic
#
#  This file is part of TwoStepSDFM.
#
#  TwoStepSDFM is free software: you can redistribute
#  it and/or modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TwoStepSDFM is distributed in the hope that it
#  will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with TwoStepSDFM. If not, see <https://www.gnu.org/licenses/>.

#' @name twoStepSDFM
#' @title Two Step Sparse Dynamic Factor Model Estimator.
#' @description
#' Estimate a sparse dynamic factor model with measurement equation
#' \deqn{
#'   \bm{x}_t = \bm{\Lambda} \bm{f}_{t} + \bm{\xi}_t,\quad \bm{\xi}_t \sim \mathcal{N}(\bm{0}, \bm{\Sigma}_{\xi}),
#' }
#' and transition equation
#' \deqn{
#'   \bm{f}_t = \sum_{p=0}^P\bm{\Phi}_p \bm{f}_{t-p} + \bm{\epsilon}_t,\quad \bm{\epsilon}_t \sim \mathcal{N}(\bm{0}, \bm{\Sigma}_{f}).
#' }
#' using sparse principal components  analysis and the Kalman Filter and 
#' Smoother according to \insertRef{franjic2024nowcasting}{TwoStepSDFM}.
#' 
#' @param data Numeric (no_of_vars \eqn{\times}{x} no_of_obs) matrix of data or 
#' zoo/xts object sampled at the same frequency.
#' @param delay Integer vector of variable delays, measured as the number of 
#' months since the latest available observation.
#' @param selected Integer vector of the number of selected variables for each 
#' factor.
#' @param no_of_factors Integer number of factors.
#' @param max_factor_lag_order Integer maximum order of the VAR process in the 
#' transition equation.
#' @param lag_estim_criterion Information criterion used for the estimation of 
#' the factor VAR order (`"BIC"` (default), `"AIC"`, `"HIC"`).
#' @param decorr_errors Logical, whether or not the errors should be 
#' decorrelated.
#' @param ridge_penalty Ridge penalty.
#' @param lasso_penalty Numeric vector, lasso penalties for each factor (set to 
#' NULL to disable as stopping criterion).
#' @param max_iterations Integer maximum number of iterations.
#' @param max_no_steps Integer number of LARS steps (set to NULL to disable as 
#' stopping criterion).
#' @param weights Numeric vector, weights for each variable weighing the 
#' \eqn{\ell_1}{`l_1`} size constraint.
#' @param comp_null Numeric computational zero.
#' @param spca_conv_crit Conversion threshold for the SPCA algorithm.
#' @param parallel Logical, whether or not to use Eigen's internal parallel 
#' matrix operations.
#' @param fcast_horizon Integer number of additional Filter predictions into the 
#' future.
#' @param jitter Numerical jitter for stability of internal solver algorithms. 
#' The jitter is added to the diagonal entries of the variance covariance matrix 
#' of the measurement errors.
#' @param svd_method Either "fast" or "precise". Option "fast" uses Eigen's 
#' BDCSVD divide and conquer method for the computation of the singular values. 
#' Option "precise" (default) implements the slower, but numerically more stable 
#' JacobiSVD method \insertCite{eigenweb}{TwoStepSDFM}.
#' 
#' @details
#' The function performs a two-step estimation procedure for sparse dynamic 
#' factor models as described in \insertRef{franjic2024nowcasting}{TwoStepSDFM}. 
#' In the first step, the factor loading matrix is estimated using SPCA 
#' \insertCite{zou2006sparse}{TwoStepSDFM}. This will shrink some of the 
#' loadings towards or exactly to zero. In the second step the latent factors
#' are estimated using the univariate representation of the Kalman Filter and
#' Smoother \insertCite{koopman2000fast}{TwoStepSDFM}.
#' 
#' The function takes three stopping criteria for the SPCA algorithm:
#' `selected`, `lasso_penalty`, and `max_no_steps`. The argument `weights`
#' allows specifying weights for the \eqn{\ell_1}{l1} constraint.  `svd_method` 
#' controls the decomposition method for internal SVDs. For a detailed 
#' description of these arguments and the SPCA step, see 
#' \code{\link{sparsePCA}}.
#' 
#' With respect to the univariate representation of the Kalman filter and 
#' smoother, `decorr_errors` indicates whether the data should be decorrelated 
#' internally prior to filtering and smoothing. `jitter` is added to the 
#' diagonal elements of the measurement variance–covariance matrix. For more 
#' details, see \code{\link{kalmanFilterSmoother}}.
#' 
#' For more information on the two-step estimation procedure see 
#' \insertRef{franjic2024nowcasting}{TwoStepSDFM}.
#' 
#' @return 
#' An object of class SDFMFit with main components:
#' #' \describe{
#'   \item{data}{Original data object.}
#'   \item{loading_matrix_estim}{Numeric matrix of estimated factor loadings.}
#'   \item{smoothed_factors}{Object containing the SPCA factor estimates. The 
#'   object inherits its class from data: If data is provided as `zoo`, 
#'   `factor_estim` will be a `zoo` object. If data is provided as matrix, 
#'   `factor_estim` will be a (`no_of_factors`\eqn{\times}{x}`no_of_obs` 
#'   matrix.}
#'   \item{smoothed_state_variance}{(`no_of_factors`\eqn{\times}{x}(
#'   `no_of_factors` * `no_of_obs`)) matrix, where each (`no_of_factors`
#'   \eqn{\times}{x}`no_of_factors`) block represents the smoother uncertainty 
#'   at time point\eqn{t}{t}.}
#'   \item{factor_var_lag_order}{Integer order of the VAR process in the state 
#'   equation.}
#'   \item{error_var_cov_cholesky_factor}{Numeric lower-triangular Cholesky 
#'   factor of the estimated measurement error variance–covariance matrix.}
#'   \item{llt_success_code}{Integer indicating the status of the Cholesky 
#'   factorization: `0` = LLT succeeded, `-1` = LLT failed but LDLT succeeded, 
#'   `-2` = both failed and errors are treated as uncorrelated.}
#' }
#' 
#' @author
#' Domenic Franjic
#' 
#' @references
#' \insertRef{koopman2000fast}{TwoStepSDFM}
#' 
#' \insertRef{zou2006sparse}{TwoStepSDFM}
#' 
#' \insertRef{eigenweb}{TwoStepSDFM}
#' 
#' \insertRef{franjic2024nowcasting}{TwoStepSDFM}
#' 
#' @seealso
#' \code{\link{sparsePCA}}: Routine for fitting estimating a sparse factor 
#' loading matrix.
#'  
#' \code{\link{kalmanFilterSmoother}}: Routine for filtering and smoothing 
#' latent factors.
#' 
#' \code{\link{twoStepDenseDFM}}: Two-step estimation routine for a dense 
#' dynamic factor model.
#' 
#' @examples
#' data(factor_model)
#' no_of_vars <- dim(factor_model$data)[2]
#' no_of_factors <- dim(factor_model$factors)[2]
#' sdfm_fit <- twoStepSDFM(data = factor_model$data, delay = factor_model$delay,
#'                         selected = rep(floor(0.5 * no_of_vars), no_of_factors),
#'                         no_of_factors = no_of_factors)
#' print(sdfm_fit)
#' sdfm_plots <- plot(sdfm_fit)
#' sdfm_plots$`Factor Time Series Plots`
#' sdfm_plots$`Loading Matrix Heatmap`
#' sdfm_plots$`Meas. Error Var.-Cov. Matrix Heatmap`
#' sdfm_plots$`Meas. Error Var.-Cov. Eigenvalue Plot`
#' 
#' @export
twoStepSDFM <- function (data, 
                         delay, 
                         selected, 
                         no_of_factors, 
                         max_factor_lag_order = 10, 
                         lag_estim_criterion = "BIC", 
                         decorr_errors = TRUE, 
                         ridge_penalty = 1e-06, 
                         lasso_penalty = NULL, 
                         max_iterations = 1000, 
                         max_no_steps = NULL, 
                         weights = NULL, 
                         comp_null = 1e-15, 
                         spca_conv_crit = 1e-04, 
                         parallel = FALSE, 
                         fcast_horizon = 0, 
                         jitter = 1e-08, 
                         svd_method = "precise") {
  func_call <- match.call()
  
  # Mishandling of data
  if (!is.zoo(data) && !is.xts(data)) {
    data_r <- try(t(as.matrix(data)), silent = TRUE)
    if (inherits(data_r, "try-error")) {
      stop(paste0("data must be a matrix, convertible to a matrix or a time-series/zoo object"))
    }
  }
  else {
    data_r <- try(coredata(data), silent = TRUE)
    if (inherits(data_r, "try-error")) {
      stop(paste0("data must be a matrix, convertible to a matrix or a time-series/zoo object"))
    }
  }
  if (!is.numeric(data_r)) {
    stop(paste0("data has non-numeric elements."))
  }
  if (any(is.infinite(data_r))) {
    stop(paste0("data cannot have (-)Inf values."))
  }
  no_of_variables <- dim(data_r)[2]
  no_of_observations <- dim(data_r)[1]
  
  # Mishandling of delay
  if (is.null(delay)) {
    delay <- matrix(rep(0, no_of_variables), ncol = 1)
  }
  else {
    delay <- checkPositiveSignedParameterVector(delay, "delay", no_of_variables)
  }
  na_ind <- FALSE
  for (col in 1:dim(data_r)[2]) {
    na_ind <- any(is.na(data_r[1:(no_of_observations - delay[col]), col]))
    if (na_ind) {
      stop(paste0("data has NA values outside the ragged edges."))
    }
  }
  obs_ind <- FALSE
  for (col in 1:dim(data_r)[2]) {
    if (delay[col] > 0) {
      obs_ind <- !all(is.na(data_r[(no_of_observations - delay[col] + 1):no_of_observations, col]))
    }
    if (obs_ind) {
      stop(paste0("data has observed values inside the ragged edges."))
    }
  }
  
  # Mishandling of dimensions and other misc. parameters
  if (no_of_variables >= no_of_observations) {
    stop(paste0("Too few observations as no_of-variables >= no_of_observations."))
  }
  no_of_factors <- checkPositiveSignedInteger(no_of_factors, "no_of_factors")
  if (no_of_factors == 0) {
    stop("no_of_factors must be strictly positive.")
  }
  if (no_of_factors > no_of_variables) {
    stop(paste0("no_of_factors must be smaller than no_of_variables."))
  }
  max_factor_lag_order <- checkPositiveSignedInteger(max_factor_lag_order, "max_factor_lag_order")
  if (max_factor_lag_order == 0) {
    stop(paste0("max_factor_lag_order must be strictly positve."))
  }
  decorr_errors <- checkBoolean(decorr_errors, "decorr_errors")
  if (is.null(lag_estim_criterion)) {
    stop(paste0("lag_estim_criterion must be either \"BIC\", \"AIC\", or \"HIC\"."))
  }
  if (!(lag_estim_criterion %in% c("AIC", "BIC", "HIC"))) {
    stop(paste0("lag_estim_criterion must be either \"BIC\", \"AIC\", or \"HIC\"."))
  }
  spca_conv_crit <- checkPositiveDouble(spca_conv_crit, "spca_conv_crit")
  if (spca_conv_crit == 0) {
    warning("spca_conv_crit should not be exactly 0. It will be jittered before further use.")
    spca_conv_crit <- 1e-15
  }
  parallel <- checkBoolean(parallel, "parallel")
  fcast_horizon <- checkPositiveSignedInteger(fcast_horizon, "fcast_horizon")
  jitter <- checkPositiveDouble(jitter, "jitter")
  if (!(svd_method %in% c("fast", "precise"))) {
    stop(paste0("svd_method must be \"fast\" for usage of Eigen's BDCSVD or \"precise\" for usage of Eigen's JacobiSVD"))
  }
  comp_null <- checkPositiveDouble(comp_null, "comp_null")
  if (comp_null == 0) {
    warning("comp_null should not be exactly 0. It will be jittered before further use.")
    comp_null <- 1e-15
  }
  
  # Mishandling of selected
  if (is.null(selected)) {
    selected <- matrix(rep(no_of_variables, no_of_factors), ncol = 1)
  }
  else {
    selected <- checkPositiveSignedParameterVector(selected, "selected", no_of_factors)
  }
  if (any(selected > no_of_variables)) {
    warning(paste0("The elements in selected should not exceed the number of variables ", 
                   no_of_variables, ". The corresponding variables are set to ", no_of_variables, "."))
    selected[which(selected > no_of_variables), ] <- no_of_variables
  }
  
  # Mishandling of ridge_penalty and lasso_penalty
  ridge_penalty <- checkPositiveDouble(ridge_penalty, "ridge_penalty")
  if (!is.null(lasso_penalty)) {
    if (!is.numeric(lasso_penalty) || any(is.na(lasso_penalty))) {
      stop(paste0("lasso_penalty must be a vector of non-NA numeric values."))
    }
    if (length(lasso_penalty) != no_of_factors) {
      stop(paste0("lasso_penalty must be of length no_of_factors = ", no_of_factors))
    }
    if (any(lasso_penalty < 0)) {
      stop(paste0("All elements of lasso_penalty non-negative."))
    }
  }
  else {
    lasso_penalty <- rep(-2147483647L, no_of_factors)
  }
  
  # Mishandling of max_iterations and max_no_steps
  max_iterations <- checkPositiveSignedInteger(max_iterations, "max_iterations")
  if (!is.null(max_no_steps)) {
    max_no_steps <- checkPositiveSignedInteger(max_no_steps, "max_no_steps")
    if (max_no_steps == 0) {
      stop(paste0("max_no_steps must be strictly positve."))
    }
  }
  else {
    max_no_steps <- -2147483647L
  }
  
  # Mishandling of weights
  if (!is.null(weights)) {
    if (!is.numeric(weights) || any(is.na(weights)) || any(weights <= 
                                                           0)) {
      stop(paste0("weights must be a vector of non-NA numeric values strictly greater 0."))
    }
    if (length(weights) != no_of_variables) {
      stop(paste0("weights must be of length no_of_variables = ", no_of_variables))
    }
    if (sum(weights) != 1) {
      message("weights are standardised to sum to 1")
      weights <- weights / sum(weights)
    }
  }
  else {
    weights <- rep(1, no_of_variables)
  }
  
  result <- runSDFMKFS(X_in = data_r, delay = delay, selected = selected, 
                       R = as.integer(no_of_factors), order = as.integer(max_factor_lag_order), 
                       decorr_errors = decorr_errors, crit = lag_estim_criterion, 
                       l2 = ridge_penalty, l1 = lasso_penalty, max_iterations = as.integer(max_iterations), 
                       steps = max_no_steps, weights = weights, comp_null = comp_null, 
                       spca_conv_crit = spca_conv_crit, parallel = parallel, 
                       fcast_horizon = fcast_horizon, jitter = jitter, svd_method = svd_method)
  
  # Re-name the results
  names(result) <- c("loading_matrix_estimate", "filtered_state_variance", 
                     "companion_form_smoothed_factors", "smoothed_state_variance", 
                     "error_var_cov_cholesky_factor", "factor_var_lag_order", 
                     "llt_success_code")
  
  # Retrieve the factors and loading matrix from the companion forms
  result$smoothed_factors <- result$companion_form_smoothed_factors[1:no_of_factors, 1:(no_of_observations + fcast_horizon), drop = FALSE]
  rownames(result$smoothed_factors) <- paste0("Factor ", 1:no_of_factors)
  result$loading_matrix_estimate <- result$loading_matrix_estimate[, 1:no_of_factors, drop = FALSE]
  
  # Store the data in the return object
  result$data <- data
  
  # Retrieve the correct KFS uncertainty blocks from the companion form
  no_of_cols <- no_of_observations * no_of_factors
  block_size <- result$factor_var_lag_order * no_of_factors
  temp_smoothed_state_variance <- result$smoothed_state_variance
  result$smoothed_state_variance <- matrix(NaN, no_of_factors, no_of_factors * (no_of_observations + fcast_horizon))
  result$smoothed_state_variance[, 1:no_of_factors] <- temp_smoothed_state_variance[1:no_of_factors, 1:no_of_factors]
  for (curr_obs in 2:(no_of_observations + fcast_horizon)) {
    block_starting_index <- (curr_obs - 1) * block_size + 1
    block_ending_index <- block_starting_index + no_of_factors - 1
    factor_block_starting_ind <- (curr_obs - 1) * no_of_factors + 1
    factor_block_ending_ind <- factor_block_starting_ind + no_of_factors - 1
    if (curr_obs <= no_of_observations) {
      result$smoothed_state_variance[, factor_block_starting_ind:factor_block_ending_ind] <- 
        temp_smoothed_state_variance[1:no_of_factors, block_starting_index:block_ending_index]
    }
    else {
      result$smoothed_state_variance[, factor_block_starting_ind:factor_block_ending_ind] <- 
        result$filtered_state_variance[1:no_of_factors, block_starting_index:block_ending_index]
    }
  }
  
  # Re-shuffle the results and cut some of them for logical coherency and debloating
  result <- result[c("data", "loading_matrix_estimate", "smoothed_factors", "smoothed_state_variance", 
                     "factor_var_lag_order", "error_var_cov_cholesky_factor", "llt_success_code")]
  
  # Compute the Cholesky factor as runDFMKFS only returns the inverse of the lower triangular Cholesky factor
  result$error_var_cov_cholesky_factor <- tryCatch({
    solve(result$error_var_cov_cholesky_factor)
  }, error = function(e) {
    return(paste("ERROR:", conditionMessage(e)))
  })
  if (is.matrix(result$error_var_cov_cholesky_factor)) {
    result$error_var_cov_cholesky_factor[upper.tri(result$error_var_cov_cholesky_factor)] <- 0
  }
  
  # Turn the smoothed factors into zoo object if data is a zoo object
  if (is.zoo(data) || is.xts(data)) {
    start_vector <- c(year(time(data)[1]), month(time(data)[1]))
    result$smoothed_factors <- as.zoo(ts(t(result$smoothed_factors), start = start_vector, frequency = 12))
  }
  
  # Collect preliminary objects in the return object
  result$call <- func_call
  result$factor_fcast_horizon <- fcast_horizon
  result$data_delay <- delay
  class(result) <- "SDFMFit"
  return(result)
}

#' @name print.SDFMFit
#' @title Generic printing function for SDFMFit S3 objects
#' @description
#' Print a compact summary of an `SDFMFit` object.
#'
#' @param x `SDFMFit` object.
#' @param ... Additional parameters for the plotting functions.
#'
#' @return
#' No return value; Prints a summary to the console.
#'
#' @author
#' Domenic Franjic
#' 
#' @export
print.SDFMFit <- function (x, ...) 
{
  simulated_time_series <- is.zoo(x$smoothed_factors)
  no_of_factors <- ifelse(simulated_time_series, dim(x$smoothed_factors)[2], dim(x$smoothed_factors)[1])
  no_of_obs <- ifelse(simulated_time_series, dim(x$data)[1], dim(x$data)[2])
  cat("Simulated Dynamic Factor Model\n")
  cat("=========================================================================\n")
  cat("No. of Observations                        :", ifelse(simulated_time_series, dim(x$data)[1], dim(x$data)[2]), "\n")
  cat("No. of Variables                           :", ifelse(simulated_time_series, dim(x$data)[2], dim(x$data)[1]), "\n")
  cat("No. of Factors                             :", no_of_factors, "\n")
  cat("Factor Lag Order                           :", x$factor_var_lag_order,  "\n")
  cat("No. of zero elements in the loading matrix :", sum(x$loading_matrix_estimate == 0), "\n")
  if (x$llt_success_code == -1) {
    cat("Info: LLT failed. Used robust LDLT instead.\n")
  }
  else if (x$llt_success_code == -1) {
    cat("Warning: LLT and LDLT failed. Used uncorrelated errors.\n")
  }
  cat("=========================================================================\n")
  cat("Head of the factors :\n")
  max_print <- min(5, no_of_obs)
  if (simulated_time_series) {
    print(head(x$smoothed_factors, max_print))
  }
  else {
    print(x$smoothed_factors[, 1:max_print])
  }
  cat("Tail of the factors :\n")
  if (simulated_time_series) {
    print(tail(x$smoothed_factors, max_print))
  }
  else {
    print(x$smoothed_factors[, (dim(x$smoothed_factors)[2] - (max_print - 1)):(dim(x$smoothed_factors)[2])])
  }
  max_print_loadings <- min(5, ifelse(simulated_time_series, dim(x$smoothed_factors)[1], dim(x$smoothed_factors)[2]))
  cat("Head of the loading matrix :\n")
  print(head(x$loading_matrix_estimate, max_print_loadings))
  cat("Tail of the loading matrix :\n")
  print(tail(x$loading_matrix_estimate, max_print_loadings))
  cat("=========================================================================\n")
  invisible(x)
}

#' @name plot.SDFMFit
#' @title Generic plotting function for SDFMFit S3 objects
#' @param x `SDFMFit` object.
#' @param axis_text_size Numeric size of x- and y-axis labels. Passed to ggplot2 
#' `theme(..., text = element_text(size = axis_text_size))`.
#' @param legend_title_text_size Numeric size of x- and y-axis labels. Passed to
#' ggplot2 
#' `theme(..., legend.title = element_text(size = legend_title_text_size))`.
#' @param ... Additional parameters for the plotting functions.
#' 
#' @return
#' A named list of plot objects:
#' \describe{
#'   \item{`Factor Time Series Plots`}{`patchwork`/`ggplot` object graphing the 
#'   estimated factors over time with 95% confidence bands based on the smoother 
#'   uncertainty of the Kalman Filter and Smoother.}
#'   \item{`Loading Matrix Heatmap`}{`ggplot` object showing a heatmap of the 
#'   estimated factor loadings. Zeros are highlighted in black.}
#'   \item{`Meas. Error Var.-Cov. Matrix Heatmap`}{`ggplot` object showing a 
#'   heatmap of the measurement error variance-covariance matrix.}
#'   \item{`Eigenvalue Plot`}{`ggplot` object showing a bar plot of the 
#'   eigenvalues of the measurement error variance–covariance matrix.}
#'  }
#' 
#' @author
#' Domenic Franjic
#' 
#' @export
plot.SDFMFit <- function (x, 
                          axis_text_size = 20, 
                          legend_title_text_size = 20, 
                          ...) 
{
  out_list <- list()
  if (is.zoo(x$data)) {
    series_names <- colnames(x$data)
    no_of_factors <- dim(x$smoothed_factors)[2]
    no_of_obs <- dim(x$factor_estimate)[2]
    time_vector <- as.Date(time(x$smoothed_factors))
    factors <- x$smoothed_factors
  }
  else {
    series_names <- rownames(x$data)
    no_of_factors <- dim(x$smoothed_factors)[1]
    no_of_obs <- dim(x$factor_estimate)[2]
    time_vector <- 1:dim(x$smoothed_factors)[2]
    factors <- t(x$smoothed_factors)
    factors <- as.zoo(ts(factors, start = c(1, 1), frequency = 12))
  }
  
  out_list$`Factor Time Series Plots` <- plotFactorEstimates(factors, x$smoothed_state_variance, no_of_factors, axis_text_size)
  
  out_list$`Loading Matrix Heatmap` <- plotLoadingHeatMap(x$loading_matrix_estim, 
                                                          series_names, no_of_factors,
                                                          axis_text_size, legend_title_text_size)
  
  if (is.character(x$error_var_cov_cholesky_factor)) {
    if (is.zoo(x$data)) {
      residuals <- coredata(na.omit(x$data)) - coredata(x$factor_estimate) %*% t(x$loading_matrix_estim)
    }
    else {
      residuals <- na.omit(t(x$data)) - t(x$factor_estimate) %*% t(x$loading_matrix_estim)
    }
    measurement_error_var_cov_df <- as.data.frame(t(residuals) %*% residuals * 1/(dim(residuals)[1] - 1))
    out_list$`Meas. Error Var.-Cov. Matrix Heatmap` <- plotMeasVarCovHeatmap(measurement_error_var_cov_df, 
                                                                             series_names, 
                                                                             axis_text_size, 
                                                                             legend_title_text_size)
  }
  else {
    measurement_error_var_cov_df <- as.data.frame(x$error_var_cov_cholesky_factor %*% t(x$error_var_cov_cholesky_factor))
    out_list$`Meas. Error Var.-Cov. Matrix Heatmap` <- plotMeasVarCovHeatmap(measurement_error_var_cov_df, 
                                                                             series_names, 
                                                                             axis_text_size, 
                                                                             legend_title_text_size)
  }
  
  out_list$`Meas. Error Var.-Cov. Eigenvalue Plot` <- plotMeasVarCovEigenvalues(eigen(measurement_error_var_cov_df)$values, 
                                                                                no_of_factors, axis_text_size, legend_title_text_size)
  
  return(out_list)
}

#' @name predict.SDFMFit
#' @title Generic plotting function for SDFMFit S3 objects
#' @description
#' Predict all missing observations due to ragged edges in the data set plus 
#' horizon steps ahead.
#' 
#' @param object `SDFMFit` object.
#' @param horizon Number of forecasting steps into the future. Must be smaller 
#' than or equal to `x$factor_fcast_horizon`.
#' @param ... Additional parameters for the prediction function.
#' 
#' @return
#' A named list of plot objects:
#' \describe{
#'   \item{data}{Object containing the original data. The object inherits its 
#'   class from `object$data`: If data is provided as `zoo`, `data` will be a 
#'   `zoo` object. If `data` is provided as matrix, `data` will be a 
#'   (`no_of_factors`\eqn{\times}{x}`no_of_obs`) matrix.}
#'   \item{data_missing_pred}{Object containing only the predictions of all 
#'   missing observations plus the forecasts. Inherits its class from 
#'   `object$data` as above.}
#'   \item{data_imputed}{Object containing the observed data, predictions of 
#'   all missing observations plus the forecasts. Inherits its class from 
#'   `object$data` as above.}
#'  }
#' 
#' @author
#' Domenic Franjic
#' 
#' @examples
#' data(factor_model)
#' no_of_vars <- dim(factor_model$data)[2]
#' no_of_factors <- dim(factor_model$factors)[2]
#' sdfm_fit <- twoStepSDFM(data = factor_model$data, delay = factor_model$delay,
#'                         selected = rep(floor(0.5 * no_of_vars), no_of_factors),
#'                         no_of_factors = no_of_factors, fcast_horizon = 5)
#' dfm_fit <- twoStepDenseDFM(data = factor_model$data, delay = factor_model$delay, 
#'                            no_of_factors = no_of_factors, fcast_horizon = 5)
#' predict(sdfm_fit, horizon = 5)
#' predict(dfm_fit, horizon = 5)
#' 
#' @export
predict.SDFMFit <- function (object, 
                             horizon = 0,
                             ...) {
  horizon <- checkPositiveSignedInteger(horizon, "horizon")
  if (horizon > object$factor_fcast_horizon) {
    stop("There are not enough forecasts of the estimated factors. Re-run twoStepSDFM/twoStepDenseDFM setting fcast_horizon >= horizon")
  }
  if (is.zoo(object$data)) {
    no_of_obs <- dim(object$data)[1]
    no_of_vars <- dim(object$data)[2]
    time_vector <- as.Date(time(object$smoothed_factors))
    factors <- object$smoothed_factors
    data <- object$data
  }
  else {
    no_of_obs <- dim(object$data)[2]
    no_of_vars <- dim(object$data)[1]
    time_vector <- 1:dim(object$smoothed_factors)[2]
    factors <- t(object$smoothed_factors)
    factors <- as.zoo(ts(factors, start = c(1, 1), frequency = 12))
    data <- as.zoo(ts(data, start = c(1, 1), frequency = 12))
  }
  
  data_pred <- (object$loading_matrix_estimate %*% t(coredata(factors)))[, 1:(no_of_obs + horizon)]
  object$data
  result <- list()
  result$data <- object$data
  result$data_missing_pred <- matrix(NaN, no_of_vars, no_of_obs + horizon)
  result$data_imputed <- matrix(NaN, no_of_vars, no_of_obs + horizon)
  result$data_imputed[, 1:no_of_obs] <- t(coredata(data))
  for (var in 1:no_of_vars) {
    if (object$data_delay[var] > 0) {
      curr_predictions <- data_pred[var, (no_of_obs - object$data_delay[var] + 1):(no_of_obs + horizon)]
      result$data_missing_pred[var, (no_of_obs - object$data_delay[var] + 1):(no_of_obs + horizon)] <- curr_predictions
      result$data_imputed[var, (no_of_obs - object$data_delay[var] + 1):(no_of_obs + horizon)] <- curr_predictions
    }
    else if (horizon > 0) {
      curr_predictions <- data_pred[var, (no_of_obs + 1):(no_of_obs + horizon)]
      result$data_missing_pred[var, (no_of_obs + 1):(no_of_obs + horizon)] <- curr_predictions
      result$data_imputed[var, (no_of_obs + 1):(no_of_obs + horizon)] <- curr_predictions
    }
  }
  if (is.zoo(object$data)) {
    result$data_missing_pred <- as.zoo(ts(t(result$data_missing_pred), start = c(year(time_vector[1]), month(time_vector[1])), 
                                          frequency = 12))
    result$data_imputed <- as.zoo(ts(t(result$data_imputed), start = c(year(time_vector[1]), month(time_vector[1])), 
                                     frequency = 12))
  }
  
  return(result)
}

test_that("", {
  
  # Load data
  data("factor_model")
  data_zoo <- scale(factor_model$data)
  data_mat <- t(coredata(scale(factor_model$data)))
  no_of_factors <- dim(factor_model$factors)[2]
  no_of_vars <- dim(data_zoo)[2]
  selected <- rep(floor(no_of_vars * 0.5), no_of_factors)

  # Parsing zoo object
  expect_silent(sdfm_fit_zoo <- twoStepSDFM(data = data_zoo, delay = factor_model$delay, selected = selected,
                                            no_of_factors = no_of_factors))
  
  # Parsing a matrix object
  expect_silent(sdfm_fit_mat <- twoStepSDFM(data = data_mat, delay = factor_model$delay, selected = selected,
                                            no_of_factors = no_of_factors))
  
  # Basic checks 
  expect_equal(t(coredata(sdfm_fit_zoo$data)), sdfm_fit_mat$data)
  expect_equal(sdfm_fit_zoo$loading_matrix_estimate, sdfm_fit_mat$loading_matrix_estimate)
  expect_equal(t(coredata(sdfm_fit_zoo$smoothed_factors)), sdfm_fit_mat$smoothed_factors)
  expect_equal(sdfm_fit_zoo$smoothed_state_variance, sdfm_fit_mat$smoothed_state_variance)
  expect_equal(sdfm_fit_zoo$factor_var_lag_order, sdfm_fit_mat$factor_var_lag_order)
  expect_equal(sdfm_fit_zoo$error_var_cov_cholesky_factor, sdfm_fit_mat$error_var_cov_cholesky_factor)
  expect_equal(sdfm_fit_zoo$llt_success_code, sdfm_fit_mat$llt_success_code)
  expect_equal(sdfm_fit_zoo$factor_fcast_horizon, sdfm_fit_mat$factor_fcast_horizon)
  expect_equal(sdfm_fit_zoo$data_delay, sdfm_fit_mat$data_delay)
  expect_true(is.zoo(sdfm_fit_zoo$smoothed_factors))
  expect_true(!is.zoo(sdfm_fit_mat$smoothed_factors))
  
})

#' @useDynLib TwoStepSDFM, .registration=TRUE
#' @importFrom Rcpp sourceCpp
#' @import zoo
#' @import xts
#' @import lubridate
NULL

# SPDX-License-Identifier: GPL-3.0-or-later
#
#  Copyright (C) 2024-2026 Domenic Franjic
#
#  This file is part of TwoStepSDFM.
#
#  TwoStepSDFM is free software: you can redistribute
#  it and/or modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TwoStepSDFM is distributed in the hope that it
#  will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with TwoStepSDFM. If not, see <https://www.gnu.org/licenses/>.

#' @name noOfFactors
#' @title Estimate the number of Factors
#' @description
#' Estimate the number of factors of a linear Gaussian latent factor model using 
#' the eigenvalue slope test of \insertRef{onatski2009testing}{TwoStepSDFM} and 
#' the infromation criterion based approach of 
#' \insertRef{bai2002determining}{TwoStepSDFM}.
#' @param data Numeric (no_of_vars \eqn{\times}{x} no_of_obs) matrix of data or 
#' zoo/xts object sampled at the same frequency.
#' @param min_no_factors Integer minimum number of factors to be tested.
#' @param max_no_factors Integer maximum number of factors to be tested (should 
#' be at most min_no_factors + 17).
#' @param confidence_threshold Numeric threshold value to stop the testing procedure.
#' 
#' @details
#' The \insertCite{onatski2009testing}{TwoStepSDFM} procedure splits the data 
#' matrix along the time dimension into two equally sized (`no_of_vars` 
#' \eqn{\times}{x} `cut_off`) sub-matrices \eqn{\bm{X}_{1/2}}{`data_fst_half`} 
#' and \eqn{\bm{X}_{2/2}}{`data_snd_half`}. It then proceeds to build 
#' \eqn{\tilde{\bm{X}} := \bm{X}_{1/2} + i\bm{X}_{2/2}}{`complex_data = data_fst_half + i * data_snd_half`},
#' where \eqn{i=\sqrt{-1}}{`i=sqrt(-1)`}. We then compute the eigenvalues of the 
#' Gram matrix \eqn{\tilde{\bm{X}} 
#' \tilde{\bm{X}}^{\dagger}}{`complex_data %*% Conj(t(complex_data))`}, where 
#' \eqn{\tilde{\bm{X}}^{\dagger}}{`Conj(t(complex_data))`} represents the 
#' adjoint. Finally, a test based on the computed eigenvalues is performed. 
#' This test is an iterative testing procedure, starting by testing the null
#' that the true number of factors is `min_no_factors`. If the test is rejected
#' by comparison of the \eqn{p}{p}-value against `confidence_threshold`, we 
#' test whether the true number of factors is `min_no_factors + 1` until we can 
#' no longer reject at `confidence_threshold` or `max_no_factors` is reached.
#' 
#' As the distribution of the eigenvalues under the null is nonstandard
#' \insertCite{onatski2009testing}{TwoStepSDFM}, 
#' simulated critical values are used. They are retrieved from
#' \insertRef{onatski2009testing_supl}{TwoStepSDFM}. As the range of the 
#' simulated critical values is limited, the minimum and maximum number of 
#' potential factors is limited such that `max_no_factors` should be no more 
#' than `min_no_factors + 17`. However, it is recommended to operate well below 
#' this maximum as the test size decreases with 
#' `max_no_factors - min_no_factors`. 
#' 
#' The \insertCite{bai2002determining}{TwoStepSDFM} information criterion 
#' determines the number of factors by minimising a BIC. Here, three different 
#' penalty terms are provided. It is up to the user to determine the most 
#' appropriate for the problem at hand. In general, however, the second 
#' information criterion is used.
#' 
#' @return 
#' An object of class `NoOfFactorsFit` with components:
#' \describe{
#'   \item{no_of_factors}{Integer estimated number of factors.}
#'   \item{p_value}{Numeric \eqn{p}{p}-value of the final test.}
#'   \item{confidence_threshold}{Numeric significance level used.}
#'   \item{statistic}{Numeric test statistic value of the last test.}
#'   \item{eigen_values}{Numeric vector of eigenvectors of the complex data Gram
#'   matrix.}
#' }
#'
#' @author
#' Domenic Franjic
#' 
#' @references
#' \insertRef{onatski2009testing}{TwoStepSDFM}
#' 
#' \insertRef{onatski2009testing_supl}{TwoStepSDFM}
#'
#' @examples
#' data(factor_model)
#' no_of_factors_estim <- noOfFactors(data = factor_model$data, min_no_factors = 1, 
#'                                    max_no_factors = 5, confidence_threshold = 0.05)
#' print(no_of_factors_estim)
#' factor_estim_plots <- plot(no_of_factors_estim)
#' factor_estim_plots$`Eigen Value Plot Test Procedure`
#' factor_estim_plots$`IC plot for IC1`
#' factor_estim_plots$`IC plot for IC2`
#' factor_estim_plots$`IC plot for IC3`
#' 
#' @export
noOfFactors <- function(data, min_no_factors = 1, max_no_factors = 7, confidence_threshold = 0.05){
  func_call <- match.call()
  
  # Mishandling of data
  if(!is.zoo(data) && !is.xts(data)){
    data_r <- try(as.matrix(data), silent = TRUE)
    if (inherits(data_r, "try-error")) {
      stop(paste0("data must be a matrix, convertible to a matrix or a time-series/zoo object"))
    }
  }else{
    data_r <- try(t(coredata(data)), silent = TRUE)
    if (inherits(data_r, "try-error")) {
      stop(paste0("data must be a matrix, convertible to a matrix or a time-series/zoo object"))
    }
  }
  if(!is.numeric(data_r)){
    stop(paste0("data has non-numeric elements."))
  }
  if(any(is.infinite(data_r))){
    stop(paste0("data cannot have (-)Inf values."))
  }
  na_ind <- -unique(which(is.na(data_r), arr.ind = TRUE)[, 2])
  if(length(na_ind) != 0){
    message(paste0("Cut ", length(na_ind)," observations due to NAs."))
    no_na_data <- as.matrix(data_r[, na_ind, drop = FALSE])
  }else{
    no_na_data <- as.matrix(data_r[, , drop = FALSE])
  }
  
  
  # Mishandling of max_no_factors and min_no_factors
  max_no_factors <- checkPositiveSignedInteger(max_no_factors, "max_no_factors");
  if(max_no_factors >= dim(no_na_data)[1] - 2){
    stop(paste0("max_no_factors must be smaller than dim(no_na_data)[1] - 2 = ", dim(no_na_data)[1] - 2, "."))
  }
  min_no_factors <- checkPositiveSignedInteger(min_no_factors, "min_no_factors");
  if(min_no_factors <= 0){
    stop(paste0("min_no_factors must be strictly positive."))
  }
  if(min_no_factors >= max_no_factors){
    stop(paste0("max_no_factors must be strictly greater than min_no_factors."))
  }
  if (7 < max_no_factors - min_no_factors) {
    warning(paste0("Power of the test might be low as max_no_factors - min_no_factors = ", 
                   max_no_factors - min_no_factors," > 7."))
  }
  if (18 < max_no_factors - min_no_factors) {
    stop(paste0("Critical values for max_no_factors - min_no_factors = ", max_no_factors - min_no_factors, 
                " > 18 not available. Decrease max_no_factors"))
  }
  
  # Mishandling of confidence_threshold = 0.05
  confidence_threshold <- checkPositiveDouble(confidence_threshold, "confidence_threshold")
  if(confidence_threshold <= 0 || confidence_threshold >= 1){
    stop(paste0("confidence_threshold must be in (0,1)."))
  }
  
  # The values for the test-statistics stem: https://www.econometricsociety.org/publications/econometrica/2009/09/01/testing-hypotheses-about-number-factors-large-factor-models (Last accessed: 25.11.2025, 10:03)
  file_path <- system.file("extdata", "Onatski_test_stats_csv.txt", package = "TwoStepSDFM")
  test_values <- as.matrix(read.table(file_path, sep = ",", header = FALSE))
  results <- list()
  results$test <- runNoOfFactorsTest(no_na_data, test_values, min_no_factors, max_no_factors, confidence_threshold)
  results$ic <- runNoOfFactorsInfoCrit(no_na_data, max_no_factors)
  results$ic$ic_one_min <- which.min(results$ic$information_crit[, 1])
  results$ic$ic_two_min <- which.min(results$ic$information_crit[, 2])
  results$ic$ic_three_min <- which.min(results$ic$information_crit[, 3])
  results$max_no_factors <- max_no_factors
  
  results$call <- call
  class(results) <- "NoOfFactorsFit"
  return(results)
}


#' @name print.NoOfFactorsFit
#' @title Generic printing function for NoOfFactorsFit S3 objects
#' @description
#' Print a compact summary of an `NoOfFactorsFit` object.
#'
#' @param x `NoOfFactorsFit` object.
#' @param ... Additional parameters for the plotting functions.
#'
#' @return
#' No return value; Prints a summary to the console.
#'
#' @author
#' Domenic Franjic
#' 
#' @export
print.NoOfFactorsFit <- function(x, ...) {
  cat("Results of the Onatsky testing procedure:\n")
  cat(paste0("The estimated no. of factors is ", x$test$no_of_factors, " with a p-value of ", x$test$p_value, " and a critical value of alpha = ", x$test$confidence_threshold))
  if(x$test$no_of_factors == x$max_no_factors - 1){
    cat("\n")
    warning(paste0("No. of factors has been chosen as max_no_factors - 1 = ", x$max_no_factors - 1, 
                   ". It might be necessary to increase max_no_factors and repeat the procedure"))
  }
  cat("\n\n")
  cat("Results of the Bai and Ng information criteria:\n")
  cat(paste0("No. of factors according to IC1: ", x$ic$ic_one_min, "\n"))
  cat(paste0("No. of factors according to IC2: ", x$ic$ic_one_min, "\n"))
  cat(paste0("No. of factors according to IC3: ", x$ic$ic_one_min, "\n"))
  invisible(x)
}

#' @name plot.NoOfFactorsFit
#' @title Generic plotting function for NoOfFactorsFit S3 objects
#' @description
#' Create diagnostic plots for an `NoOfFactorsFit` object, 
#'
#' @param x `NoOfFactorsFit` object.
#' @param axis_text_size Numeric size of x- and y-axis labels. Prased to ggplot2 
#' `theme(..., text = element_text(size = axis_text_size))`.
#' @param legend_title_text_size Numeric size of x- and y-axis labels. Prased to
#' ggplot2 
#' `theme(..., legend.title = element_text(size = legend_title_text_size))`.
#' @param ... Additional parameters for the plotting functions.
#'
#' @return
#' A named list of plot objects:
#' \describe{
#'   \item{`Eigen Value Plot`}{`ggplot` object showing a bar plot of the 
#'   eigenvalues of the complex data Gram matrix.}
#' }
#'
#' @author
#' Domenic Franjic
#' 
#' @export
plot.NoOfFactorsFit <- function(x, 
                                axis_text_size = 20, 
                                legend_title_text_size = 20, 
                                ...) {
  out_list <- list()
  
  # Result plot for Onatsky testing procedure
  out_list$`Eigen Value Plot Test Procedure` <- plotMeasVarCovEigenvalues(x$test$eigen_values,
                                                                          x$test$no_of_factors,
                                                                          axis_text_size,
                                                                          legend_title_text_size)
  
  # Result plots for IC criteria of Bai and Ng
  out_list$`IC plot for IC1` <- plotInformationCrit(x$ic$information_crit[, 1], x$ic$ic_one_min, axis_text_size)
  out_list$`IC plot for IC2` <- plotInformationCrit(x$ic$information_crit[, 2], x$ic$ic_two_min, axis_text_size)
  out_list$`IC plot for IC3` <- plotInformationCrit(x$ic$information_crit[, 3], x$ic$ic_three_min, axis_text_size)
  
  return(out_list)
}

test_that("", {

  # Load data
  data("factor_model")
  data <- scale(factor_model$data)

  # Parsing zoo object
  expect_no_error(factor_fit_zoo <- noOfFactors(data = data, min_no_factors = 1, max_no_factors = 5))
  
  # Parsing matrix object
  expect_no_error(factor_fit_matrix <- noOfFactors(data = t(coredata(data)), min_no_factors = 1, max_no_factors = 5))
  
  # Check whether the results are equal
  expect_equal(factor_fit_zoo, factor_fit_matrix)
  
})

#' @useDynLib TwoStepSDFM, .registration=TRUE
#' @importFrom Rcpp sourceCpp
#' @importFrom Rdpack reprompt
#' @import zoo
#' @import xts
#' @import lubridate
#' @import ggplot2
#' @import stats
#' @import utils
#' @import patchwork
NULL

# SPDX-License-Identifier: GPL-3.0-or-later
#
#  Copyright (C) 2024-2026 Domenic Franjic
#
#  This file is part of TwoStepSDFM.
#
#  TwoStepSDFM is free software: you can redistribute
#  it and/or modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TwoStepSDFM is distributed in the hope that it
#  will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with TwoStepSDFM. If not, see <https://www.gnu.org/licenses/>.

#' @name kalmanFilterSmoother
#' @title Univariate Representation of the Multivariate Kalman Filter and 
#' Smoother
#' @description
#' Filter and smooth the latent states/factors of a linear Gaussian state-space 
#' model, with measurement equation
#' \deqn{
#'   \bm{x}_t = \bm{\Lambda} \bm{f}_{t} + \bm{\xi}_t,\quad \bm{\xi}_t \sim \mathcal{N}(\bm{0}, \bm{\Sigma}_{\xi}),
#' }
#' and transition equation
#' \deqn{
#'   \bm{f}_t = \sum_{p=0}^P\bm{\Phi}_p \bm{f}_{t-p} + \bm{\epsilon}_t,\quad \bm{\epsilon}_t \sim \mathcal{N}(\bm{0}, \bm{\Sigma}_{f}).
#' }
#' for t = 1, \dots, T. For filtering and smoothing, the univariate 
#' representation of the multivariate Kalman Filter and Smoother is implemented 
#' according to \insertRef{koopman2000fast}{TwoStepSDFM}.
#' 
#' @param data Numeric (no_of_vars \eqn{\times}{x} no_of_obs) matrix of data or 
#' zoo/xts object sampled at the same frequency.
#' @param delay Integer vector of variable delays.
#' @param no_of_factors Integer number of factors.
#' @param loading_matrix Numeric (no_of_vars \eqn{\times}{x} no_of_factors) 
#' loading matrix.
#' @param meas_error_var_cov Numeric (no_of_factors \eqn{\times}{x} 
#' no_of_factors) variance-covariance matrix of the measurement errors.
#' @param trans_error_var_cov Numeric (no_of_vars \eqn{\times}{x} no_of_vars) 
#' variance-covariance matrix of the transition errors.
#' @param trans_var_coeff Either a list of length max_factor_lag_order with each
#' entry a numeric (no_of_factors \eqn{\times}{x} no_of_factors) VAR 
#' coefficient matrix or a matrix of dimensions (no_of_factors x(no_of_factors 
#' * max_factor_lag_order)) holding the VAR coefficients of the factor VAR 
#' process in each (no_of_factors \eqn{\times}{x} no_of_factors) block.
#' @param factor_lag_order Integer order of the VAR process in the state 
#' equation.
#' @param fcast_horizon Integer number of additional Filter predictions into the 
#' future.
#' @param decorr_errors Logical, whether or not the errors should be 
#' decorrelated (should be `TRUE` if `meas_error_var_cov` is not diagonal).
#' @param comp_null Computational zero.
#' @param parallel Logical, whether or not to use Eigen's internal parallel 
#' matrix operations.
#' @param jitter Numerical jitter for stability of internal solver algorithms. 
#' The jitter is added to the diagonal entries of the variance-covariance matrix 
#' of the measurement errors.
#' 
#' @details
#' To implement the univariate representation of the Kalman Filter and Smoother, 
#' the measurement error term has to be cross-sectionally uncorrelated. If 
#'`meas_error_var_cov` is not diagonal, one should set `decorr_errors = TRUE` so
#' that the data can be decorrelated internally prior to filtering and smoothing.
#'  
#' When decorrelating, the function first adds `jitter` to the diagonal elements 
#' of `meas_error_var_cov` and then tries to compute the Cholesky factor via 
#' Eigen's standard LLT decomposition \insertCite{eigenweb}{TwoStepSDFM}. If the
#' initial decorrelation fails, it silently switches to Eigen's more robust, 
#' but slower, LDLT decomposition with pivoting 
#' \insertCite{eigenweb}{TwoStepSDFM}. If this also fails, it is likely that 
#' `meas_error_var_cov` is not well-behaved. The analysis should be repeated 
#' with a larger `jitter` or a more robust variance-covariance matrix 
#' (estimator). The success of the internal Cholesky decomposition is reported 
#' by `llt_success_code`.
#' 
#' @return
#' An object of class `KFSFit` with components:
#' \describe{
#'   \item{data}{Original data matrix.}
#'   \item{smoothed_factors}{Object containing the smoothed factor estimates. 
#'   The object inherits its class from `data`: If `data` is provided as `zoo`, 
#'   `smoothed_factors` will be a `zoo` object. If `data` is provided as 
#'   `matrix`, `smoothed_factors` will be a (`no_of_factors` \eqn{\times}{x} 
#'   `no_of_obs`) matrix.}
#'   \item{smoothed_state_variance}{(`no_of_factors` \eqn{\times}{x} 
#'   (`no_of_factors * no_of_obs`)) matrix, where each (`no_of_factors` 
#'   \eqn{\times}{x} `no_of_factors`) block represents the smoother uncertainty
#'   at time point \eqn{t}{t}}
#'   \item{factor_var_lag_order}{Integer order of the VAR process in the state 
#'   equation.}
#'   \item{llt_success_code}{Integer indicating the status of the Cholesky
#'   factorization: `0` = LLT succeeded, `-1` = LLT failed but LDLT succeeded,
#'   `-2` = both failed and errors are treated as uncorrelated.}
#' }
#'
#' @author
#' Domenic Franjic
#' 
#' @references
#' \insertRef{koopman2000fast}{TwoStepSDFM}
#' 
#' \insertRef{eigenweb}{TwoStepSDFM}
#'
#' @examples
#' data(factor_model)
#' no_of_factors <- dim(factor_model$factors)[2]
#' factor_lag_order <- dim(factor_model$trans_var_coeff)[2] / no_of_factors
#' filter_fit <- kalmanFilterSmoother(data = factor_model$data, delay = factor_model$delay, 
#'                                    no_of_factors = no_of_factors, 
#'                                    loading_matrix = factor_model$loading_matrix, 
#'                                    meas_error_var_cov = factor_model$meas_error_var_cov, 
#'                                    trans_error_var_cov = factor_model$trans_error_var_cov,
#'                                    trans_var_coeff = factor_model$trans_var_coeff, 
#'                                    factor_lag_order = factor_lag_order, 
#'                                    fcast_horizon = 5, decorr_errors = TRUE,  
#'                                    comp_null = 1e-15, parallel = FALSE,  jitter = 1e-8)
#' print(filter_fit)
#' filter_plots <- plot(filter_fit)
#' filter_plots$`Factor Time Series Plots`
#' 
#' @export
kalmanFilterSmoother <- function(data,
                                 delay, 
                                 no_of_factors,
                                 loading_matrix,
                                 meas_error_var_cov,
                                 trans_error_var_cov,
                                 trans_var_coeff, 
                                 factor_lag_order,
                                 fcast_horizon = 0, 
                                 decorr_errors = TRUE, 
                                 comp_null = 1e-15, 
                                 parallel = FALSE, 
                                 jitter = 1e-8
) {
  func_call <- match.call()
  
  # Misshandling of the data matrix
  if(!is.zoo(data) && !is.xts(data)){
    data_r <- try(t(as.matrix(data)), silent = TRUE)
    if (inherits(data_r, "try-error")) {
      stop(paste0("data must be a matrix, convertible to a matrix or a time-series/zoo object"))
    }
  }else{
    data_r <- try(coredata(data), silent = TRUE)
    if (inherits(data_r, "try-error")) {
      stop(paste0("data must be a matrix, convertible to a matrix or a time-series/zoo object"))
    }
  }
  if(!is.numeric(data_r)){
    stop(paste0("data has non-numeric elements."))
  }
  if(any(is.infinite(data_r))){
    stop(paste0("data cannot have (-)Inf values."))
  }
  data_r[is.na(data_r)] <- 0 # Override R NAs as they seem to not get properly parsed to C++
  
  # Mishandling of delay
  no_of_vars <- dim(data_r)[2]
  no_of_obs <- dim(data_r)[1]
  if(is.null(delay)){
    delay <- matrix(rep(0, no_of_vars), ncol = 1)
  }else{
    delay <- checkPositiveSignedParameterVector(delay, "delay", no_of_vars)
  }
  
  # Check for NAs in the dataset outside the ragged edges
  na_ind <- FALSE
  for(col in 1:dim(data_r)[2]){
    na_ind <- any(is.na(data_r[1:(no_of_obs - delay[col]), col]))
  }
  if(na_ind){
    warning(paste0("data has NA values. AccordingThe corresponding time points will not be considered in th estimation of the loading matrix.")) 
  }
  
  # Mishandling of trans_error_var_cov
  trans_error_var_cov <- try(as.matrix(trans_error_var_cov), silent = TRUE)
  if (inherits(trans_error_var_cov, "try-error")) {
    stop(paste0("trans_error_var_cov must be a matrix or convertible to a matrix object"))
  }
  if(dim(trans_error_var_cov)[1] != dim(trans_error_var_cov)[2]){
    stop(paste0("trans_error_var_cov must be square."))
  }
  if(dim(trans_error_var_cov)[1] != no_of_factors){
    stop(paste0("trans_error_var_cov must be of dimensions no_of_factors x no_of_factors = ", no_of_factors, "x", no_of_factors))
  }
  if(!all.equal(trans_error_var_cov, t(trans_error_var_cov), tolerance = comp_null)){
    stop(paste0("trans_error_var_cov is not symmetric at tolerance level comp_null = ", comp_null, "."))
  }
  
  # Mishandling of meas_error_var_cov
  meas_error_var_cov <- try(as.matrix(meas_error_var_cov), silent = TRUE)
  if (inherits(meas_error_var_cov, "try-error")) {
    stop(paste0("meas_error_var_cov must be a matrix or convertible to a matrix object"))
  }
  if(dim(meas_error_var_cov)[1] != dim(meas_error_var_cov)[2]){
    stop(paste0("meas_error_var_cov must be square."))
  }
  if(dim(meas_error_var_cov)[1] != no_of_vars){
    stop(paste0("meas_error_var_cov must be of dimensions no_of_vars x no_of_vars = ", no_of_vars, "x", no_of_vars))
  }
  if(!all.equal(meas_error_var_cov, t(meas_error_var_cov), tolerance = comp_null)){
    stop(paste0("meas_error_var_cov is not symmetric at tolerance level comp_null = ", comp_null, "."))
  }
  
  #M´Mishandling of loading_matrix
  loading_matrix <- try(as.matrix(loading_matrix), silent = TRUE)
  if (inherits(loading_matrix, "try-error")) {
    stop(paste0("loading_matrix must be a matrix or convertible to a matrix object"))
  }
  if(dim(loading_matrix)[1] != no_of_vars || dim(loading_matrix)[2] != no_of_factors){
    stop(paste0("loading_matrix must be of dimensions no_of_vars x no_of_factors = ", no_of_vars, "x", no_of_factors))
  }
  
  # Mishandling of trans_var_coeff
  coeff_mat_parsing_error_txt <- "trans_var_coeff must be a matrix, convertible to a matrix object, or a list of no_of_factors x no_of_factors matrices of factor_lag_order"
  if(is.list(trans_var_coeff) && !is.object(trans_var_coeff)){
    if(length(trans_var_coeff) != factor_lag_order){
      stop(paste0(coeff_mat_parsing_error_txt))
    }
    for(i in 1:length(trans_var_coeff)){
      trans_var_coeff[[i]] <- try(as.matrix(trans_var_coeff[[i]]), silent = TRUE)
      if (inherits(trans_var_coeff[[i]], "try-error")) {
        stop(paste0(coeff_mat_parsing_error_txt))
      }
      if(dim(trans_var_coeff[[i]])[1] != no_of_factors ||
         dim(trans_var_coeff[[i]])[2] != no_of_factors){
        stop(paste0(coeff_mat_parsing_error_txt))
      }
    }
  }else{
    trans_var_coeff <- try(as.matrix(trans_var_coeff), silent = TRUE)
    if (inherits(trans_var_coeff, "try-error")) {
      stop(paste0(coeff_mat_parsing_error_txt))
    }
    if(dim(trans_var_coeff)[1] != no_of_factors || dim(trans_var_coeff)[2] != no_of_factors * factor_lag_order){
      stop(paste0("loading_matrix must be of dimensions no_of_factors x (no_of_factors * factor_lag_order) = ", no_of_factors, "x", no_of_factors * factor_lag_order))
    }
  }
  
  # Mishandling of number of factors
  no_of_factors <- checkPositiveSignedInteger(no_of_factors, "no_of_factors")
  if(no_of_factors == 0){
    stop("no_of_factors must be strictly positive.")
  }
  if(no_of_factors > no_of_vars){
    stop(paste0("no_of_factors must be smaller than no_of_vars."))
  }
  
  # Mishandling of number of factor_lag_order
  factor_lag_order <- checkPositiveSignedInteger(factor_lag_order, "factor_lag_order")
  if(factor_lag_order < 0){
    stop(paste0("factor_lag_order must be non-negative."))
  }
  
  # Mishandling of number of factor_lag_order
  fcast_horizon <- checkPositiveSignedInteger(fcast_horizon, "fcast_horizon")
  if(fcast_horizon < 0){
    stop(paste0("fcast_horizon must be non-negative."))
  }
  
  # Mishandling of check_rank
  decorr_errors <- checkBoolean(decorr_errors, "decorr_errors")
  
  # Mishandling of number of comp_null
  comp_null <- checkPositiveDouble(comp_null, "comp_null")
  if(comp_null < 0){
    stop(paste0("comp_null must be non-negative."))
  }
  
  # Mishandling of check_rank
  parallel <- checkBoolean(parallel, "parallel")
  
  # Mishandling of number of comp_null
  jitter <- checkPositiveDouble(jitter, "jitter")
  if(jitter < 0){
    stop(paste0("jitter must be non-negative."))
  }
  
  result <- runUVKFS(X_in = data_r,
                     delay = delay, 
                     state_var_cov = trans_error_var_cov,
                     measurement_var_cov = meas_error_var_cov,
                     loading_matrix = loading_matrix, 
                     factor_var_coefficient_matrices = trans_var_coeff, 
                     R = no_of_factors, 
                     order = factor_lag_order,
                     fcast_horizon = fcast_horizon, 
                     decorr_errors = decorr_errors, 
                     comp_null = comp_null, 
                     parallel = parallel, 
                     jitter = jitter)
  
  # Rename the results
  names(result) <- c("filtered_state_variance", "companion_form_smoothed_factors", 
                     "smoothed_state_variance", "llt_success_code")
  result$data <- data
  result$factor_var_lag_order <- factor_lag_order
  
  # Create the non-companion-form factors and loading matrix
  result$smoothed_factors <- result$companion_form_smoothed_factors[1:no_of_factors, 1:(no_of_obs + fcast_horizon), drop = FALSE]
  rownames(result$smoothed_factors) <- paste0("Factor ", 1:no_of_factors)
  result$loading_matrix_estimate <- result$loading_matrix_estimate[, 1:no_of_factors, drop = FALSE]
  result$data <- data
  no_of_cols <- no_of_obs * no_of_factors
  block_size <- factor_lag_order * no_of_factors
  temp_smoothed_state_variance <- result$smoothed_state_variance
  result$smoothed_state_variance <- matrix(NaN, no_of_factors, no_of_factors * (no_of_obs + fcast_horizon))
  result$smoothed_state_variance[, 1:no_of_factors] <- temp_smoothed_state_variance[1:no_of_factors, 1:no_of_factors]
  for(curr_obs in 2:(no_of_obs + fcast_horizon)){
    block_starting_index <- (curr_obs - 1) * block_size + 1
    block_ending_index <- block_starting_index + no_of_factors - 1
    factor_block_starting_ind <- (curr_obs - 1) * no_of_factors + 1
    factor_block_ending_ind <- factor_block_starting_ind + no_of_factors - 1
    if(curr_obs <= no_of_obs){
      result$smoothed_state_variance[, factor_block_starting_ind:factor_block_ending_ind] <- temp_smoothed_state_variance[1:no_of_factors, block_starting_index:block_ending_index]
    }else{
      result$smoothed_state_variance[, factor_block_starting_ind:factor_block_ending_ind] <- result$filtered_state_variance[1:no_of_factors, block_starting_index:block_ending_index]
    }
  }
  
  # Re-shuffle the results objects to be in a more logical ordering
  result <- result[c("data", "smoothed_factors", "smoothed_state_variance", "factor_var_lag_order", "llt_success_code")]
  
  if(is.zoo(data) || is.xts(data)){ # Also convert factors to time series
    start_vector <- c(year(time(data)[1]), month(time(data)[1]))
    result$smoothed_factors <- as.zoo(ts(t(result$smoothed_factors), start = start_vector, frequency = 12))
    colnames(result$smoothed_factors) <- paste0("Factor ", 1:no_of_factors)
  }
  
  result$call <- func_call
  class(result) <- "KFSFit"
  return(result)
}

#' @name print.KFSFit
#' @title Generic printing function for KFSFit S3 objects
#' @description
#' Print a compact summary of a `KFSFit` object.
#'
#' @param x `KFSFit` object.
#' @param ... Additional parameters.
#'
#' @return No return value, called for side effects.
#'
#' @author
#' Domenic Franjic
#' 
#' @export
print.KFSFit <- function(x, ...) {
  simulated_time_series <- is.zoo(x$smoothed_factors)
  no_of_factors <- ifelse(simulated_time_series, dim(x$smoothed_factors)[2], dim(x$smoothed_factors)[1])
  no_of_obs <- ifelse(simulated_time_series, dim(x$data)[1], dim(x$data)[2])
  cat("Simulated Dynamic Factor Model\n")
  cat("=========================================================================\n")
  cat("No. of Observations                        :", no_of_obs, "\n")
  cat("No. of Factors                             :", no_of_factors, "\n")
  cat("Factor Lag Order                           :", x$factor_var_lag_order, "\n")
  if(x$llt_success_code == -1){
    cat("Info: LLT failed. Used robust LDLT instead.\n")
  }else if(x$llt_success_code == -2){
    cat("Warning: LLT and LDLT failed. Used uncorrelated errors.\n")
  }
  cat("=========================================================================\n")
  cat("Head of the factors :\n")
  max_print <- min(5, no_of_obs)
  if(simulated_time_series){
    print(head(x$smoothed_factors, max_print))
  }else{
    print(x$smoothed_factors[, 1:max_print])
  }
  cat("Tail of the factors :\n")
  if(simulated_time_series){
    print(tail(x$smoothed_factors, max_print))
  }else{
    print(x$smoothed_factors[, (dim(x$smoothed_factors)[2] - (max_print - 1)):(dim(x$smoothed_factors)[2])])
  }
  cat("=========================================================================\n")
  invisible(x)
}

#' @name plot.KFSFit
#' @title Generic plotting function for KFSFit S3 objects
#' @description
#' Create diagnostic plots for a `KFSFit` object.
#'
#' @param x `KFSFit` object.
#' @param axis_text_size Numeric size of x- and y-axis labels. Prased to ggplot2 
#' `theme(..., text = element_text(size = axis_text_size))`.
#' @param legend_title_text_size Numeric size of x- and y-axis labels. Prased to
#' ggplot2 
#' `theme(..., legend.title = element_text(size = legend_title_text_size))`.
#' @param ... Additional parameters for the plotting functions.
#'
#' @return
#' A named list of `patchwork`/`ggplot` objects:
#' \describe{
#'   \item{`Factor Time Series Plots`}{`patchwork`/`ggplot` object graphing the 
#'   estimated factors over time with 95% confidence bands based on the smoother
#'   uncertainty of the Kalman Filter and Smoother.}
#' }
#'
#' @author
#' Domenic Franjic
#' 
#' @export
plot.KFSFit <- function(x,
                        axis_text_size = 20, 
                        legend_title_text_size = 20, 
                        ...) {
  out_list <- list()
  if(is.zoo(x$data)){
    series_names <- colnames(x$data)
    no_of_factors <- dim(x$smoothed_factors)[2]
    time_vector <- as.Date(time(x$smoothed_factors))
    factors <- x$smoothed_factors
  }else{
    series_names <- rownames(x$data)
    no_of_factors <- dim(x$smoothed_factors)[1]
    time_vector <- 1:dim(x$smoothed_factors)[2]
    factors <- t(x$smoothed_factors)
    factors <- as.zoo(ts(factors, start = c(1, 1), frequency = 12))
  }

  out_list$`Factor Time Series Plots` <- plotFactorEstimates(factors, x$smoothed_state_variance,
                                                             no_of_factors, axis_text_size)
  
  return(out_list)
}

#' @useDynLib TwoStepSDFM, .registration=TRUE
#' @importFrom Rcpp sourceCpp
#' @import zoo
#' @import xts
#' @import lubridate
#' @import ggplot2
#' @import stats
#' @import utils
NULL

# SPDX-License-Identifier: GPL-3.0-or-later
#
#  Copyright (C) 2024-2026 Domenic Franjic
#
#  This file is part of TwoStepSDFM.
#
#  TwoStepSDFM is free software: you can redistribute
#  it and/or modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TwoStepSDFM is distributed in the hope that it
#  will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with TwoStepSDFM. If not, see <https://www.gnu.org/licenses/>.

#' @name forecastWrapper
#' @title Internal forecasting wrapper function
#' @description
#' This function is for internal use only and may change in future releases
#' without notice. Users should use `nowcast()` instead for a stable and
#' supported interface.
#' Helper function to check parameter vectors for positive signed integer values
#' @keywords internal
forecastWrapper <- function(target_variables,
                            quarterly_predictors,
                            factors,
                            target_variable_delay,
                            quarterly_delay,
                            lag_estim_criterion,
                            max_fcast_horizon,
                            max_ar_lag_order,
                            max_predictor_lag_order,
                            jitter
)
{
  no_of_factors <- dim(factors)[1]
  no_of_target_vars <- dim(target_variables)[1]
  qtrly_predictors_missing <- is.null(quarterly_predictors)
  if(qtrly_predictors_missing){
    no_of_qrtly_vars <- 0
  }else{
    no_of_qrtly_vars <- dim(quarterly_predictors)[1]
  }
  no_of_vars  <- no_of_qrtly_vars + no_of_target_vars
  no_of_qrtly_obs <- (dim(factors)[2] - 2) / 3
  min_fcast_horizons <- ifelse(target_variable_delay == 0, 
                               0,
                               -floor(target_variable_delay / 3) + 1)
  return_object <- list()
  
  # Store all monthly predictors and the monthly factors in a single matrix
  all_qtrly_data_delay <- c(floor(c(target_variable_delay, quarterly_delay) / 3), rep(0, no_of_factors))
  
  # Start quarterfication loop according to Mariano and Murasawa #
  
  # Note: It is implicitly assumed that the stationary monthly data starts at the 
  #   second month of the first quarter. Further, it is assumed that the data set
  #   ends with an observation in the last month of the quarter. This is handled 
  #   by the higher level function calling this wrapper.
  
  all_qrtly_data <- matrix(NaN, no_of_vars + no_of_factors, no_of_qrtly_obs)
  for(t in seq(5, dim(factors)[2], 3)){
    all_qrtly_data[1:no_of_target_vars, (t - 2)/3] <- target_variables[, t]
    if(!qtrly_predictors_missing){
      all_qrtly_data[(no_of_target_vars + 1):(no_of_target_vars + no_of_qrtly_vars), 
                     (t - 2)/3] <- quarterly_predictors[, t]
    }
    all_qrtly_data[(no_of_target_vars + no_of_qrtly_vars + 1):(no_of_vars + no_of_factors), 
                   (t - 2)/3] <- rowSums(cbind(1/3 * factors[, t, drop = FALSE],
                                               2/3 * factors[, t - 1, drop = FALSE],
                                               1 * factors[, t - 2, drop = FALSE],
                                               2/3 * factors[, t - 3, drop = FALSE],
                                               1/3 * factors[, t - 4, drop = FALSE]),
                                         na.rm = TRUE)
  }
  
  # End quarterfication loop according to Mariano and Murasawa #
  
  # Start ARDL estimation loop over the target variables #
  
  fcasts <- matrix(NaN, no_of_target_vars, max_fcast_horizon - min(min_fcast_horizons) + 1)
  for(current_target in 1:no_of_target_vars){
    
    # Start ARDL estimation loop over the predictor #
    
    current_fcasts <- matrix(NaN, no_of_vars + no_of_factors, max_fcast_horizon - min_fcast_horizons[current_target] + 1)
    for(current_predictor in 1:(no_of_vars + no_of_factors)){
      if(current_target == current_predictor){
        next # Skip using the current target_variable as a single predictor as its always included as predictor
      }      
      if(all_qtrly_data_delay[current_target] < all_qtrly_data_delay[current_predictor]){
        next # Skip a predictor if it is dalyed further back compared to the target variable (we do not expect forecasting gains from using variables that are further behind then the target)
      }
      
      rel_fcast_horizons <- min_fcast_horizons[current_target]:max_fcast_horizon + all_qtrly_data_delay[current_predictor]
      for(h in rel_fcast_horizons){
        
        # Fit the model for the specific forecasting horizon
        horizon_adjustment <- which(rel_fcast_horizons == h)
        
        horizon_specific_target <- matrix(
          all_qrtly_data[current_target, 
                         (horizon_adjustment + 1):(no_of_qrtly_obs - all_qtrly_data_delay[current_target])],
          ncol = 1)
        
        horizon_specific_ar_lag <- matrix(
          all_qrtly_data[current_target, 
                         1:(no_of_qrtly_obs - all_qtrly_data_delay[current_target] - horizon_adjustment)],
          ncol = 1)
        
        horizon_specific_predictor <- matrix(
          all_qrtly_data[current_predictor, 
                         (horizon_adjustment + 1 - h):(no_of_qrtly_obs - all_qtrly_data_delay[current_target] - h)],
          ncol = 1)
        
        if(max_ar_lag_order != 0){
          ardl_fit <- runARDL(horizon_specific_target,
                              horizon_specific_ar_lag,
                              horizon_specific_predictor,
                              max(max_ar_lag_order - max(h, 0), 1), 
                              max(max_predictor_lag_order - max(h, 0), 1),
                              lag_estim_criterion,
                              jitter)
          
          # Forecast
          forecast_predictors <- matrix(1, sum(ardl_fit$optimL_lag_order) + 3, 1) # Add three for the intercept and the "contemporaenous" observations
          
          forecast_predictors[2:(ardl_fit$optimL_lag_order[1] + 2), ] <- 
            head(all_qrtly_data[current_target, 
                                (no_of_qrtly_obs - all_qtrly_data_delay[current_target]):1],
                 ardl_fit$optimL_lag_order[1] + 1)
          
          forecast_predictors[(ardl_fit$optimL_lag_order[1] + 3):(ardl_fit$optimL_lag_order[1] + ardl_fit$optimL_lag_order[2] + 3), ] <- 
            head(all_qrtly_data[current_predictor, 
                                (no_of_qrtly_obs - all_qtrly_data_delay[current_predictor]):1],
                 ardl_fit$optimL_lag_order[2] + 1)
          
          current_fcasts[current_predictor, which(rel_fcast_horizons == h)] <-
            matrix(ardl_fit$coefficients, nrow = 1) %*% forecast_predictors
        }else{
          ardl_fit <- runDL(horizon_specific_target,
                            horizon_specific_predictor,
                            max(max_predictor_lag_order - max(h, 0), 1),
                            lag_estim_criterion,
                            jitter = jitter)
          
          # Forecast
          forecast_predictors <- matrix(1, ardl_fit$optimL_lag_order + 2, 1) # Add two for the intercept and the "contemporaenous" observations
          
          forecast_predictors[2:(ardl_fit$optimL_lag_order[1] + 2), ] <- 
            head(all_qrtly_data[current_predictor, 
                                (no_of_qrtly_obs - all_qtrly_data_delay[current_predictor]):1],
                 ardl_fit$optimL_lag_order + 1)
          
          current_fcasts[current_predictor, which(rel_fcast_horizons == h)] <-
            matrix(ardl_fit$coefficients, nrow = 1) %*% forecast_predictors
        }
        
      }
      
      # End loop over the forecasting horizons #
      
    }
    
    # Store the final point forecast using simple forecast averaging for each target
    if(qtrly_predictors_missing){
      rownames(current_fcasts) <- c(rownames(target_variables), rownames(factors))
    }else{
      rownames(current_fcasts) <- c(rownames(target_variables), rownames(quarterly_predictors), rownames(factors))
    }
    return_object[[current_target]] <- current_fcasts
    names(return_object)[current_target] <- paste0("Single Predictor Forecasts ", rownames(target_variables)[current_target], collapse = "")
    fcasts[current_target, (max_fcast_horizon - min(min_fcast_horizons) - length(rel_fcast_horizons) + 2):(max_fcast_horizon - min(min_fcast_horizons) + 1)] <-
      colMeans(current_fcasts, na.rm = TRUE)
    
    # Start ARDL estimation loop over the predictor #
    
  }
  
  # End ARDL estimation loop over the target variables #
  
  rownames(fcasts) <- rownames(target_variables)
  return_object[[no_of_target_vars + 1]] <- fcasts
  names(return_object)[no_of_target_vars + 1] <- "Avg. Point Forecast"
  
  return(return_object)
  
}

#' @useDynLib TwoStepSDFM, .registration=TRUE
#' @importFrom Rcpp sourceCpp
#' @import zoo
#' @import xts
#' @import lubridate
#' @import ggplot2
#' @import stats
#' @import utils
NULL

# SPDX-License-Identifier: GPL-3.0-or-later
#
#  Copyright (C) 2024-2026 Domenic Franjic
#
#  This file is part of TwoStepSDFM.
#
#  TwoStepSDFM is free software: you can redistribute
#  it and/or modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TwoStepSDFM is distributed in the hope that it
#  will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with TwoStepSDFM. If not, see <https://www.gnu.org/licenses/>.

#' Helper function to check unsigned integer function parameter
#' @keywords internal
checkPositiveSignedInteger <- function(parameter, name, bit_size = 64) {
  if(!is.numeric(parameter)){
    stop(paste0(name, " is not numeric."))
  }
  if(length(parameter) != 1 || is.na(parameter)) {
    stop(paste0(name, " must be a single, non-NA numeric value."))
  }
  if(is.infinite(parameter)){
    stop(paste0(name, " cannot be Inf."))
  }
  if(parameter %% 1 != 0){
    warning(paste0(name, " is not an integer. It will be truncated before further use."))
    parameter <- floor(parameter)
  }
  if(parameter < 0 || parameter > 2^(bit_size - 1) - 1) {
    stop(paste0(name, " must be a non-negative signed 64bit integer.")) 
  }
  
  return(parameter)
}

#' Helper function to check parameter matrices
#' @keywords internal
checkParameterMatrix <- function(parameter, name, no_of_rows, no_of_cols){
  parameter <- try(as.matrix(parameter), silent = TRUE)
  if(inherits(parameter, "try-error")) {
    stop(paste0(name, " must be a matrix or convertible to a matrix."))
  }
  if(!is.numeric(parameter)){
    stop(paste0(name, " has non-numeric elements."))
  }
  if (dim(parameter)[1] != no_of_rows || dim(parameter)[2] != no_of_cols) {
    stop(paste0(name, " must be of dimensions (", no_of_rows, "x", no_of_cols, ") but is (", dim(parameter)[1], "x", dim(parameter)[2], ")."))
  }
  if(any(is.infinite(parameter))){
    stop(paste0(name, " cannot have (-)Inf values."))
  }
  if(any(is.na(parameter))){
    stop(paste0(name, " has NA values."))
  }

  return(parameter)
}

#' Helper function to check parameter vectors for positive signed integer values
#' @keywords internal
checkPositiveSignedParameterVector <- function(parameter, name, size){
  parameter <- try(as.matrix(parameter), silent = TRUE)
  if(inherits(parameter, "try-error")) {
    stop(paste0(name, " must be a matrix or convertible to a matrix"))
  }
  if(all(dim(parameter) == c(1, size))) {
    parameter <- t(parameter)
  }
  if(any(dim(parameter) != c(size, 1))) {
    stop(paste0(name, " must be a vector of a matrix of dimensions (", size, 
                "x1) or (1x", size, "). The provided matrix is (", dim(parameter)[1], 
                "x", dim(parameter)[2], ")."))
  }
  if(!is.numeric(parameter)){
    stop(paste0(name, " has non-numeric elements."))
  }
  if(any(is.na(parameter))){
    stop(paste0(name, " has NA values."))
  }
  if(any(parameter %% 1 != 0)) {
    warning(paste0("At least one element of ", name, " is not an integer. It will be truncated before further use."))
    parameter <- floor(parameter)
  }
  if(any(parameter < 0) || any(parameter > 2^63 - 1)) {
    stop(paste0(name, " must be a non-negative signed 64bit integer."))
  }

  return(parameter)
}

#' Helper function to check parameter vectors for positive signed integer values
#' @keywords internal
checkBoolean <- function(parameter, name){
  if(is.null(parameter))
  {
    stop(paste0(name, " must be a single, non-NA boolean value."))
  }
  if(!is.logical(parameter) && !(parameter %in% c(0, 1))){
    stop(paste0(name, " must be a single, non-NA boolean value."))
  }
  if (length(parameter) != 1 || is.na(parameter)) {
    stop(paste0(name, " must be a single, non-NA boolean value."))
  }
  
  return(as.logical(parameter))
}

#' Helper function to check positive double function parameter
#' @keywords internal
checkPositiveDouble <- function(parameter, name) {
  if(!is.numeric(parameter)){
    stop(paste0(name, " is not numeric."))
  }
  if(is.infinite(parameter)){
    stop(paste0(name, " cannot be (-)Inf."))
  }
  if (length(parameter) != 1 || is.na(parameter)) {
    stop(paste0(name, " must be a single, non-NA numeric value."))
  }
  if (parameter < 0) {
    stop(paste0(name, " must be non-negative."))
  }
  
  return(parameter)
}

#' Helper function to check positive double function parameter
#' @keywords internal
makeRaggedEdges <- function(data, delay){
no_of_observations <- dim(data)[1]
no_of_variables <- dim(data)[2]
for(n in 1:no_of_variables){
  if(delay[n] == 0){
    next
  }
  data[(no_of_observations + 1 - delay[n]):(no_of_observations), n] <- NaN
}
return(data)
}

#' @useDynLib TwoStepSDFM, .registration=TRUE
#' @importFrom Rcpp sourceCpp
#' @import zoo
#' @import xts
#' @import lubridate
#' @import ggplot2
#' @import stats
#' @import utils
NULL

# SPDX-License-Identifier: GPL-3.0-or-later
#
#  Copyright (C) 2024-2026 Domenic Franjic
#
#  This file is part of TwoStepSDFM.
#
#  TwoStepSDFM is free software: you can redistribute
#  it and/or modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TwoStepSDFM is distributed in the hope that it
#  will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with TwoStepSDFM. If not, see <https://www.gnu.org/licenses/>.

#' Helper for plotting factor time series
#' @keywords internal
plotFactorEstimates <- function(factors, smoothed_state_variance, no_of_factors,
                                axis_text_size) {
  pot_list_factors <- list()
  seq_along_dates <- 1:length(as.Date(time(factors)))
  for(factor in 1:no_of_factors){
    correction_factor <- 1.96 * sqrt(pmax(smoothed_state_variance[factor, seq(1, length(as.Date(time(factors))) * no_of_factors, by = no_of_factors) + (factor - 1)], 1e-15))
    current_factor <- data.frame(
      Date = as.Date(time(factors)),
      Value = factors[seq_along_dates, factor],
      `Upper 95%-CI` = factors[seq_along_dates, factor] + correction_factor,
      `Lower 95%-CI` = factors[seq_along_dates, factor] - correction_factor,
      check.names = FALSE
    )
    
    current_line_plot <- ggplot(current_factor, aes(x = Date, y = Value)) +
      geom_ribbon(aes(ymin = `Lower 95%-CI`, ymax = `Upper 95%-CI`), fill = "#88ccee", alpha = 0.4) +
      geom_line(colour = "black") +
      labs(title = paste0("Factor ", factor), y = "") +
      theme_minimal() + 
      theme(axis.text.x = element_text(angle = 0, vjust = 1, hjust = 1),
            text = element_text(size = axis_text_size))
    
    pot_list_factors[[factor]] <- current_line_plot
  }
  
  return( patchwork::wrap_plots(pot_list_factors, ncol = 1))
}

#' Helper for plotting loading matrix heat maps
#' @keywords internal
plotLoadingHeatMap <- function(loading_matrix_estim, series_names, no_of_factors, axis_text_size,
                                legend_title_text_size) {
  lambda_df <- as.data.frame(loading_matrix_estim)
  colnames(lambda_df) <- paste0("Factor ", 1:no_of_factors)
  if(dim(loading_matrix_estim)[2] == 1){
    stacked_loadings <- lambda_df
    stacked_loadings$Factor <- "Factor 1"
  }else{
    stacked_loadings <- stack(lambda_df[, ]) 
  }
  colnames(stacked_loadings) <- c("Loading", "Factor")
  stacked_loadings$Variable <- factor(rep(series_names, no_of_factors), levels = rev(series_names))
  heat_map_plot <- ggplot(stacked_loadings, aes(x = Factor, y = Variable)) +
    geom_tile(data = stacked_loadings, aes(fill = Loading), width = 0.9, height = 0.8) +
    geom_tile(data = subset(stacked_loadings, Loading == 0), fill = "black", width = 0.9, height = 0.8) +
    scale_fill_gradient2(low = "#88ccee", high = "#117733", na.value = "#882255", mid = "#FFFFFF") +
    scale_x_discrete(expand = c(0, 0)) +
    theme_minimal() +
    theme(axis.text.x = element_text(angle = 90, vjust = 1, hjust = 1),
          text = element_text(size = axis_text_size),
          legend.title = element_text(size = legend_title_text_size),
          strip.text.y = element_blank(),
          panel.spacing = unit(0.01, "lines"),
          panel.grid.major = element_blank(), 
          panel.grid.minor = element_blank())
  
  return(heat_map_plot)
}

#' Helper for plotting measurement error var.cov. heatmap
#' @keywords internal
plotMeasVarCovHeatmap <- function(measurement_error_var_cov_df, series_names, axis_text_size,
                               legend_title_text_size) {
  colnames(measurement_error_var_cov_df) <- series_names
  stacked_measurement_error_var_cov <- stack(measurement_error_var_cov_df[, series_names])
  colnames(stacked_measurement_error_var_cov) <- c("(Co-)Variance", "Variable")
  stacked_measurement_error_var_cov$`(Co-)Variable` <- factor(rep(series_names, length(series_names)), levels = rev(series_names))
  heat_map_plot <- 
    ggplot(stacked_measurement_error_var_cov, aes(x = Variable, y = `(Co-)Variable`)) +
    geom_tile(data = stacked_measurement_error_var_cov, aes(fill = `(Co-)Variance`), width = 0.8, height = 0.8) +
    scale_fill_gradient2(low = "#88ccee", high = "#117733", na.value = "#882255", mid = "#FFFFFF") +
    scale_x_discrete(expand = c(0, 0)) +
    theme_minimal() +
    theme(axis.text.x = element_text(angle = 90, vjust = 1, hjust = 1),
          text = element_text(size = axis_text_size),
          legend.title = element_text(size = legend_title_text_size),
          strip.text.y = element_blank(),
          panel.spacing = unit(0.01, "lines"),
          panel.grid.major = element_blank(), 
          panel.grid.minor = element_blank())
  
  return(heat_map_plot)
}

#' Helper for plotting measurement error var.cov. eigenvalues
#' @keywords internal
plotMeasVarCovEigenvalues <- function(eigen_values, no_of_factors, axis_text_size,
                                  legend_title_text_size) {
  eig_val_df <- data.frame("Value" = eigen_values,
                           "cut_off" = "normal",
                           "Eigen Value" = paste0("E.V. ", 1:length(eigen_values)),
                           check.names = FALSE)
  eig_val_df$`Eigen Value` <- factor(eig_val_df$`Eigen Value`, levels = eig_val_df$`Eigen Value`) 
  eig_val_df$cut_off[no_of_factors] <- "highlight"
  eig_val_plot <- ggplot(eig_val_df, aes(x = `Eigen Value`, y = Value, , fill = cut_off)) +
    geom_col() +
    geom_text(data = eig_val_df[no_of_factors, , drop = FALSE], 
              aes(x = `Eigen Value`, y = 0, label = "No. of factors chosen"),
              angle = 90, vjust = 0.5, hjust = 0, color = "white", size = axis_text_size * 0.33,
              inherit.aes = FALSE) + 
    scale_fill_manual(values = c(normal   = "#88ccee", highlight = "#117733")) +
    theme_minimal() +
    theme(axis.text.x = element_text(angle = 90, vjust = 1, hjust = 1),
          text = element_text(size = axis_text_size),
          legend.position = "none")
  
  return(eig_val_plot)
}

#' Helper for plotting ICs for selecting the number of factors
#' @keywords internal
plotInformationCrit <- function(information_crit, no_of_factors, axis_text_size){
  data <- data.frame("no_of_factors" = 1:length(information_crit),
                     "information_crit" = information_crit)
  ggplot(data, aes(x = no_of_factors, y = information_crit)) +
    geom_point(size = 5, colour = "#88ccee") +
    geom_point(x = no_of_factors, y = min(information_crit), color = "#117733", size = 5) +
    labs(x = "Number of Factors", y = "BIC") +
    theme_minimal() + 
    scale_x_continuous(breaks = 1:length(information_crit)) +
    theme(
      axis.title = element_text(size = axis_text_size),
      axis.text  = element_text(size = axis_text_size)
    )
}

#' @useDynLib TwoStepSDFM, .registration=TRUE
#' @importFrom Rcpp sourceCpp
#' @importFrom Rdpack reprompt
#' @import zoo
#' @import xts
#' @import lubridate
#' @import ggplot2
#' @import stats
#' @import utils
NULL

# SPDX-License-Identifier: GPL-3.0-or-later
#
#  Copyright (C) 2024-2026 Domenic Franjic
#
#  This file is part of TwoStepSDFM.
#
#  TwoStepSDFM is free software: you can redistribute
#  it and/or modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TwoStepSDFM is distributed in the hope that it
#  will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with TwoStepSDFM. If not, see <https://www.gnu.org/licenses/>.

#' @name twoStepDenseDFM
#' @title Two Step Dense Dynamic Factor Model Estimator.
#' @description
#' Estimate a dense dynamic factor model with measurement equation
#' \deqn{
#'   \bm{x}_t = \bm{\Lambda} \bm{f}_{t} + \bm{\xi}_t,\quad \bm{\xi}_t \sim \mathcal{N}(\bm{0}, \bm{\Sigma}_{\xi}),
#' }
#' and transition equation
#' \deqn{
#'   \bm{f}_t = \sum_{p=0}^P\bm{\Phi}_p \bm{f}_{t-p} + \bm{\epsilon}_t,\quad \bm{\epsilon}_t \sim \mathcal{N}(\bm{0}, \bm{\Sigma}_{f}).
#' }
#' using principal components  analysis and the Kalman Filter and 
#' Smoother according to \insertRef{Giannone2008Nowcasting}{TwoStepSDFM} and
#' \insertRef{Doz2011Two_step}{TwoStepSDFM}.
#' 
#' @param data Numeric (no_of_vars \eqn{\times}{x} no_of_obs) matrix of data or 
#' zoo/xts object sampled at the same frequency.
#' @param delay Integer vector of variable delays.
#' @param no_of_factors Integer number of factors.
#' @param max_factor_lag_order Integer maximum order of the VAR process in the 
#' transition equation.
#' @param lag_estim_criterion Information criterion used for the estimation of 
#' the factor VAR order (`"BIC"` (default), `"AIC"`, `"HIC"`).
#' @param decorr_errors Logical, whether or not the errors should be 
#' decorrelated.
#' @param comp_null Numeric computational zero.
#' @param parallel Logical, whether or not to use Eigen's internal parallel 
#' matrix operations.
#' @param fcast_horizon Integer number of additional Filter predictions into the 
#' future.
#' @param jitter Numerical jitter for stability of internal solver algorithms. 
#' The jitter is added to the diagonal entries of the variance covariance matrix 
#' of the measurement errors.
#' 
#' @details
#' The function performs a two-step estimation procedure for dense dynamic 
#' factor models as described in \insertRef{Giannone2008Nowcasting}{TwoStepSDFM}
#' and \insertRef{Doz2011Two_step}{TwoStepSDFM}. In the first step, the factor 
#' loading matrix is estimated using PCA. In the second step the latent factors
#' are estimated using the univariate representation of the Kalman Filter and
#' Smoother \insertCite{koopman2000fast}{TwoStepSDFM}.
#' 
#' With respect to the univariate representation of the Kalman filter and 
#' smoother, `decorr_errors` indicates whether the data should be decorrelated 
#' internally prior to filtering and smoothing. `jitter` is added to the 
#' diagonal elements of the measurement variance–covariance matrix. For more 
#' details, see \code{\link{kalmanFilterSmoother}}.
#' 
#' @return 
#' An object of class SDFMFit with main components:
#' \describe{
#'   \item{data}{Original data object.}
#'   \item{loading_matrix_estim}{Numeric matrix of estimated factor loadings.}
#'   \item{smoothed_factors}{Object containing the SPCA factor estimates. The 
#'   object inherits its class from data: If data is provided as `zoo`, 
#'   `factor_estim` will be a `zoo` object. If data is provided as matrix, 
#'   `factor_estim` will be a (`no_of_factors`\eqn{\times}{x}`no_of_obs` 
#'   matrix.}
#'   \item{smoothed_state_variance}{(`no_of_factors`\eqn{\times}{x}(
#'   `no_of_factors` * `no_of_obs`)) matrix, where each (`no_of_factors`
#'   \eqn{\times}{x}`no_of_factors`) block represents the smoother uncertainty 
#'   at time point\eqn{t}{t}.}
#'   \item{factor_var_lag_order}{Integer order of the VAR process in the state 
#'   equation.}
#'   \item{error_var_cov_cholesky_factor}{Numeric lower-triangular Cholesky 
#'   factor of the estimated measurement error variance–covariance matrix.}
#'   \item{llt_success_code}{Integer indicating the status of the Cholesky 
#'   factorization: `0` = LLT succeeded, `-1` = LLT failed but LDLT succeeded, 
#'   `-2` = both failed and errors are treated as uncorrelated.}
#' }
#' 
#' @author
#' Domenic Franjic
#' 
#' @references
#' \insertRef{koopman2000fast}{TwoStepSDFM}
#' 
#' \insertRef{Giannone2008Nowcasting}{TwoStepSDFM}
#' 
#' \insertRef{eigenweb}{TwoStepSDFM}
#' 
#' \insertRef{Doz2011Two_step}{TwoStepSDFM}
#' 
#' @seealso
#' \code{\link{sparsePCA}}: Routine for fitting estimating a sparse factor 
#' loading matrix.
#'  
#' \code{\link{kalmanFilterSmoother}}: Routine for filtering and smoothing 
#' latent factors.
#' 
#' \code{\link{twoStepSDFM}}: Two-step estimation routine for a sparse dynamic 
#' factor model.
#' 
#' @examples
#' data(factor_model)
#' no_of_vars <- dim(factor_model$data)[2]
#' no_of_factors <- dim(factor_model$factors)[2]
#' dfm_fit <- twoStepDenseDFM(data = factor_model$data, delay = factor_model$delay, 
#'                            no_of_factors = no_of_factors)
#' print(dfm_fit)
#' dfm_plots <- plot(dfm_fit)
#' dfm_plots$`Factor Time Series Plots`
#' dfm_plots$`Loading Matrix Heatmap`
#' dfm_plots$`Meas. Error Var.-Cov. Matrix Heatmap`
#' dfm_plots$`Meas. Error Var.-Cov. Eigenvalue Plot`
#' 
#' @export
twoStepDenseDFM <- function (data, 
                             delay, 
                             no_of_factors, 
                             max_factor_lag_order = 10, 
                             lag_estim_criterion = "BIC", 
                             decorr_errors = TRUE, 
                             comp_null = 1e-15, 
                             parallel = FALSE, 
                             fcast_horizon = 0, 
                             jitter = 1e-08) {
  func_call <- match.call()
  
  # Mishandling of data
  if (!is.zoo(data) && !is.xts(data)) {
    data_r <- try(t(as.matrix(data)), silent = TRUE)
    if (inherits(data_r, "try-error")) {
      stop(paste0("data must be a matrix, convertible to a matrix or a time-series/zoo object"))
    }
  }
  else {
    data_r <- try(coredata(data), silent = TRUE)
    if (inherits(data_r, "try-error")) {
      stop(paste0("data must be a matrix, convertible to a matrix or a time-series/zoo object"))
    }
  }
  if (!is.numeric(data_r)) {
    stop(paste0("data has non-numeric elements."))
  }
  if (any(is.infinite(data_r))) {
    stop(paste0("data cannot have (-)Inf values."))
  }
  no_of_variables <- dim(data_r)[2]
  no_of_observations <- dim(data_r)[1]
  
  # Mishandling of delay
  if (is.null(delay)) {
    delay <- matrix(rep(0, no_of_variables), ncol = 1)
  }
  else {
    delay <- checkPositiveSignedParameterVector(delay, "delay", no_of_variables)
  }
  na_ind <- FALSE
  for (col in 1:dim(data_r)[2]) {
    na_ind <- any(is.na(data_r[1:(no_of_observations - delay[col]), col]))
    if (na_ind) {
      stop(paste0("data has NA values outside the ragged edges."))
    }
  }
  obs_ind <- FALSE
  for (col in 1:dim(data_r)[2]) {
    if (delay[col] > 0) {
      obs_ind <- !all(is.na(data_r[(no_of_observations - delay[col] + 1):no_of_observations, col]))
    }
    if (obs_ind) {
      stop(paste0("data has observed values inside the ragged edges."))
    }
  }
  
  # Mishandling of dimensions and other misc. parameters
  if (no_of_variables >= no_of_observations) {
    stop(paste0("Too few observations as no_of_variables >= no_of_observations."))
  }
  no_of_factors <- checkPositiveSignedInteger(no_of_factors, "no_of_factors")
  if (no_of_factors == 0) {
    stop("no_of_factors must be strictly positive.")
  }
  if (no_of_factors > no_of_variables) {
    stop(paste0("no_of_factors must be smaller than no_of_variables."))
  }
  max_factor_lag_order <- checkPositiveSignedInteger(max_factor_lag_order, "max_factor_lag_order")
  if (max_factor_lag_order == 0) {
    stop(paste0("max_factor_lag_order must be strictly positive."))
  }
  decorr_errors <- checkBoolean(decorr_errors, "decorr_errors")
  if (is.null(lag_estim_criterion)) {
    stop(paste0("lag_estim_criterion must be either \"BIC\", \"AIC\", or \"HIC\"."))
  }
  if (!(lag_estim_criterion %in% c("AIC", "BIC", "HIC"))) {
    stop(paste0("lag_estim_criterion must be either \"BIC\", \"AIC\", or \"HIC\"."))
  }
  comp_null <- checkPositiveDouble(comp_null, "comp_null")
  if (comp_null == 0) {
    warning("comp_null should not be exactly 0. It will be jittered before further use.")
    comp_null <- 1e-15
  }
  parallel <- checkBoolean(parallel, "parallel")
  fcast_horizon <- checkPositiveSignedInteger(fcast_horizon, "fcast_horizon")
  jitter <- checkPositiveDouble(jitter, "jitter")
  
  # Estimate model parameter and filter latent factors
  result <- runDFMKFS(X_in = data_r, delay = delay, R = as.integer(no_of_factors), 
                      order = as.integer(max_factor_lag_order),  decorr_errors = decorr_errors, 
                      crit = lag_estim_criterion, comp_null = comp_null, 
                      parallel = parallel, fcast_horizon = fcast_horizon, 
                      jitter = jitter)
  
  # Re-name the results
  names(result) <- c("loading_matrix_estimate", "filtered_state_variance", 
                     "companion_form_smoothed_factors", "smoothed_state_variance", 
                     "error_var_cov_cholesky_factor", "factor_var_lag_order", 
                     "llt_success_code")
  
  # Retrieve the factors and loading matrix from the companion forms
  result$smoothed_factors <- result$companion_form_smoothed_factors[1:no_of_factors, 1:(no_of_observations + fcast_horizon), drop = FALSE]
  rownames(result$smoothed_factors) <- paste0("Factor ", 1:no_of_factors)
  result$loading_matrix_estimate <- result$loading_matrix_estimate[, 1:no_of_factors, drop = FALSE]
  
  # Store the data in the return object
  result$data <- data
  
  # Retrieve the correct KFS uncertainty blocks from the companion form
  no_of_cols <- no_of_observations * no_of_factors
  block_size <- result$factor_var_lag_order * no_of_factors
  temp_smoothed_state_variance <- result$smoothed_state_variance
  result$smoothed_state_variance <- matrix(NaN, no_of_factors, no_of_factors * (no_of_observations + fcast_horizon))
  result$smoothed_state_variance[, 1:no_of_factors] <- temp_smoothed_state_variance[1:no_of_factors, 1:no_of_factors]
  for (curr_obs in 2:(no_of_observations + fcast_horizon)) {
    block_starting_index <- (curr_obs - 1) * block_size + 1
    block_ending_index <- block_starting_index + no_of_factors - 1
    factor_block_starting_ind <- (curr_obs - 1) * no_of_factors + 1
    factor_block_ending_ind <- factor_block_starting_ind + no_of_factors - 1
    if (curr_obs <= no_of_observations) {
      result$smoothed_state_variance[, factor_block_starting_ind:factor_block_ending_ind] <- 
        temp_smoothed_state_variance[1:no_of_factors, block_starting_index:block_ending_index]
    }
    else {
      result$smoothed_state_variance[, factor_block_starting_ind:factor_block_ending_ind] <- 
        result$filtered_state_variance[1:no_of_factors, block_starting_index:block_ending_index]
    }
  }
  
  # Re-shuffle the results and cut some of them for logical coherency and debloating
  result <- result[c("data", "loading_matrix_estimate", "smoothed_factors", "smoothed_state_variance", 
                     "factor_var_lag_order", "error_var_cov_cholesky_factor", "llt_success_code")]
  
  # Compute the Cholesky factor as runDFMKFS only returns the inverse of the lower triangular Cholesky factor
  result$error_var_cov_cholesky_factor <- tryCatch({
    solve(result$error_var_cov_cholesky_factor)
  }, error = function(e) {
    return(paste("ERROR:", conditionMessage(e)))
  })
  if (is.matrix(result$error_var_cov_cholesky_factor)) {
    result$error_var_cov_cholesky_factor[upper.tri(result$error_var_cov_cholesky_factor)] <- 0
  }
  
  # Turn the smoothed factors into zoo object if data is a zoo object
  if (is.zoo(data) || is.xts(data)) {
    start_vector <- c(year(time(data)[1]), month(time(data)[1]))
    result$smoothed_factors <- as.zoo(ts(t(result$smoothed_factors), start = start_vector, frequency = 12))
  }
  
  # Collect preliminary objects in the return object
  result$call <- func_call
  result$factor_fcast_horizon <- fcast_horizon
  result$data_delay <- delay
  class(result) <- "SDFMFit"
  return(result)
}

#' @useDynLib TwoStepSDFM, .registration=TRUE
#' @importFrom Rcpp sourceCpp
#' @importFrom Rdpack reprompt
#' @import zoo
#' @import xts
#' @import lubridate
#' @import ggplot2
#' @import stats
#' @import utils
#' @import grDevices
NULL

# SPDX-License-Identifier: GPL-3.0-or-later
#
#  Copyright (C) 2024-2026 Domenic Franjic
#
#  This file is part of TwoStepSDFM.
#
#  TwoStepSDFM is free software: you can redistribute
#  it and/or modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TwoStepSDFM is distributed in the hope that it
#  will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with TwoStepSDFM. If not, see <https://www.gnu.org/licenses/>.

#' @name sparsePCA
#' @title Sparse Principal Components Analysis
#' @description
#' Estimate sparse sparse principal components via SPCA according to 
#' \insertRef{zou2006sparse}{TwoStepSDFM}.
#' 
#' @param data Numeric (no_of_vars \eqn{\times}{x} no_of_obs) matrix of data or 
#' zoo/xts object sampled at the same frequency.
#' @param delay Integer vector of variable delays, measured as the number of 
#' months since the latest available observation.
#' @param selected Integer vector of the number of selected variables for each 
#' factor.
#' @param no_of_factors Integer number of factors.
#' @param ridge_penalty Numeric ridge penalty.
#' @param lasso_penalty Numeric vector, lasso penalties for each factor (set to 
#' NULL to disable as stopping criterion).
#' @param max_iterations Integer maximum number of iterations.
#' @param max_no_steps Integer number of LARS steps (set to NULL to disable as 
#' stopping criterion).
#' @param weights Numeric vector, weights for each variable weighing the 
#' \eqn{\ell_1}{`l_1`} size constraint.
#' @param comp_null Numeric computational zero.
#' @param spca_conv_crit Conversion threshold for the SPCA algorithm.
#' @param parallel Logical, whether or not to use Eigen's internal parallel 
#' matrix operations.
#' @param svd_method Either "fast" or "precise". Option "fast" uses Eigen's 
#' BDCSVD divide and conquer method for the computation of the singular values. 
#' Option "precise" (default) implements the slower, but numerically more stable 
#' JacobiSVD method.
#' @param normalise Logical, whether to normalise the loading matrix as in 
#' \insertRef{zou2020elnet}{TwoStepSDFM}. Default is `TRUE`.
#' @param comp_var_expl Logical, whether to compute the relative variance 
#' explained by each factor. Default is `TRUE``.`
#'
#' @details
#' The function takes three stopping criteria: `selected`, `lasso_penalty`, and
#' `max_no_steps`. With `selected` the SPCA algorithm stops if each column of 
#' the estimated loading matrix has the corresponding number of non-zero 
#' loadings. This allows the user to directly control the degree of sparsity of 
#' each factor loading. With `lasso_penalty`, the SPCA algorithm stops as soon 
#' as the side-constraints of the inherent elastic-net problem are no longer 
#' satisfied. With `max_no_steps`, the SPCA algorithm only takes that many LARS 
#' steps for each factor loading's individual elastic-net problem before 
#' stopping. If all criteria are provided, the first one satisfied will stop the
#' algorithm. For details see also \insertCite{zou2006sparse}{TwoStepSDFM} and 
#' \insertCite{zou2020elnet}{TwoStepSDFM}.
#' 
#' Loosely, each SPCA algorithm iteration solves an elastic-net type problem for
#' each column of the loading matrix. One can extend this problem to the 
#' adaptive elastic-net \insertCite{zou2009adaptive}{TwoStepSDFM}. The variable 
#' `weights` lets the user provide weights for each observation. These weights 
#' must be strictly greater than zero and are normalised internally to represent 
#' relative weights. For more information on the computational implementation of
#' the weight extension in the context of SPCA see 
#' \insertRef{zou2024general}{TwoStepSDFM}.
#'  
#' In each SPCA algorithm iteration, the function executes an SVD. To this end, 
#' Eigen provides two alternatives \insertCite{eigenweb}{TwoStepSDFM}: Option 
#' `precise` makes use of JacobiSVD. This method is numerically more stable, but 
#' computationally costly, especially for medium to large matrices. Option 
#' `fast` makes use of BDCSVD. This divide-and-conquer approach can lead to 
#' significant performance gains with respect to large matrices. BDCSVD, 
#' however, can be numerically unstable when Eigen is compiled with aggressive 
#' speed optimisations. In the context of the `R`, this should be of no concern.
#' By default, `R` and most packages are compiled with "mild" `-O2` optimisation
#' and without any additional aggressive optimisation flags. Nonetheless, one 
#' should checker whether both variants provide reasonably close results before 
#' switching to `fast`. For more information see 
#' \insertRef{eigenweb}{TwoStepSDFM}.
#' 
#' @return
#' An object of class `SPCAFit` with components:
#' \describe{
#'   \item{data}{Original data matrix.}
#'   \item{loading_matrix_estim}{Numeric matrix of estimated factor loadings.}
#'   \item{factor_estim}{Object containing the SPCA factor estimates. The 
#'   object inherits its class from `data`: If `data` is provided as `zoo`, 
#'   `factor_estim` will be a `zoo` object. If `data` is provided as  `matrix`, 
#'   `factor_estim` will be a (`no_of_factors` \eqn{\times}{x} `no_of_obs`) 
#'   matrix.}
#'   \item{total_var_expl}{Numeric total variance explained.}
#'   \item{pct_var_expl}{Numeric vector relative variance explained by each 
#'   factor.}
#' }
#'
#' @author
#' Domenic Franjic
#' 
#' @references
#' \insertRef{zou2006sparse}{TwoStepSDFM}
#' 
#' \insertRef{zou2009adaptive}{TwoStepSDFM}
#' 
#' \insertRef{eigenweb}{TwoStepSDFM}
#' 
#' \insertRef{zou2020elnet}{TwoStepSDFM}
#' 
#' \insertRef{zou2024general}{TwoStepSDFM}
#'
#' @examples
#' data(factor_model)
#' set.seed(17032026)
#' no_of_factors <- 3
#' no_of_vars <- dim(factor_model$data)[2]
#' selected <- rep(floor(0.5 * no_of_vars), no_of_factors)
#' lasso_penalty <- exp(runif(no_of_factors, -10, 1))
#' max_no_steps <- 1000
#' spca_fit <- sparsePCA(data = factor_model$data, delay = factor_model$delay, 
#'                       selected = selected, no_of_factors = no_of_factors, 
#'                       ridge_penalty = 1e-2, lasso_penalty = lasso_penalty,
#'                       max_iterations = 1000, weights = NULL, 
#'                       max_no_steps = max_no_steps, comp_null = 1e-15,
#'                       spca_conv_crit = 1e-04, parallel = FALSE, 
#'                       svd_method = "precise", normalise = FALSE,
#'                       comp_var_expl = TRUE)
#' print(spca_fit)
#' spca_plots <- plot(spca_fit)
#' spca_plots$`Factor Time Series Plots`
#' spca_plots$`Loading Matrix Heatmap`
#' spca_plots$`Meas. Error Var.-Cov. Matrix Heatmap`
#' spca_plots$`Eigenvalue Plot`
#' spca_plots$`Variance Explained Chart`
#'
#' @export
sparsePCA <- function(data,
                      delay,
                      selected,
                      no_of_factors,
                      ridge_penalty = 1e-6,
                      lasso_penalty = NULL,
                      max_iterations = 1000,
                      weights = NULL,
                      max_no_steps = NULL,
                      comp_null = 1e-15,
                      spca_conv_crit = 1e-4,
                      parallel = FALSE,
                      svd_method = "precise",
                      normalise = TRUE,
                      comp_var_expl = TRUE) {
  func_call <- match.call()
  
  # Misshandling of the data matrix
  if(!is.zoo(data) && !is.xts(data)){
    data_r <- try(t(as.matrix(data)), silent = TRUE)
    if (inherits(data_r, "try-error")) {
      stop(paste0("data must be a matrix, convertible to a matrix or a time-series/zoo object"))
    }
  }else{
    data_r <- try(coredata(data), silent = TRUE)
    if (inherits(data_r, "try-error")) {
      stop(paste0("data must be a matrix, convertible to a matrix or a time-series/zoo object"))
    }
  }
  if(!is.numeric(data_r)){
    stop(paste0("data has non-numeric elements."))
  }
  if(any(is.infinite(data_r))){
    stop(paste0("data cannot have (-)Inf values."))
  }
  data_r[is.na(data_r)] <- 0 # Override R NAs as they seem to not get properly parsed to C++
  
  # Mishandling of delay
  no_of_variables <- dim(data_r)[2]
  no_of_observations <- dim(data_r)[1]
  if(is.null(delay)){
    delay <- matrix(rep(0, no_of_variables), ncol = 1)
  }else{
    delay <- checkPositiveSignedParameterVector(delay, "delay", no_of_variables)
  }
  
  # Check for NAs in the dataset outside the ragged edges
  na_ind <- FALSE
  for(col in 1:dim(data_r)[2]){
    na_ind <- any(is.na(data_r[1:(no_of_observations - delay[col]), col]))
  }
  if(na_ind){
    warning(paste0("data has NA values. AccordingThe corresponding time points will not be considered in th estimation of the loading matrix.")) 
  }
  
  # # Misshandling of dimensions
  # if(no_of_variables >= no_of_observations){
  #   warning(paste0("Too few observations as no_of-variables >= no_of_observations."))
  # }
  
  # Mishandling of selected
  if(is.null(selected)){
    selected <- matrix(rep(no_of_variables, no_of_factors), ncol = 1)
  }else{
    selected <- checkPositiveSignedParameterVector(selected, "selected", no_of_factors)
  }
  if(any(selected > no_of_variables)){
    warning(paste0("The elements in selected should not exceed the number of variables ", no_of_variables, ". The corresponding variables are set to ", no_of_variables, "."))
    selected[which(selected > no_of_variables),] <- no_of_variables
  }
  
  # Mishandling of number of factors
  no_of_factors <- checkPositiveSignedInteger(no_of_factors, "no_of_factors")
  if(no_of_factors == 0){
    stop("no_of_factors must be strictly positive.")
  }
  if(no_of_factors > no_of_variables){
    stop(paste0("no_of_factors must be smaller than no_of_variables."))
  }
  
  # Mishandling of ridge penalty
  ridge_penalty <- checkPositiveDouble(ridge_penalty, "ridge_penalty")
  
  # Mishandling of lasso_penalty penalty
  if (!is.null(lasso_penalty)){
    if(!is.numeric(lasso_penalty) || any(is.na(lasso_penalty))){
      stop(paste0("lasso_penalty must be a vector of non-NA numeric values."))
    }
    if(length(lasso_penalty) != no_of_factors){
      stop(paste0("lasso_penalty must be of length no_of_factors = ", no_of_factors))
    }
    if(any(lasso_penalty < 0)){
      stop(paste0("All elements of lasso_penalty non-negative."))
    }
  }else{
    lasso_penalty <- rep(-2147483647L, no_of_factors)
  }
  
  # Mishandlilng of max_iterations
  max_iterations <- checkPositiveSignedInteger(max_iterations, "max_iterations")
  
  # Mishandling of max_no_steps
  if(!is.null(max_no_steps)){
    max_no_steps <- checkPositiveSignedInteger(max_no_steps, "max_no_steps")
    if(max_no_steps == 0){
      stop(paste0("max_no_steps must be strictly positve."))
    }
  }else{
    max_no_steps <- -2147483647L # C++ INT_MIN
  }
  
  if(!is.null(weights)){
    if(!is.numeric(weights) || any(is.na(weights)) || any(weights <= 0)){
      stop(paste0("weights must be a vector of non-NA numeric values strictly greater 0."))
    }
    if(length(weights) != no_of_variables){
      stop(paste0("weights must be of length no_of_variables = ", no_of_variables))
    }
    if(sum(weights) != 1){
      message("weights are standardised to sum to 1")
      weights <- weights / sum(weights)
    }
  }else{
    weights <- rep(1, no_of_variables)
  }
  
  # Mishandling of comp_null
  comp_null <- checkPositiveDouble(comp_null, "comp_null")
  if(comp_null == 0){
    warning("comp_null should not be exactly 0. It will be jittered before further use.")
    comp_null <- 1e-15
  }
  
  # Mishandling of spca_conv_crit
  spca_conv_crit <- checkPositiveDouble(spca_conv_crit, "spca_conv_crit")
  if(spca_conv_crit == 0){
    warning("spca_conv_crit should not be exactly 0. It will be jittered before further use.")
    spca_conv_crit <- 1e-15
  }
  
  # Mishandling of check_rank
  parallel <- checkBoolean(parallel, "parallel")
  
  # Misshandling of svd_method
  if(!(svd_method %in% c("fast", "precise"))){
    stop(paste0("svd_method must be \"fast\" for usage of Eigen's BDCSVD or \"precise\" for usage of Eigen's JacobiSVD"))
  }
  
  # Misshandling of normalise
  normalise <- checkBoolean(normalise, "normalise")
  
  # Misshandling of comp_var_expl
  comp_var_expl <- checkBoolean(comp_var_expl, "comp_var_expl")
  
  result <- runSPCA(
    X_in = data_r,
    delay = delay, 
    selected = selected, 
    R = as.integer(no_of_factors), 
    ridge_penalty, 
    lasso_penalty, 
    max_iterations = max_iterations, 
    steps = max_no_steps, 
    weights = weights,
    comp_null = comp_null, 
    spca_conv_crit = spca_conv_crit, 
    parallel = parallel,
    svd_method = svd_method, 
    normalise = normalise, 
    comp_var_expl = comp_var_expl)
  
  # Rename the results
  names(result) <- c("loading_matrix_estim", "factor_estimate", "total_var_expl",
                     "pct_var_expl")
  result$data <- data
  
  if(is.zoo(data) || is.xts(data)){ # Also convert factors to time series
    start_vector <- c(year(time(data)[1]), month(time(data)[1]))
    result$factor_estimate <- as.zoo(ts(t(result$factor_estimate), start = start_vector, frequency = 12))
    colnames(result$factor_estimate) <- paste0("Factor ", 1:no_of_factors)
  }
  
  result$call <- func_call
  class(result) <- "SPCAFit"
  return(result)
}

#' @name print.SPCAFit
#' @title Generic printing function for SPCAFit S3 objects
#' @description
#' Print a compact summary of an `SPCAFit` object.
#'
#' @param x `SPCAFit` object.
#' @param ... Additional parameters for the plotting functions.
#'
#' @return
#' No return value; Prints a summary to the console.
#'
#' @author
#' Domenic Franjic
#' 
#' @export
print.SPCAFit <- function(x, ...) {
  simulated_time_series <- is.zoo(x$factor_estimate)
  no_of_factors <- ifelse(simulated_time_series, dim(x$factor_estimate)[2], dim(x$factor_estimate)[1])
  no_of_obs <- ifelse(simulated_time_series, dim(x$data)[1], dim(x$data)[2])
  cat("Simulated Dynamic Factor Model\n")
  cat("=========================================================================\n")
  cat("No. of Observations                        :", ifelse(simulated_time_series, dim(x$data)[1], dim(x$data)[2]), "\n")
  cat("No. of Variables                           :", ifelse(simulated_time_series, dim(x$data)[2], dim(x$data)[1]), "\n")
  cat("No. of Factors                             :", no_of_factors, "\n")
  cat("No. of zero elements in the loading matrix :", sum(x$loading_matrix_estim == 0), "\n")
  cat("=========================================================================\n")
  cat("Head of the factors :\n")
  max_print <- min(5, no_of_obs)
  if(simulated_time_series){
    print(head(x$factor_estimate, max_print))
  }else{
    print(x$factor_estimate[, 1:max_print])
  }
  cat("Tail of the factors :\n")
  if(simulated_time_series){
    print(tail(x$factor_estimate, max_print))
  }else{
    print(x$factor_estimate[, (dim(x$factor_estimate)[2] - (max_print - 1)):(dim(x$factor_estimate)[2])])
  }
  max_print_loadings <- min(5, ifelse(simulated_time_series, dim(x$factor_estimate)[1], dim(x$factor_estimate)[2]))
  cat("Head of the loading matrix :\n")
  print(head(x$loading_matrix_estim, max_print_loadings))
  cat("Tail of the loading matrix :\n")
  print(tail(x$loading_matrix_estim, max_print_loadings))
  cat("=========================================================================\n")
  invisible(x)
}

#' @name plot.SPCAFit
#' @title Generic plotting function for SPCAFit S3 objects
#' @description
#' Create diagnostic plots for an `SPCAFit` object.
#'
#' @param x `SPCAFit` object.
#' @param axis_text_size Numeric size of x- and y-axis labels. Passed to ggplot2 
#' `theme(..., text = element_text(size = axis_text_size))`.
#' @param legend_title_text_size Numeric size of x- and y-axis labels. Passed to
#' ggplot2 
#' `theme(..., legend.title = element_text(size = legend_title_text_size))`.
#' @param ... Additional parameters for the plotting functions.
#'
#' @return
#' A named list of plot objects:
#' \describe{
#'   \item{`Factor Time Series Plots`}{`patchwork`/`ggplot` object showing the 
#'   estimated factors over time.}
#'   \item{`Loading Matrix Heatmap`}{`ggplot` object showing a heatmap of the 
#'   estimated factor loadings. Zeros are highlighted in black.}
#'   \item{`Meas. Error Var.-Cov. Matrix Heatmap`}{`ggplot` object showing a 
#'   heatmap of the measurement error variance–covariance matrix.}
#'   \item{`Eigenvalue Plot`}{`ggplot` object showing a bar plot of the 
#'   eigenvalues of the measurement error variance–covariance matrix.}
#'  }
#' 
#' @author
#' Domenic Franjic
#' 
#' @export
plot.SPCAFit <- function (x, 
                          axis_text_size = 20, 
                          legend_title_text_size = 20,
                          ...) {
  out_list <- list()
  if (is.zoo(x$data)) {
    series_names <- colnames(x$data)
    no_of_factors <- dim(x$factor_estimate)[2]
    no_of_obs <- dim(x$factor_estimate)[1]
    time_vector <- as.Date(time(x$factor_estimate))
    factors <- x$factor_estimate
  } else {
    series_names <- rownames(x$data)
    no_of_factors <- dim(x$factor_estimate)[1]
    no_of_obs <- dim(x$factor_estimate)[2]
    time_vector <- 1:dim(x$factor_estimate)[2]
    factors <- t(x$factor_estimate)
    factors <- as.zoo(ts(factors, start = c(1, 1), frequency = 12))
  }
  
  out_list$`Factor Time Series Plots` <- plotFactorEstimates(factors, matrix(0, no_of_factors, no_of_factors * no_of_obs), 
                                                             no_of_factors, axis_text_size)
  
  out_list$`Loading Matrix Heatmap` <- plotLoadingHeatMap(x$loading_matrix_estim, series_names, 
                                                          no_of_factors, axis_text_size, 
                                                          legend_title_text_size)
  
  if (is.zoo(x$data)) {
    residuals <- coredata(na.omit(x$data)) - coredata(x$factor_estimate) %*% 
      t(x$loading_matrix_estim)
  } else {
    residuals <- na.omit(t(x$data)) - t(x$factor_estimate) %*% 
      t(x$loading_matrix_estim)
  }
  measurement_error_var_cov_df <- as.data.frame(t(residuals) %*% residuals * 1/(dim(residuals)[1] - 1))
  out_list$`Meas. Error Var.-Cov. Matrix Heatmap` <- plotMeasVarCovHeatmap(measurement_error_var_cov_df, 
                                                                           series_names, axis_text_size, 
                                                                           legend_title_text_size)
  
  out_list$`Eigenvalue Plot` <- plotMeasVarCovEigenvalues(eigen(measurement_error_var_cov_df)$values, 
                                                          no_of_factors, axis_text_size, 
                                                          legend_title_text_size)
  
  if (length(x$pct_var_expl) != 0) {
    var_explained_df <- data.frame(Component = c(paste0("Factor ", 1:no_of_factors), "Unexplained"), 
                                   `Variance Explained` = c(x$pct_var_expl, 1 - sum(x$pct_var_expl)), 
                                   check.names = FALSE)
    var_explained_df$Component <- factor(var_explained_df$Component)
    colourPalette <- grDevices::colorRamp(c("#88ccee", "#FFFFFF", "#117733"))
    cols_mat <- colourPalette(seq(0, 1, length.out = no_of_factors + 1))
    pie_chart_colours <- grDevices::rgb(cols_mat[, 1], cols_mat[, 2], cols_mat[, 3], maxColorValue = 255)
    names(pie_chart_colours) <- levels(var_explained_df$Component)
    var_explained_df$pct_label <- sprintf("%.3f%%", var_explained_df$`Variance Explained`)
    max_radial_position <- 1 - cumsum(var_explained_df$`Variance Explained`)
    min_radial_position <- c(1, head(max_radial_position, -1))
    var_explained_df$radial_position <- (min_radial_position + max_radial_position)/2
    out_list$`Variance Explained Chart` <- ggplot(var_explained_df, aes(x = "", y = `Variance Explained`, fill = Component)) +
      geom_col(width = 1) + coord_polar(theta = "y") +
      scale_fill_manual(values = pie_chart_colours) + 
      geom_text(aes(y = radial_position, label = pct_label), color = "black", size = axis_text_size * 0.5) +
      theme_void() + theme(text = element_text(size = axis_text_size),
                           legend.title = element_text(size = legend_title_text_size)
                           )
  } else {
    out_list$`Variance Explained Chart` <- "The relative variance explained by each factor has not been computed as comp_var_expl = FALSE"
  }
  return(out_list)
}

#' Example factor model dataset
#'
#' This is a simulated factor model dataset for demonstration and testing
#' with `TwoStepSDFM`.
#'
#' @format A `SimulData` containing the following elements:
#' \describe{
#' \item{data}{If `starting_date` is provided, a `zoo` object, else, 
#' a (`no_of_vars` \eqn{\times}{x} `no_of_obs`) numeric matrix holding the 
#' simulated data.}
#' \item{factors}{If `starting_date` is provided, a `zoo` object, else a
#'  (`no_of_factors` \eqn{\times}{x} `no_of_obs`) numeric matrix holding  the 
#'  simulated latent factors.}
#' \item{trans_var_coeff}{Numeric (`no_of_factors` \eqn{\times}{x} 
#' (`no_of_factors` * `factor_lag_order`)) factor VAR coefficient matrix.}
#' \item{loading_matrix}{Numeric factor loading matrix.}
#' \item{meas_error}{If `starting_date` is provided, a `zoo` object, else 
#' a (`no_of_vars` \eqn{\times}{x} `no_of_obs`) numeric matrix holding the 
#' fundamental measurement errors.}
#' \item{meas_error_var_cov}{Numeric measurement error 
#' variance-covariance matrix.}
#' \item{trans_error_var_cov}{Numeric transition error 
#' variance-covariance matrix.}
#' \item{frequency}{Integer vector of variable frequencies.}
#' \item{delay}{Integer vector of variable delays, measured as the number of 
#' months since the latest available observation.}
#' }
#' 
#' @source Generated via `simFM()`. For details see `factor_model$call`.
"factor_model"

#' Mixed-frequency factor model dataset
#'
#' This dataset contains simulated mixed-frequency factor model data
#' for examples in `TwoStepSDFM`.
#'
#' @format A `SimulData` containing the following elements:
#' \describe{
#' \item{data}{If `starting_date` is provided, a `zoo` object, else, 
#' a (`no_of_vars` \eqn{\times}{x} `no_of_obs`) numeric matrix holding the 
#' simulated data.}
#' \item{factors}{If `starting_date` is provided, a `zoo` object, else a
#'  (`no_of_factors` \eqn{\times}{x} `no_of_obs`) numeric matrix holding  the 
#'  simulated latent factors.}
#' \item{trans_var_coeff}{Numeric (`no_of_factors` \eqn{\times}{x} 
#' (`no_of_factors` * `factor_lag_order`)) factor VAR coefficient matrix.}
#' \item{loading_matrix}{Numeric factor loading matrix.}
#' \item{meas_error}{If `starting_date` is provided, a `zoo` object, else 
#' a (`no_of_vars` \eqn{\times}{x} `no_of_obs`) numeric matrix holding the 
#' fundamental measurement errors.}
#' \item{meas_error_var_cov}{Numeric measurement error 
#' variance-covariance matrix.}
#' \item{trans_error_var_cov}{Numeric transition error 
#' variance-covariance matrix.}
#' \item{frequency}{Integer vector of variable frequencies.}
#' \item{delay}{Integer vector of variable delays, measured as the number of 
#' months since the latest available observation.}
#' }
#' 
#' @source Generated via Generated via `simFM()`. For details see 
#' `mixed_freq_factor_model$call`.
"mixed_freq_factor_model"

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

runARDL <- function(target_variable, target_variable_predictor, predictor_variable, max_target_lags, max_predictor_lags, crit, jitter) {
    .Call(`_TwoStepSDFM_runARDL`, target_variable, target_variable_predictor, predictor_variable, max_target_lags, max_predictor_lags, crit, jitter)
}

#' @description
#' This function is for internal use only and may change in future releases
#' without notice. 
#'
runDL <- function(target_variable, predictor_variable, max_predictor_lags, crit, jitter) {
    .Call(`_TwoStepSDFM_runDL`, target_variable, predictor_variable, max_predictor_lags, crit, jitter)
}

runNoOfFactorsTest <- function(data_matrix_in, test_values, min_no_factors, max_no_factors, confidence_threshold) {
    .Call(`_TwoStepSDFM_runNoOfFactorsTest`, data_matrix_in, test_values, min_no_factors, max_no_factors, confidence_threshold)
}

runNoOfFactorsInfoCrit <- function(data_matrix_in, max_no_factors) {
    .Call(`_TwoStepSDFM_runNoOfFactorsInfoCrit`, data_matrix_in, max_no_factors)
}

#' @description
#' This function is for internal use only and may change in future releases
#' without notice. Users should use `twoStepSDFM()` instead for a stable and
#' supported interface.
#'
runSPCA <- function(X_in, delay, selected, R, l2, l1, max_iterations, steps, weights, comp_null, spca_conv_crit, parallel, svd_method, normalise, comp_var_expl) {
    .Call(`_TwoStepSDFM_runSPCA`, X_in, delay, selected, R, l2, l1, max_iterations, steps, weights, comp_null, spca_conv_crit, parallel, svd_method, normalise, comp_var_expl)
}

#' @description
#' This function is for internal use only and may change in future releases
#' without notice. Users should use `twoStepSDFM()` instead for a stable and
#' supported interface.
#'
runUVKFS <- function(X_in, delay, state_var_cov, measurement_var_cov, loading_matrix, factor_var_coefficient_matrices, R, order, fcast_horizon, decorr_errors, comp_null, parallel, jitter) {
    .Call(`_TwoStepSDFM_runUVKFS`, X_in, delay, state_var_cov, measurement_var_cov, loading_matrix, factor_var_coefficient_matrices, R, order, fcast_horizon, decorr_errors, comp_null, parallel, jitter)
}

#' @description
#' This function is for internal use only and may change in future releases
#' without notice. Users should use `twoStepSDFM()` instead for a stable and
#' supported interface.
#'
runSDFMKFS <- function(X_in, delay, selected, R, order, decorr_errors, crit, l2, l1, max_iterations, steps, weights, comp_null, spca_conv_crit, parallel, fcast_horizon, jitter, svd_method) {
    .Call(`_TwoStepSDFM_runSDFMKFS`, X_in, delay, selected, R, order, decorr_errors, crit, l2, l1, max_iterations, steps, weights, comp_null, spca_conv_crit, parallel, fcast_horizon, jitter, svd_method)
}

#' @description
#' This function is for internal use only and may change in future releases
#' without notice. Users should use `twoStepDFM()` instead for a stable and
#' supported interface.
#'
runDFMKFS <- function(X_in, delay, R, order, decorr_errors, crit, comp_null, parallel, fcast_horizon, jitter) {
    .Call(`_TwoStepSDFM_runDFMKFS`, X_in, delay, R, order, decorr_errors, crit, comp_null, parallel, fcast_horizon, jitter)
}

#' @description
#' This function is for internal use only and may change in future releases
#' without notice. Users should use `SimFM()` instead for a stable and
#' supported interface.
#'
runStaticFM <- function(T, N, S, Lambda, mu_e, Sigma_e, A, order, quarterfy, corr, beta_param, m, seed, R, burn_in, rescale, parallel) {
    .Call(`_TwoStepSDFM_runStaticFM`, T, N, S, Lambda, mu_e, Sigma_e, A, order, quarterfy, corr, beta_param, m, seed, R, burn_in, rescale, parallel)
}

#' @useDynLib TwoStepSDFM, .registration=TRUE
#' @importFrom Rcpp sourceCpp
#' @importFrom Rdpack reprompt
#' @import zoo
#' @import xts
#' @import lubridate
#' @import ggplot2
#' @import stats
#' @import utils
#' @import doParallel
#' @import doSNOW
#' @import foreach
#' @import parallel
#' @import withr
NULL

# SPDX-License-Identifier: GPL-3.0-or-later
#
#  Copyright (C) 2024-2026 Domenic Franjic
#
#  This file is part of TwoStepSDFM.
#
#  TwoStepSDFM is free software: you can redistribute
#  it and/or modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TwoStepSDFM is distributed in the hope that it
#  will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with TwoStepSDFM. If not, see <https://www.gnu.org/licenses/>.

#' @name crossVal
#' 
#' @title Cross-validate SDFM Hyper-Parameters
#' 
#' @description
#' This function uses time series cross-validation 
#' \insertCite{rob2018forecasting}{TwoStepSDFM} in combination with random 
#' hyper-parameter search \insertCite{bergstra2012random}{TwoStepSDFM} to 
#' validate the hyper-parameters of a sparse dynamic factor model as described  
#' in \insertRef{franjic2024nowcasting}{TwoStepSDFM}
#' 
#' @param data Numeric (no_of_vars \eqn{\times}{x} no_of_obs) matrix of data or 
#' zoo/xts object sampled at mixed frequencies (quarterly and monthly).
#' @param variable_of_interest Integer indicating the index of the target 
#' variables.
#' @param fcast_horizon Integer value indicating the target forecasting horizon.
#' @param delay Integer vector of variable delays, measured as the number of 
#' months since the latest available observation.
#' @param frequency Integer vector of frequencies of the variables in the data 
#' set (currently supported: `12` for monthly and `4` for quarterly data).
#' @param no_of_factors Integer number of factors.
#' @param seed 32-bit unsigned integer seed for all random processes inside the 
#' function.
#' @param min_ridge_penalty Numeric lower bound for the sampled ridge penalty 
#' coefficient candidates.
#' @param max_ridge_penalty Numeric upper bound for the sampled ridge penalty 
#' coefficient candidates.
#' @param cv_repetitions Integer number of `fcast_horizon`-step-ahead 
#' predictions computed for each candidate set.
#' @param cv_size Integer number of candidate sets.
#' @param lasso_penalty_type Character indicating the lasso penalty type. 
#' If set to ``"selected"``, the \eqn{\ell_1}{`l_1`}-size constraint will be 
#' returned as number of non-zero elements of each column of the loading matrix. 
#' If set to ``"penalty"``, the lasso size constraint will be returned. If set 
#' to ``"steps"``, the number of LARS-EN steps will be returned.
#' @param min_max_penalty Vector of size two, where the first element indicates 
#' the lower and the second element indicates the upper bound of the lasso 
#' penalty equivalent. If `lasso_penalty_type` is set to ``"selected"`` or 
#' ``"steps"``, both elements must be strictly positive integers.
#' @param max_factor_lag_order Integer maximum order of the VAR process in the 
#' transition equation.
#' @param lag_estim_criterion Information criterion used for the estimation of 
#' the factor VAR order (`"BIC"` (default), `"AIC"`, `"HIC"`).
#' @param decorr_errors Logical, whether or not the errors should be 
#' decorrelated.
#' @param max_iterations Integer maximum number of iterations of the SPCA 
#' algorithm.
#' @param weights Numeric vector, weights for each variable weighing the 
#' \eqn{\ell_1}{`l_1`} size constraint.
#' @param comp_null Numeric computational zero.
#' @param spca_conv_crit Numeric conversion criterion for the SPCA algorithm.
#' @param parallel Logical, whether or not to run the cross-validation loop in 
#' parallel.
#' @param no_of_cores Integer number of cores to use when run in parallel.
#' @param max_ar_lag_order Integer maximum number of lags of the target variable
#' included in the final ARDL prediction routine.
#' @param max_predictor_lag_order Integer maximum number of lags of the 
#' predictors included in the final ARDL prediction routine.
#' @param jitter Numerical jitter for stability of internal solver algorithms. 
#' The jitter is added to the diagonal entries of the variance covariance matrix 
#' of the measurement errors.
#' @param svd_method Either `"fast"` or `"precise"`. Option `"fast"` uses 
#' Eigen's BDCSVD divide and conquer method for the computation of the singular 
#' values. Option `"precise"` (default) implements the slower, but numerically 
#' more stable JacobiSVD method.
#' @param verbose Logical, whether to print some progress tracking output to the 
#' console. 
#' 
#' @details
#' `fcast_horizon` should be set to the target prediction horizon, as
#' hyper-parameters can differ substantially between different horizons. For
#' nowcasting, use `fcast_horizon = 0`. For backcasting, `fcast_horizon` can be
#' set to a negative number indicating the step-back backcasting horizon.
#'
#' Internally, candidates of the hyper-parameters are drawn randomly. However,
#' a regular dense DFM will always be considered by default. The ridge
#' penalty is drawn as \eqn{\exp(u)}{exp(u)}, where \eqn{u}{u} is uniformly
#' distributed between `min_ridge_penalty` and `max_ridge_penalty`. If
#' `lasso_penalty_type = "selected"`, the lasso penalty is drawn as a random
#' vector \eqn{\bm{v}}{v}, where each entry is uniformly distributed. If
#' `lasso_penalty_type = "steps"`, the lasso penalty is drawn as a random
#' value \eqn{v}{v} that is uniformly distributed. If
#' `lasso_penalty_type = "penalty"`, the lasso penalty is drawn as a random
#' vector \eqn{\exp(\bm{v})}{exp(v)}, where each entry of
#' \eqn{\bm{v}}{v} is uniformly distributed. In all three cases, the upper and
#' lower bounds of the uniform distributions governing the lasso penalties are
#' given by the first and second entry of `min_max_penalty`, respectively.
#'
#' For medium to large data sets in combination with a medium to large
#' `cv_size`, it can be beneficial to set `parallel = TRUE`. This will enable
#' parallelisation via the doParallel, doSNOW, foreach, and parallel packages
#' in R. In this case, `no_of_cores` should be set to the number of physical
#' cores of the user's machine. It is not advisable to use the number of logical
#' cores, as this can considerably deteriorate performance.
#'
#' This function serves as a direct wrapper to \code{\link{nowcast}}. For more
#' information on the additional function parameters, see the corresponding help
#' page.
#' 
#' @return
#' An object of class `SDFMcrossVal` with main components:
#' \describe{
#'   \item{`CV`}{A list with components \code{`CV Results`} (matrix of all
#'   cross-validation errors and corresponding hyper-parameter values) and
#'     \code{`Min. CV`} (row of `CV Results` with the minimum cross-validation 
#'     error).}
#'   \item{`BIC`}{A list with components `BIC Results` (matrix of all BIC values 
#'   and corresponding hyper-parameter values) and `Min. BIC` (row of
#'   `BIC Results` with the minimum BIC).}
#' }
#'
#' @author
#' Domenic Franjic
#' 
#' @references
#' \insertRef{bergstra2012random}{TwoStepSDFM}
#' 
#' \insertRef{rob2018forecasting}{TwoStepSDFM}
#' 
#' \insertRef{franjic2024nowcasting}{TwoStepSDFM}
#' 
#' @seealso
#' \code{\link{sparsePCA}}: Routine for fitting estimating a sparse factor 
#' loading matrix.
#'  
#' \code{\link{kalmanFilterSmoother}}: Routine for filtering and smoothing 
#' latent factors.
#' 
#' \code{\link{twoStepSDFM}}: Two-step estimation routine for a sparse dynamic 
#' factor model.
#' 
#' \code{\link{twoStepDenseDFM}}: Two-step estimation routine for a dense 
#' dynamic factor model.
#' 
#' @examples
#' data(mixed_freq_factor_model)
#' no_of_vars <- dim(mixed_freq_factor_model$data)[2]
#' no_of_factors <- dim(mixed_freq_factor_model$factors)[2]
#' cv_results <- crossVal(data = mixed_freq_factor_model$data, variable_of_interest = 1, 
#'                        fcast_horizon = 0, delay = mixed_freq_factor_model$delay, 
#'                        frequency = mixed_freq_factor_model$frequency,
#'                        no_of_factors = no_of_factors, seed = 25032026,
#'                        min_ridge_penalty = 1e-5, max_ridge_penalty = 10, 
#'                        cv_repetitions = 1, cv_size = 50, lasso_penalty_type = "selected",
#'                        min_max_penalty = c(5, 45), verbose = FALSE)
#' print(cv_results)
#' cv_plots <- plot(cv_results)        
#' cv_plots$`CV Results`
#' cv_plots$`BIC Results` 
#' 
#' @export
crossVal <- function(data,
                     variable_of_interest,
                     fcast_horizon,
                     delay,
                     frequency,
                     no_of_factors,
                     seed,
                     min_ridge_penalty,
                     max_ridge_penalty,
                     cv_repetitions,
                     cv_size,
                     lasso_penalty_type,
                     min_max_penalty,
                     max_factor_lag_order = 10,
                     lag_estim_criterion = "BIC",
                     decorr_errors = TRUE,
                     max_iterations = 1000,
                     weights = NULL,
                     comp_null = 1e-15,
                     spca_conv_crit = 1e-4,
                     parallel = FALSE,
                     no_of_cores = 1,
                     max_ar_lag_order = 5,
                     max_predictor_lag_order = 5,
                     jitter = 1e-8,
                     svd_method = "precise",
                     verbose = TRUE) {
  func_call <- match.call()
  
  
  # Mishandling
  
  # Mishandling of seed
  seed <- checkPositiveSignedInteger(seed, "seed", 33)
  
  # Misshandling of the data matrix
  if(!is.zoo(data) && !is.xts(data)){
    stop(paste0("data must be a time-series/zoo object"))
  }
  no_of_variables <- dim(data)[2]
  no_of_mtly_variables <- sum(frequency == 12)
  no_of_observations <- dim(data)[1]
  
  # Mishandling of frequency
  frequency <- checkPositiveSignedParameterVector(frequency, "frequency", no_of_variables)
  if (length(frequency) != no_of_variables || any(!(frequency %in% c(4, 12)))) {
    stop(paste0("frequency has non-conform values. Currently only values 4 (quarterly data) and 12 (monthly data) are supported."))
  }
  
  # Mishandling of delay
  if(is.null(delay)){
    delay <- matrix(rep(0, no_of_variables), ncol = 1)
  }else{
    delay <- checkPositiveSignedParameterVector(delay, "delay", no_of_variables)
  }
  
  # Check for NAs in the dataset outside the ragged edges
  na_ind <- FALSE
  for(col in 1:dim(data)[2]){
    na_ind <- any(is.na(data[1:(no_of_observations - delay[col]), col]))
    if(na_ind){
      stop(paste0("data has NA values outside the ragged edges.")) 
    }
  }
  
  
  # Check for observations in the dataset inside the ragged edges
  obs_ind <- FALSE
  for(col in 1:dim(data)[2]){
    if(delay[col] > 0){
      obs_ind <- !all(is.na(data[(no_of_observations - delay[col] + 1):no_of_observations, col]))
    }
    if(obs_ind){
      stop(paste0("data has observed values inside the ragged edges.")) 
    }
  }
  
  # Mishandling of variable_of_interest
  variable_of_interest <- checkPositiveSignedInteger(variable_of_interest, "variable_of_interest")
  if(variable_of_interest == 0 || variable_of_interest > no_of_variables){
    stop(paste0("variable_of_interest must be a strictly positive integer between [1, no. of variables] = [1, ", no_of_variables, "]."))
  }
  if(frequency[variable_of_interest] != 4){
    stop(paste0("variable_of_interest must correpsond to a quarterly variable. Cross-validation for monthly target series is currently not supported."))
  }
  
  # Mishandling of fcast_horizon
  fcast_horizon <- checkPositiveDouble(fcast_horizon, "fcast_horizon")
  
  # Mishandling of min_ridge_penalty
  min_ridge_penalty <- checkPositiveDouble(min_ridge_penalty, "min_ridge_penalty")
  if(min_ridge_penalty == 0){
    warning("min_ridge_penalty should not be exactly 0. It will be jittered before further use.")
    min_ridge_penalty <- 1e-15
  }
  
  # Mishandling of max_ridge_penalty
  max_ridge_penalty <- checkPositiveDouble(max_ridge_penalty, "max_ridge_penalty")
  if(max_ridge_penalty < min_ridge_penalty){
    stop(paste0("max_ridge_penalty cannot be smaller than min_ridge_penalty."))
  }
  
  # Mishandling of cv_repetitions
  cv_repetitions <- checkPositiveSignedInteger(cv_repetitions, "cv_repetitions")
  if(cv_repetitions == 0){
    stop(paste0("cv_repetitions must be striclty positive."))
  }
  
  # Mishandling of cv_size
  cv_size <- checkPositiveSignedInteger(cv_size, "cv_size")
  if(cv_size <= 1){
    stop(paste0("cv_size must be striclty greater 1."))
  }
  
  # Mishandling of no_of_cores
  no_of_cores <- checkPositiveSignedInteger(no_of_cores, "no_of_cores")
  if(no_of_cores == 0){
    stop(paste0("no_of_cores must be a strictly positive integer."))
  }
  if(no_of_cores > floor(parallel::detectCores() / 2)){
    warning(paste0("no_of_cores is bigger than half the number of (physical) cores, i.e., ", floor(parallel::detectCores() / 2), ". For systems with multi-thhreading, it is recommended to use at most the number of physical cores."))
  }
  if(no_of_cores > parallel::detectCores()){
    stop(paste0("no_of_cores cannot be bigger as the maxmimum number of cores, i.e., ", parallel::detectCores()))
  }
  
  # Mishandling of lasso_penalty_type
  if(!(lasso_penalty_type %in% c("steps", "selected", "penalty"))){
    stop(paste0("lasso_penalty_type must be one of \"steps\", \"selected\", or \"penalty\"."))
  }
  
  # Mishandling of min_max_penalty
  if(length(min_max_penalty) != 2){
    stop("min_max_penalty must be of length 2.")
  }
  if(lasso_penalty_type %in% "steps"){
    min_max_penalty[1] <- checkPositiveSignedInteger(min_max_penalty[1], "The first element of min_max_pealty")
    min_max_penalty[2] <- checkPositiveSignedInteger(min_max_penalty[2], "The second element of min_max_pealty")
    if(min_max_penalty[1] == 0){
      stop(paste0("If lasso_penalty_type == \"steps\", the first element cannot be zero."))
    }
  }else if(lasso_penalty_type %in% "penalty"){
    min_max_penalty[1] <- checkPositiveDouble(min_max_penalty[1], "The first element of min_max_pealty")
    min_max_penalty[2] <- checkPositiveDouble(min_max_penalty[2], "The second element of min_max_pealty")
    if(min_max_penalty[1] == 0){
      warning("The first element of min_max_penalty should not be exactly 0. It will be jittered before further use.")
      min_max_penalty[1] <- 1e-15
    }
  }else if(lasso_penalty_type %in% "selected"){
    min_max_penalty[1] <- checkPositiveSignedInteger(min_max_penalty[1], "The first element of min_max_pealty")
    min_max_penalty[2] <- checkPositiveSignedInteger(min_max_penalty[2], "The second element of min_max_pealty")
    if(min_max_penalty[1] > sum(frequency == 12)){
      warning(paste0("The first element of min_max_penalty is bigger than the number of monthly variables. It is set to the number of variables for further use."))
      min_max_penalty[1] <- sum(frequency == 12)
    }
    if(min_max_penalty[2] > sum(frequency == 12)){
      warning(paste0("The second element of min_max_penalty is bigger than the number of monthly variables. It is set to the number of variables for further use."))
      min_max_penalty[2] <- sum(frequency == 12)
    }
  }
  if(min_max_penalty[1] >= min_max_penalty[2]){
    stop(paste0("The first element of min_max_penalty must not be bigger than the second element."))
  }
  
  # Checking whether the data-sets ends with a complete quarter of observations and cropping accordingly
  if(month(time(data))[no_of_observations] %% 3 != 0){
    warning(paste0("data must end at the last month of the final quarter. data is cropped for further use."))
    if(month(time(data))[no_of_observations] %in% c(1, 4, 7, 10)){
      data <- data[1:(no_of_observations - 1), ]
      no_of_observations <- dim(data)[1]
      delay[which(frequency == 4)] <- pmax(delay[which(frequency == 4)] - 1, 0)
    }else if(month(time(data))[no_of_observations] %in% c(2, 5, 8, 11)){
      data <- data[1:(no_of_observations - 2), ]
      no_of_observations <- dim(data)[1]
      delay[which(frequency == 4)] <- pmax(delay[which(frequency == 4)] - 2, 0)
    }
  }
  
  # Randomly draw the candidates for the hyper-parameters according to which LARS-EN stopping criterion should be used
  if(lasso_penalty_type %in% "steps"){
    candidates <- matrix(NaN, cv_size, 2)
    candidates[1, ] <- c(0.0, min_max_penalty[2])
  }else if(lasso_penalty_type %in% "penalty"){
    candidates <- matrix(NaN, cv_size, 1 + no_of_factors)
    candidates[1, ] <- c(0.0, rep(0.0, no_of_factors))
  }else if(lasso_penalty_type %in% "selected"){
    candidates <- matrix(NaN, cv_size, 1 + no_of_factors)
    candidates[1, ] <- c(0.0, rep(no_of_mtly_variables, no_of_factors))
  }
  
  log_min_ridge_penalty <- log(min_ridge_penalty)
  log_max_ridge_penalty <- log(max_ridge_penalty)
  with_seed(seed,
            {
              for(i in 2:cv_size){
                candidates[i, 1] <- exp(runif(1, log_min_ridge_penalty, log_max_ridge_penalty))
                if(lasso_penalty_type %in% "steps"){
                  candidates[i, 2] <- floor(runif(1, min_max_penalty[1], min_max_penalty[2]))
                }else if (lasso_penalty_type %in% "penalty") {
                  candidates[i, 2:(no_of_factors + 1)] <- exp(runif(no_of_factors, log(min_max_penalty[1]), log(min_max_penalty[2])))
                }else if (lasso_penalty_type %in% "selected") {
                  candidates[i, 2:(no_of_factors + 1)] <- floor(runif(no_of_factors, min_max_penalty[1], min_max_penalty[2]))
                }
              }
            }
  )
  
  cv_results <- matrix(NaN, cv_size, 1)
  bic_results <- matrix(NaN, cv_size, 1)
  if(!parallel){
    
    # Set-up progress bar
    if (verbose){
      message("Currently validating the model hyper-parameter in series.")
      pb <- txtProgressBar(max = cv_size, style = 3)
      setTxtProgressBar(pb, 0)
    }
    
    min_cv <- .Machine$double.xmax
    min_bic <- .Machine$double.xmax
    for(h in 1:cv_size){
      
      current_results <- 
        nowcastSpecificationHelper(cv_repetitions = cv_repetitions, no_of_factors = no_of_factors, no_of_variables = no_of_variables, 
                                   no_of_observations = no_of_observations, no_of_mtly_variables = no_of_mtly_variables,
                                   lasso_penalty_type = lasso_penalty_type,
                                   data = data, variable_of_interest = variable_of_interest, 
                                   fcast_horizon = fcast_horizon, delay = delay,  
                                   candidates = candidates[h, ], frequency = frequency, 
                                   max_factor_lag_order = max_factor_lag_order, 
                                   decorr_errors = decorr_errors, lag_estim_criterion = lag_estim_criterion,
                                   max_iterations = max_iterations, comp_null = comp_null, 
                                   spca_conv_crit = spca_conv_crit, max_ar_lag_order = max_ar_lag_order,
                                   max_predictor_lag_order = max_predictor_lag_order, 
                                   jitter = jitter, svd_method = svd_method, weights = weights
        )
      
      bic_results[h, 1] <- current_results$bic
      
      cv_results[h, 1] <- current_results$cv
      
      if(verbose){
        setTxtProgressBar(pb, h)
      }
    }
    if(verbose){
      close(pb)
    }
    
  }else if(parallel){
    
    # Set-up progress bar
    if (verbose) {
      message("Currently validating the model hyper-parameter in parallel.")
      pb <- txtProgressBar(max = cv_size, style = 3)
      progressFunc <- function(n) setTxtProgressBar(pb, n)
      opts <- list(progress = progressFunc)
    } else {
      pb <- NULL
      progressFunc <- NULL
      opts <- list()
    }
    
    # Set-up parallelisation
    cl <- makeCluster(no_of_cores)
    registerDoSNOW(cl)
    global_vars_to_export <- c("nowcastSpecificationHelper", "makeRaggedEdges")
    
    h_indices <- 1:cv_size
    results <- foreach(h = h_indices, 
                       .packages = c("zoo", "xts", "TwoStepSDFM", "lubridate"), 
                       .options.snow = opts,
                       .combine = 'rbind',
                       .multicombine = TRUE,
                       .export = global_vars_to_export) %dopar% {
                         
                         current_results <- 
                           nowcastSpecificationHelper(cv_repetitions = cv_repetitions, no_of_factors = no_of_factors, no_of_variables = no_of_variables, 
                                                      no_of_observations = no_of_observations, no_of_mtly_variables = no_of_mtly_variables,
                                                      lasso_penalty_type = lasso_penalty_type,
                                                      data = data, variable_of_interest = variable_of_interest, 
                                                      fcast_horizon = fcast_horizon, delay = delay,  
                                                      candidates = candidates[h, ], frequency = frequency, 
                                                      max_factor_lag_order = max_factor_lag_order, 
                                                      decorr_errors = decorr_errors, lag_estim_criterion = lag_estim_criterion,
                                                      max_iterations = max_iterations, comp_null = comp_null, 
                                                      spca_conv_crit = spca_conv_crit, max_ar_lag_order = max_ar_lag_order,
                                                      max_predictor_lag_order = max_predictor_lag_order, 
                                                      jitter = jitter, svd_method = svd_method, weights)
                         
                         out <- as.data.frame(matrix(NaN, 1, 2))
                         colnames(out) <- names(current_results)
                         out$cv <- current_results$cv
                         out$bic <- current_results$bic
                         out
                       }
    if(verbose && !is.null(pb)){
      close(pb)
    }
    stopCluster(cl)
    
    cv_results[, 1] <- results$cv
    bic_results[, 1] <- results$bic
    
  }
  
  cv_out <- cbind(cv_results, candidates)
  bic_out <- cbind(bic_results, candidates)
  if(lasso_penalty_type %in% "selected"){
    colnames(cv_out) <- c("CV Errors",
                          "Ridge Penalty",
                          paste0(paste0("Factor ", 1:no_of_factors), " # non-zero Loadings"))
    colnames(bic_out) <- c("BIC",
                           "Ridge Penalty",
                           paste0(paste0("Factor ", 1:no_of_factors), " # non-zero Loadings"))
    
  }else if(lasso_penalty_type %in% "penalty"){
    colnames(cv_out) <- c("CV Errors",
                          "Ridge Penalty",
                          paste0(paste0("Factor ", 1:no_of_factors), " Lasso Penalty"))
    colnames(bic_out) <- c("BIC",
                           "Ridge Penalty",
                           paste0(paste0("Factor ", 1:no_of_factors), " Lasso Penalty"))
    
  }else if(lasso_penalty_type %in% "steps"){
    colnames(cv_out) <- c("CV Errors",
                          "Ridge Penalty",
                          "Maximum No. of LARS Steps")
    colnames(bic_out) <- c("BIC",
                           "Ridge Penalty",
                           "Maximum No. of LARS Steps")
    
  }
  
  result <- list()
  result$CV <- list(
    `CV Results` = cv_out,
    `Min. CV` = cv_out[which.min(cv_out[, 1]), , drop = FALSE]
  )
  result$BIC <- list(
    `BIC Results` = bic_out,
    `Min. BIC` = bic_out[which.min(bic_out[, 1]), , drop = FALSE]
  )
  
  result$call <- match.call()
  class(result) <- "SDFMcrossVal"
  return(result)
}

#' Helper function to wrap the nowcasting routine
#' @keywords internal
nowcastSpecificationHelper <- function(cv_repetitions, no_of_factors, no_of_variables,
                                       no_of_observations, no_of_mtly_variables,
                                       lasso_penalty_type, data, variable_of_interest, 
                                       fcast_horizon, delay,  candidates, frequency, 
                                       max_factor_lag_order, decorr_errors, 
                                       lag_estim_criterion, max_iterations,  comp_null, 
                                       spca_conv_crit,  max_ar_lag_order, 
                                       max_predictor_lag_order, jitter, svd_method,
                                       weights){
  
  fcast_error <- c()
  fcast_ind <- 1
  for(t in rev(seq(from = delay[variable_of_interest], by = 3, length.out = cv_repetitions))){
    
    oos_observation <- data[no_of_observations - t, variable_of_interest]
    is_data <- makeRaggedEdges(data[1:(no_of_observations - t), , drop = FALSE], delay)
    current_no_of_obs <- dim(is_data)[1]
    if(lasso_penalty_type %in% "selected"){
      current_nowcast <- nowcast(data = is_data, variables_of_interest = variable_of_interest, 
                                 max_fcast_horizon = fcast_horizon, delay = delay, 
                                 selected = candidates[2:(no_of_factors + 1)],
                                 frequency = frequency, no_of_factors = no_of_factors, 
                                 max_factor_lag_order = max_factor_lag_order, 
                                 decorr_errors = decorr_errors, lag_estim_criterion = lag_estim_criterion,
                                 ridge_penalty = candidates[1], lasso_penalty = NULL,
                                 max_iterations = max_iterations, max_no_steps = NULL,
                                 weights = weights,
                                 comp_null = comp_null, spca_conv_crit = spca_conv_crit,
                                 parallel = FALSE, max_ar_lag_order = max_ar_lag_order,
                                 max_predictor_lag_order = max_predictor_lag_order, jitter = jitter,
                                 svd_method = svd_method)
    }else if(lasso_penalty_type %in% "penalty"){
      current_nowcast <- nowcast(data = is_data, variables_of_interest = variable_of_interest, 
                                 max_fcast_horizon = fcast_horizon, delay = delay, 
                                 selected =   rep(no_of_mtly_variables, no_of_factors),
                                 frequency = frequency, no_of_factors = no_of_factors, 
                                 max_factor_lag_order = max_factor_lag_order, 
                                 decorr_errors = decorr_errors, lag_estim_criterion = lag_estim_criterion,
                                 ridge_penalty = candidates[1], lasso_penalty = candidates[2:(no_of_factors + 1)],
                                 max_iterations = max_iterations, max_no_steps = NULL,
                                 weights = weights,
                                 comp_null = comp_null, spca_conv_crit = spca_conv_crit,
                                 parallel = FALSE, max_ar_lag_order = max_ar_lag_order,
                                 max_predictor_lag_order = max_predictor_lag_order, jitter = jitter,
                                 svd_method = svd_method)
    }else if(lasso_penalty_type %in% "steps"){
      current_nowcast <- nowcast(data = is_data, variables_of_interest = variable_of_interest, 
                                 max_fcast_horizon = fcast_horizon, delay = delay, 
                                 selected =   rep(no_of_mtly_variables, no_of_factors),
                                 frequency = frequency, no_of_factors = no_of_factors, 
                                 max_factor_lag_order = max_factor_lag_order, 
                                 decorr_errors = decorr_errors, lag_estim_criterion = lag_estim_criterion,
                                 ridge_penalty = candidates[1], lasso_penalty = NULL,
                                 max_iterations = max_iterations, max_no_steps = candidates[2],
                                 weights = weights,
                                 comp_null = comp_null, spca_conv_crit = spca_conv_crit,
                                 parallel = FALSE, max_ar_lag_order = max_ar_lag_order,
                                 max_predictor_lag_order = max_predictor_lag_order, jitter = jitter,
                                 svd_method = svd_method)
      
    }
    
    nowcast_indicator <- which(as.yearqtr(time(current_nowcast$Forecasts)) == as.yearqtr(time(is_data)[current_no_of_obs]))
    fcast_error[fcast_ind] <- coredata(current_nowcast$Forecasts[fcast_horizon + nowcast_indicator, 2]) - coredata(oos_observation)
    fcast_ind <- fcast_ind + 1
  }
  
  bic_h <- (mean((t(coredata(is_data[, which(frequency == 12)]))
                  - current_nowcast$`SDFM Fit`$loading_matrix_estimate
                  %*% t(coredata(current_nowcast$`SDFM Fit`$smoothed_factors[1:current_no_of_obs, , drop = FALSE]))
  )^2, na.rm = TRUE)
  + sum(current_nowcast$`SDFM Fit`$loading_matrix_estimate != 0)
  * log(no_of_variables * current_no_of_obs) / (no_of_variables * current_no_of_obs)
  )
  
  cv_h <- mean(fcast_error^2)
  
  return(list(cv = cv_h, bic = bic_h))
  
}

#' @method print SDFMcrossVal
#' @title Generic print function for SDFMcrossVal S3 objects
#' 
#' @param x `SDFMcrossVal` object.
#' @param ... Additional parameters for the plotting functions.
#'
#' @return
#' No return value; Prints a summary to the console.
#'
#' @author
#' Domenic Franjic
#' 
#' @export
print.SDFMcrossVal <- function(x, ...) {
  
  # Extrcat which LARS stopping criterion has been used
  if(any(grepl("Lasso", colnames(x$CV$`CV Results`), ignore.case = FALSE))){
    crit_label <- "Optimum Lasso Penalties"
  }else if(any(grepl("# non-zero Loadings", colnames(x$CV$`CV Results`), ignore.case = FALSE))){
    crit_label <- "# of non-zero Loadings per Factor"
  }
  
  cat("Cross-Validation Results\n")
  cat("=========================================================================\n")
  cat("Cross-Validation Error: ", x$CV$`Min. CV`[1], "\n")
  cat("Optimum Ridge Penalty : ", x$CV$`Min. CV`[2], "\n")
  if(any(grepl("Maximum No. of LARS Steps", colnames(x$CV$`CV Results`), ignore.case = FALSE))){
    cat("# of LARS steps       : ", x$CV$`Min. CV`[3], "\n")
  }else{
    cat("\n")
    cat(crit_label, "\n")
    for(n in 3:length(x$CV$`Min. CV`)){
      cat("Factor ", n - 2, "            :", x$CV$`Min. CV`[n], "\n")  
    }
  }
  cat("=========================================================================\n")
  cat("\n")
  cat("BIC Results\n")
  cat("=========================================================================\n")
  cat("BIC                   : ", x$BIC$`Min. BIC`[1], "\n")
  cat("Optimum Ridge Penalty : ", x$BIC$`Min. BIC`[2], "\n")
  if(any(grepl("Maximum No. of LARS Steps", colnames(x$CV$`CV Results`), ignore.case = FALSE))){
    cat("# of LARS steps       : ", x$BIC$`Min. BIC`[3], "\n")
  }else{
    cat("\n")
    cat(crit_label, "\n")
    for(n in 3:length(x$BIC$`Min. BIC`)){
      cat("Factor ", n - 2, "            :", x$BIC$`Min. BIC`[n], "\n")  
    }
  }
  cat("=========================================================================\n")
}

#' @method plot SDFMcrossVal
#' @title Generic plotting function for SDFMcrossVal S3 objects
#' @param x `SDFMcrossVal` object.
#' @param axis_text_size Numeric size of x- and y-axis labels. Prased to ggplot2 
#' `theme(..., text = element_text(size = axis_text_size))`.
#' @param legend_title_text_size Numeric size of x- and y-axis labels. Prased to
#' ggplot2 
#' `theme(..., legend.title = element_text(size = legend_title_text_size))`.
#' @param ... Additional parameters for the plotting functions.
#' 
#' @return
#' A named list of `ggplot` objects:
#' \describe{
#'   \item{`CV Results`}{`ggplot` object of the cross-validation error against 
#'   the log Ridge penalty. The overall sparsity level of the loading matrix
#'   induced by the lasso penalty is indicated by point shapes and colours.}
#'   \item{`BIC Results`}{`ggplot` object of the BIC against the log Ridge 
#'   penalty. The overall sparsity level of the loading matrix induced by the 
#'   lasso penalty is indicated by point shapes and colours.}
#' }
#'
#' @author
#' Domenic Franjic
#' 
#' @export
plot.SDFMcrossVal <- function(x,                         
                              axis_text_size = 20, 
                              legend_title_text_size = 20,  
                              ...) {
  out_list <- list()
  
  # Plot depending on which stopping criterion has been used
  if(any(grepl("Lasso", colnames(x$CV$`CV Results`), ignore.case = FALSE))){
    
    # Cross-validation results
    cv_data <- data.frame(x$CV$`CV Results`, check.names = FALSE)
    cv_data$`Lasso Penalties` <- c(NaN)
    for(i in 1:dim(cv_data)[1]){
      cv_data$`Lasso Penalties`[i] <- paste0("(", 
                                             paste0(sprintf("%.2f", cv_data[i, 3:(dim(cv_data)[2] - 1)]), collapse = ";"), 
                                             ")")
    }
    avg_lasso_penalty <- rowMeans(cv_data[, 3:(dim(cv_data)[2] - 1), drop = FALSE])
    breaks <- seq(from = min(avg_lasso_penalty, na.rm = TRUE), 
                  to = max(avg_lasso_penalty, na.rm = TRUE), 
                  length.out = 6)
    breaks[length(breaks)] <- breaks[length(breaks)] + 0.000006
    labels <- paste0("[", sprintf("%.2f", floor(breaks[1:5] / 0.01) * 0.01),
                     ", ",
                     sprintf("%.2f", floor(breaks[2:6] / 0.01) * 0.01),
                     ")")
    labels[5] <- paste0("[", sprintf("%.2f", floor(breaks[5] / 0.01) * 0.01),
                        ", ",
                        sprintf("%.2f", floor(breaks[6] / 0.01) * 0.01),
                        "]")
    binned_data_equal_width <- cut(avg_lasso_penalty,
                                   breaks = breaks,
                                   right = FALSE,
                                   include.lowest = TRUE,
                                   labels = labels)
    cv_data$`Avg. Lasso Penalty` <- as.factor(binned_data_equal_width)
    best_combo <- cv_data$`Lasso Penalties`[which.min(cv_data$`CV Errors`)]
    best_ridge <- cv_data$`Ridge Penalty`[which.min(cv_data$`CV Errors`)]
    best_cv_error <- min(cv_data$`CV Errors`)
    y_min_limit <- best_cv_error
    y_max_limit <- max(cv_data$`CV Errors`)
    out_list$`CV Results` <- ggplot(cv_data, aes(x = `Ridge Penalty`, y = `CV Errors`, colour = `Avg. Lasso Penalty`, 
                                                 shape = `Avg. Lasso Penalty`)) +
      geom_point(size = 3.5) +
      geom_hline(yintercept = cv_data$`CV Errors`[1], colour = "black") +
      scale_colour_manual(values =  c("#88CCEE", "#44799E", "#000000", "#41784A", "#117733"),
                          name = "Avg. Lasso Penalty") +
      scale_shape_discrete(name = "Avg. Lasso Penalty") +
      geom_point(data = subset(cv_data, `CV Errors` == min(`CV Errors`)), aes(x = `Ridge Penalty`, y = `CV Errors`), 
                 colour = "black", fill = "#882255", size = 7, shape = 22) +
      scale_y_continuous(trans = "log10", limits = c(y_min_limit, y_max_limit)) + 
      scale_x_continuous(trans = "log10") +
      annotate("text",  x = best_ridge, y = best_cv_error,
               label = best_combo, angle = 0, vjust = 1.6, hjust = 1, size = 4, color = "darkred") +
      labs(x = "log Ridge Penalty",
           y = "log CV Error") +
      theme_minimal() + 
      theme(text = element_text(size = axis_text_size),
            legend.title = element_text(size = legend_title_text_size))
    
    # BIC results
    bic_data <- data.frame(x$BIC$`BIC Results`, check.names = FALSE)
    bic_data$`Lasso Penalties` <- c(NaN)
    for(i in 1:dim(bic_data)[1]){
      bic_data$`Lasso Penalties`[i] <- paste0("(", 
                                              paste0(sprintf("%.2f", bic_data[i, 3:(dim(bic_data)[2] - 1)]), collapse = ";"), 
                                              ")")
    }
    bic_data$`Avg. Lasso Penalty` <- as.factor(binned_data_equal_width)
    best_bic_combo <- bic_data$`Lasso Penalties`[which.min(bic_data$`BIC`)]
    best_bic_ridge <- bic_data$`Ridge Penalty`[which.min(bic_data$`BIC`)]
    best_bic <- min(bic_data$`BIC`)
    y_bic_min_limit <- best_cv_error
    y_bic_max_limit <- max(bic_data$`BIC`)
    out_list$`BIC Results` <- ggplot(bic_data, aes(x = `Ridge Penalty`, y = `BIC`, colour = `Avg. Lasso Penalty`, 
                                                   shape = `Avg. Lasso Penalty`)) +
      geom_point(size = 3.5) +
      geom_hline(yintercept = bic_data$BIC[1], colour = "black") +
      scale_colour_manual(values =  c("#88CCEE", "#44799E", "#000000", "#41784A", "#117733"),
                          name = "Avg. Lasso Penalty") +
      scale_shape_discrete(name = "Avg. Lasso Penalty") +
      geom_point(data = subset(bic_data, `BIC` == min(`BIC`)), aes(x = `Ridge Penalty`, y = `BIC`), 
                 colour = "black", fill = "#882255", size = 7, shape = 22) +
      scale_y_continuous(limits = c(y_bic_min_limit, y_bic_max_limit)) + 
      scale_x_continuous(trans = "log10") +
      annotate("text",  x = best_bic_ridge, y = best_bic,
               label = best_bic_combo, angle = 0, vjust = 1.6, hjust = 1, size = 4, color = "darkred") +
      labs(x = "log Ridge Penalty",
           y = "BIC") +
      theme_minimal() + 
      theme(text = element_text(size = axis_text_size),
            legend.title = element_text(size = legend_title_text_size))
    
  }else if(any(grepl("# non-zero Loadings", colnames(x$CV$`CV Results`), ignore.case = FALSE))){
    
    cv_data <- data.frame(x$CV$`CV Results`, check.names = FALSE)
    cv_data$`# non-zero Loadings` <- c(NaN)
    for(i in 1:dim(cv_data)[1]){
      cv_data$`# non-zero Loadings`[i] <- paste0("(", 
                                                 paste0(cv_data[i, 3:(dim(cv_data)[2] - 1)], collapse = ";"), 
                                                 ")")
    }
    sparsity_ratios <- 1 - rowSums(cv_data[, 3:(dim(cv_data)[2] - 1), drop = FALSE]) / rowSums(cv_data[1, 3:(dim(cv_data)[2] - 1), drop = FALSE])
    breaks <- seq(from = min(sparsity_ratios, na.rm = TRUE), 
                  to = max(sparsity_ratios, na.rm = TRUE), 
                  length.out = 6)
    breaks[length(breaks)] <- breaks[length(breaks)] + 0.000006
    labels <- paste0("[", sprintf("%.2f", floor(breaks[1:5] / 0.01) * 0.01),
                     ", ",
                     sprintf("%.2f", floor(breaks[2:6] / 0.01) * 0.01),
                     ")")
    labels[5] <- paste0("[", sprintf("%.2f", floor(breaks[5] / 0.01) * 0.01),
                        ", ",
                        sprintf("%.2f", floor(breaks[6] / 0.01) * 0.01),
                        "]")
    binned_data_equal_width <- cut(sparsity_ratios,
                                   breaks = breaks,
                                   right = FALSE,
                                   include.lowest = TRUE,
                                   labels = labels)
    cv_data$`Sparsity Ratio` <- as.factor(binned_data_equal_width)
    best_combo <- cv_data$`# non-zero Loadings`[which.min(cv_data$`CV Errors`)]
    best_ridge <- cv_data$`Ridge Penalty`[which.min(cv_data$`CV Errors`)]
    best_cv_error <- min(cv_data$`CV Errors`)
    y_min_limit <- best_cv_error
    y_max_limit <- max(cv_data$`CV Errors`)
    out_list$`CV Results` <- ggplot(cv_data, aes(x = `Ridge Penalty`, y = `CV Errors`, colour = `Sparsity Ratio`, 
                                                 shape = `Sparsity Ratio`)) +
      geom_point(size = 3.5) +
      geom_hline(yintercept = cv_data$`CV Errors`[1], colour = "black") +
      scale_colour_manual(values =  c("#88CCEE", "#44799E", "#000000", "#41784A", "#117733"),
                          name = "Sparsity Ratio") +
      scale_shape_discrete(name = "Sparsity Ratio") +
      geom_point(data = subset(cv_data, `CV Errors` == min(`CV Errors`)), aes(x = `Ridge Penalty`, y = `CV Errors`), 
                 colour = "black", fill = "#882255", size = 7, shape = 22) +
      scale_y_continuous(trans = "log10", limits = c(y_min_limit, y_max_limit)) + 
      scale_x_continuous(trans = "log10") +
      annotate("text",  x = best_ridge, y = best_cv_error,
               label = best_combo, angle = 0, vjust = 1.6, hjust = 1, size = 4, color = "darkred") +
      labs(x = "log Ridge Penalty",
           y = "log CV Error") +
      theme_minimal() + 
      theme(text = element_text(size = axis_text_size),
            legend.title = element_text(size = legend_title_text_size))
    
    # BIC results
    bic_data <- data.frame(x$BIC$`BIC Results`, check.names = FALSE)
    bic_data$`# non-zero Loadings` <- c(NaN)
    for(i in 1:dim(bic_data)[1]){
      bic_data$`# non-zero Loadings`[i] <- paste0("(", 
                                                  paste0(sprintf("%.0f", bic_data[i, 3:(dim(bic_data)[2] - 1)]), collapse = ";"), 
                                                  ")")
    }
    bic_data$`Sparsity Ratio` <- as.factor(binned_data_equal_width)
    best_bic_combo <- bic_data$`# non-zero Loadings`[which.min(bic_data$`BIC`)]
    best_bic_ridge <- bic_data$`Ridge Penalty`[which.min(bic_data$`BIC`)]
    best_bic <- min(bic_data$`BIC`)
    y_bic_min_limit <- best_cv_error
    y_bic_max_limit <- max(bic_data$`BIC`)
    out_list$`BIC Results` <- ggplot(bic_data, aes(x = `Ridge Penalty`, y = `BIC`, colour = `Sparsity Ratio`, 
                                                   shape = `Sparsity Ratio`)) +
      geom_point(size = 3.5) +
      geom_hline(yintercept = bic_data$BIC[1], colour = "black") +
      scale_colour_manual(values =  c("#88CCEE", "#44799E", "#000000", "#41784A", "#117733"),
                          name = "Sparsity Ratio") +
      scale_shape_discrete(name = "Sparsity Ratio") +
      geom_point(data = subset(bic_data, `BIC` == min(`BIC`)), aes(x = `Ridge Penalty`, y = `BIC`), 
                 colour = "black", fill = "#882255", size = 7, shape = 22) +
      scale_y_continuous(limits = c(y_bic_min_limit, y_bic_max_limit)) + 
      scale_x_continuous(trans = "log10") +
      annotate("text",  x = best_bic_ridge, y = best_bic,
               label = best_bic_combo, angle = 0, vjust = 1.6, hjust = 1, size = 4, color = "darkred") +
      labs(x = "log Ridge Penalty",
           y = "BIC") +
      theme_minimal() + 
      theme(text = element_text(size = axis_text_size),
            legend.title = element_text(size = legend_title_text_size))
    
  }else if(any(grepl("Maximum No. of LARS Steps", colnames(x$CV$`CV Results`), ignore.case = FALSE))){
    cv_data <- data.frame(x$CV$`CV Results`, check.names = FALSE)
    breaks <- floor(seq(from = min(cv_data$`Maximum No. of LARS Steps`, na.rm = TRUE), 
                        to = max(cv_data$`Maximum No. of LARS Steps`, na.rm = TRUE), 
                        length.out = 6))
    breaks[length(breaks)] <- breaks[length(breaks)] + 0.000006
    labels <- paste0("[", sprintf("%.0f", floor(breaks[1:5] / 0.01) * 0.01),
                     ", ",
                     sprintf("%.0f", floor(breaks[2:6] / 0.01) * 0.01),
                     ")")
    labels[5] <- paste0("[", sprintf("%.0f", floor(breaks[5] / 0.01) * 0.01),
                        ", ",
                        sprintf("%.0f", floor(breaks[6] / 0.01) * 0.01),
                        "]")
    binned_data_equal_width <- cut(cv_data$`Maximum No. of LARS Steps`,
                                   breaks = breaks,
                                   right = FALSE,
                                   include.lowest = TRUE,
                                   labels = labels)
    cv_data$`# of LARS Steps` <- as.factor(binned_data_equal_width)
    best_combo <- cv_data$`Maximum No. of LARS Steps`[which.min(cv_data$`CV Errors`)]
    best_ridge <- cv_data$`Ridge Penalty`[which.min(cv_data$`CV Errors`)]
    best_cv_error <- min(cv_data$`CV Errors`)
    y_min_limit <- best_cv_error
    y_max_limit <- max(cv_data$`CV Errors`)
    out_list$`CV Results` <- ggplot(cv_data, aes(x = `Ridge Penalty`, y = `CV Errors`, colour = `# of LARS Steps`, 
                                                 shape = `# of LARS Steps`)) +
      geom_point(size = 3.5) +
      geom_hline(yintercept = cv_data$`CV Errors`[1], colour = "black") +
      scale_colour_manual(values =  c("#88CCEE", "#44799E", "#000000", "#41784A", "#117733"),
                          name = "# of LARS Steps") +
      scale_shape_discrete(name = "# of LARS Steps") +
      geom_point(data = subset(cv_data, `CV Errors` == min(`CV Errors`)), aes(x = `Ridge Penalty`, y = `CV Errors`), 
                 colour = "black", fill = "#882255", size = 7, shape = 22) +
      scale_y_continuous(trans = "log10", limits = c(y_min_limit, y_max_limit)) + 
      scale_x_continuous(trans = "log10") +
      annotate("text",  x = best_ridge, y = best_cv_error,
               label = best_combo, angle = 0, vjust = 1.6, hjust = 1, size = 4, color = "darkred") +
      labs(x = "log Ridge Penalty",
           y = "log CV Error") +
      theme_minimal() + 
      theme(text = element_text(size = axis_text_size),
            legend.title = element_text(size = legend_title_text_size))
    
    # BIC results
    bic_data <- data.frame(x$BIC$`BIC Results`, check.names = FALSE)
    bic_data$`# of LARS Steps` <- as.factor(binned_data_equal_width)
    best_bic_combo <- bic_data$`Maximum No. of LARS Steps`[which.min(bic_data$`BIC`)]
    best_bic_ridge <- bic_data$`Ridge Penalty`[which.min(bic_data$`BIC`)]
    best_bic <- min(bic_data$`BIC`)
    y_bic_min_limit <- best_cv_error
    y_bic_max_limit <- max(bic_data$`BIC`)
    out_list$`BIC Results` <- ggplot(bic_data, aes(x = `Ridge Penalty`, y = `BIC`, colour = `# of LARS Steps`, 
                                                   shape = `# of LARS Steps`)) +
      geom_point(size = 3.5) +
      geom_hline(yintercept = bic_data$BIC[1], colour = "black") +
      scale_colour_manual(values =  c("#88CCEE", "#44799E", "#000000", "#41784A", "#117733"),
                          name = "# of LARS Steps") +
      scale_shape_discrete(name = "# of LARS Steps") +
      geom_point(data = subset(bic_data, `BIC` == min(`BIC`)), aes(x = `Ridge Penalty`, y = `BIC`), 
                 colour = "black", fill = "#882255", size = 7, shape = 22) +
      scale_y_continuous(limits = c(y_bic_min_limit, y_bic_max_limit)) + 
      scale_x_continuous(trans = "log10") +
      annotate("text",  x = best_bic_ridge, y = best_bic,
               label = best_bic_combo, angle = 0, vjust = 1.6, hjust = 1, size = 4, color = "darkred") +
      labs(x = "log Ridge Penalty",
           y = "BIC") +
      theme_minimal() + 
      theme(text = element_text(size = axis_text_size),
            legend.title = element_text(size = legend_title_text_size))
  }
  
  return(out_list)
}

test_that("", {
  
  # Load data
  data("mixed_freq_factor_model")
  data_zoo <- scale(mixed_freq_factor_model$data)
  no_of_factors <- dim(mixed_freq_factor_model$factors)[2]
  no_of_mtly_vars <- sum(mixed_freq_factor_model$frequency == 12)
  variable_of_interest <- 1
  fcast_horizon <- 1
  delay <- mixed_freq_factor_model$delay
  frequency <- mixed_freq_factor_model$frequency
  seed <- 09102025
  min_ridge_penalty <- 0.01
  max_ridge_penalty <- 1
  lasso_penalty_type <- "selected"
  min_max_penalty <- c(10, no_of_mtly_vars - 1)
  cv_repetitions <- 3
  cv_size <- 10
  parallel = TRUE
  no_of_cores <- 2
  max_ar_lag_order = 5
  max_predictor_lag_order = 5
  
  # Set seed to check whether cross-val changes with the global RNG stat
  set.seed(seed)
  old_seed <- .Random.seed
  
  # CV results in series
  expect_no_error(cv_series <- crossVal(data = data_zoo, variable_of_interest = variable_of_interest, fcast_horizon = fcast_horizon,
                                        delay = delay, frequency = frequency, no_of_factors = no_of_factors,
                                        seed = seed, min_ridge_penalty = min_ridge_penalty, max_ridge_penalty = max_ridge_penalty,
                                        cv_repetitions = cv_repetitions, cv_size = cv_size, lasso_penalty_type = lasso_penalty_type,
                                        min_max_penalty = min_max_penalty, verbose = FALSE))
  
  # CV results in parallel
  expect_no_error(cv_parallel <- crossVal(data = data_zoo, variable_of_interest = variable_of_interest, fcast_horizon = fcast_horizon,
                                          delay = delay, frequency = frequency, no_of_factors = no_of_factors,
                                          seed = seed, min_ridge_penalty = min_ridge_penalty, max_ridge_penalty = max_ridge_penalty,
                                          cv_repetitions = cv_repetitions, cv_size = cv_size, lasso_penalty_type = lasso_penalty_type,
                                          min_max_penalty = min_max_penalty, verbose = FALSE, parallel = TRUE, no_of_cores = no_of_cores))
  
  # Basic checks (set call null as they are different by construction)
  cv_series$call <- NULL
  cv_parallel$call <- NULL
  expect_equal(cv_series, cv_parallel)
  
  
  # Test selecting according to the max number of steps
  expect_no_error(cv_series_steps <- crossVal(data = data_zoo, variable_of_interest = variable_of_interest, fcast_horizon = fcast_horizon,
                                              delay = delay, frequency = frequency, no_of_factors = no_of_factors,
                                              seed = seed, min_ridge_penalty = min_ridge_penalty, max_ridge_penalty = max_ridge_penalty,
                                              cv_repetitions = cv_repetitions, cv_size = cv_size, lasso_penalty_type = "steps",
                                              min_max_penalty = c(1, 500), verbose = FALSE))
  expect_no_error(cv_parallel_steps <- crossVal(data = data_zoo, variable_of_interest = variable_of_interest, fcast_horizon = fcast_horizon,
                                                delay = delay, frequency = frequency, no_of_factors = no_of_factors,
                                                seed = seed, min_ridge_penalty = min_ridge_penalty, max_ridge_penalty = max_ridge_penalty,
                                                cv_repetitions = cv_repetitions, cv_size = cv_size, lasso_penalty_type = "steps",
                                                min_max_penalty = c(1, 500), parallel = TRUE, 
                                                no_of_cores = no_of_cores, verbose = FALSE))
  cv_series_steps$call <- NULL
  cv_parallel_steps$call <- NULL
  expect_equal(cv_series_steps, cv_parallel_steps)
  
  # Test selecting according to the lasso penalty
  expect_no_error(cv_series_lasso <- crossVal(data = data_zoo, variable_of_interest = variable_of_interest, fcast_horizon = fcast_horizon,
                                              delay = delay, frequency = frequency, no_of_factors = no_of_factors,
                                              seed = seed, min_ridge_penalty = min_ridge_penalty, max_ridge_penalty = max_ridge_penalty,
                                              cv_repetitions = cv_repetitions, cv_size = cv_size, lasso_penalty_type = "penalty",
                                              min_max_penalty = c(0.0001, 10), verbose = FALSE))
  expect_no_error(cv_parallel_lasso <- crossVal(data = data_zoo, variable_of_interest = variable_of_interest, fcast_horizon = fcast_horizon,
                                                delay = delay, frequency = frequency, no_of_factors = no_of_factors,
                                                seed = seed, min_ridge_penalty = min_ridge_penalty, max_ridge_penalty = max_ridge_penalty,
                                                cv_repetitions = cv_repetitions, cv_size = cv_size, lasso_penalty_type = "penalty",
                                                min_max_penalty = c(0.0001, 10), parallel = TRUE, 
                                                no_of_cores = no_of_cores, verbose = FALSE))
  cv_series_lasso$call <- NULL
  cv_parallel_lasso$call <- NULL
  expect_equal(cv_series_lasso, cv_parallel_lasso)
  
  # Check whether cross-val changes with the global RNG stat
  current_seed <- .Random.seed
  expect_equal(old_seed, current_seed)
  
})

#' @useDynLib TwoStepSDFM, .registration=TRUE
#' @importFrom Rcpp sourceCpp
#' @importFrom Rdpack reprompt
#' @import zoo
#' @import xts
#' @import lubridate
#' @import ggplot2
#' @import stats
#' @import utils
NULL

# SPDX-License-Identifier: GPL-3.0-or-later
#
#  Copyright (C) 2024-2026 Domenic Franjic
#
#  This file is part of TwoStepSDFM.
#
#  TwoStepSDFM is free software: you can redistribute
#  it and/or modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TwoStepSDFM is distributed in the hope that it
#  will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with TwoStepSDFM. If not, see <https://www.gnu.org/licenses/>.

#' @name nowcast
#' @title Predict Mixed-Frequency Data via Dynamic Factor Models
#' @description
#' Backcast, nowcast, and forecast quarterly target variables via a sparse/dense 
#' DFM using additional monthly data with ragged edges. Forecasts are produced 
#' using all quarterly targets and a quarterly representation of latent monthly 
#' factors \insertCite{Mariano2003new_coincident}{TwoStepSDFM}. Final 
#' predictions are computed via equally weighted forecast averaging of ARDL 
#' models \insertCite{marcellino2010factor}{TwoStepSDFM} for each of the targets 
#' and quarterfied factors.
#' 
#' @param data Numeric (no_of_vars \eqn{\times}{x} no_of_obs) matrix of data or 
#' zoo/xts object sampled at mixed frequencies (quarterly and monthly).
#' @param variables_of_interest Integer vector indicating the index of all 
#' target variables.
#' @param max_fcast_horizon Maximum forecasting horizon of all targets.
#' @param delay Integer vector of variable delays, measured as the number of 
#' months since the latest available observation.
#' @param selected Integer vector of the number of selected variables for each 
#' factor.
#' @param frequency Integer vector of frequencies of the variables in the data 
#' set (currently supported: `12` for monthly and `4` for quarterly data).
#' @param no_of_factors Integer number of factors.
#' @param sparse Logical, if `TRUE` (default) a sparse DFM is used to estimate 
#' the model parameters and latent factors (see \code{\link{twoStepSDFM}}). 
#' Else, a dense DFM is used (see \code{\link{twoStepDenseDFM}}).
#' @param max_factor_lag_order Integer maximum order of the VAR process in the 
#' transition equation.
#' @param lag_estim_criterion Information criterion used for the estimation of 
#' the factor VAR order (`"BIC"` (default), `"AIC"`, `"HIC"`).
#' @param decorr_errors Logical, whether or not the errors should be 
#' decorrelated.
#' @param ridge_penalty Numeric ridge penalty.
#' @param lasso_penalty Numeric vector, lasso penalties for each factor (set to 
#' NULL to disable as stopping criterion).
#' @param max_iterations Integer maximum number of iterations of the SPCA 
#' algorithm.
#' @param max_no_steps Integer number of LARS steps (set to NULL to disable as 
#' stopping criterion).
#' @param weights Numeric vector, weights for each variable weighing the 
#' \eqn{\ell_1}{`l_1`} size constraint.
#' @param comp_null Numeric computational zero.
#' @param spca_conv_crit Numeric conversion criterion for the SPCA algorithm.
#' @param parallel Logical, whether or not to use Eigen's internal parallel 
#' matrix operations.
#' @param max_ar_lag_order Integer maximum number of lags of the target variable
#' included in the final ARDL prediction routine.
#' @param max_predictor_lag_order Integer maximum number of lags of the 
#' predictors included in the final ARDL prediction routine.
#' @param jitter Numerical jitter for stability of internal solver algorithms. 
#' The jitter is added to the diagonal entries of the variance covariance matrix 
#' of the measurement errors.
#' @param svd_method Either `"fast"` or `"precise"`. Option `"fast"` uses 
#' Eigen's BDCSVD divide and conquer method for the computation of the singular 
#' values. Option `"precise"` (default) implements the slower, but numerically 
#' more stable JacobiSVD method. 
#' 
#' @details
#' This function serves as a prediction wrapper for the
#' \code{\link{twoStepDenseDFM}} and \code{\link{twoStepSDFM}} functions. `data`
#' should be a mixed-frequency data set. Currently, only monthly and quarterly
#' data are supported. With respect to the quarterly data, the function expects
#' the realization of the quarterly observations to occur in the last month of
#' the quarter. Indicate quarterly and monthly variables via `frequency` by
#' setting the corresponding element of `frequency` to `4` for quarterly and to
#' `12` for monthly data.
#'
#' This function is only able to compute predictions for quarterly variables.
#' To impute the ragged edges of the monthly observations, and potentially
#' compute additional predictions for the monthly variables, call `predict` on
#' the `SDFMFit` object returned by \code{\link{twoStepDenseDFM}} / 
#' \code{\link{twoStepSDFM}} (see \code{\link{predict.SDFMFit}}).
#'
#' `max_fcast_horizon` sets the maximum number of forecasts predicted starting
#' from the final observation of the data set. For each target, the number of
#' backcasts and whether or not a nowcast should be computed is determined
#' internally. This is done in such a way that every missing quarterly
#' observation of the targets is predicted.
#'
#' `max_ar_lag_order` governs the maximum number of lags of the current target
#' used to predict said target in each ARDL model. `max_predictor_lag_order`
#' governs the maximum number of lags of each additional quarterly predictor,
#' including other potential targets and the aggregated factors, used to predict
#' any given target in each ARDL model. The actual number of lags is internally
#' estimated using the BIC. Setting `max_ar_lag_order = 0` disables the use of
#' target lags in its own prediction function.
#' 
#' `sparse` toggles between a sparse DFM and a dense DFM. If `sparse = FALSE`,
#' all SPCA stopping criteria and other parameters passed to the sparse
#' estimation routine are ignored (for details on these parameters see
#' \code{\link{twoStepDenseDFM}}). Parameters governing the Kalman Filter and
#' Smoother are passed directly to \code{\link{twoStepDenseDFM}} / 
#' \code{\link{twoStepSDFM}}. For details see the corresponding help pages.
#' 
#' @return 
#' The `nowcast` function returns named list containing the following objects:
#' \describe{
#'   \item{Forecasts}{Numeric matrix of the target variables and their 
#'   respective backcasts, nowcasts, and/or forecasts.}
#'   \item{SDFM Fit}{An `SDFMFit` object holding the estimates of the model 
#'   parameters and the latent factors (see \code{\link{twoStepSDFM}} or 
#'   \code{\link{twoStepDenseDFM}}).}
#' }
#' 
#' @author
#' Domenic Franjic
#' 
#' @references
#' \insertRef{Mariano2003new_coincident}{TwoStepSDFM}
#' 
#' \insertRef{marcellino2010factor}{TwoStepSDFM}
#' 
#' \insertRef{franjic2024nowcasting}{TwoStepSDFM}
#' 
#' @seealso
#' \code{\link{sparsePCA}}: Routine for fitting estimating a sparse factor 
#' loading matrix.
#'  
#' \code{\link{kalmanFilterSmoother}}: Routine for filtering and smoothing 
#' latent factors.
#' 
#' \code{\link{twoStepSDFM}}: Two-step estimation routine for a sparse dynamic 
#' factor model.
#' 
#' \code{\link{twoStepDenseDFM}}: Two-step estimation routine for a dense 
#' dynamic factor model.
#' 
#' @examples
#' data(mixed_freq_factor_model)
#' no_of_vars <- dim(mixed_freq_factor_model$data)[2]
#' no_of_factors <- dim(mixed_freq_factor_model$factors)[2]
#' sparse_nowcast <- nowcast(data = mixed_freq_factor_model$data, variables_of_interest = c(1, 2),
#'                           max_fcast_horizon = 4, delay = mixed_freq_factor_model$delay,
#'                           selected = rep(floor(0.5 * no_of_vars), no_of_factors), 
#'                           frequency = mixed_freq_factor_model$frequency, 
#'                           no_of_factors = no_of_factors, sparse = TRUE)
#' print(sparse_nowcast)
#' dense_nowcast <- nowcast(data = mixed_freq_factor_model$data, variables_of_interest = c(1, 2),
#'                          max_fcast_horizon = 4, delay = mixed_freq_factor_model$delay,
#'                          selected = NULL, frequency = mixed_freq_factor_model$frequency, 
#'                          no_of_factors = no_of_factors, sparse = FALSE)
#' sparse_plots <- plot(sparse_nowcast)
#' sparse_plots$`Single Pred. Fcast Density Plots Series 1`
#' 
#' @export
nowcast <- function(data,
                    variables_of_interest,
                    max_fcast_horizon,
                    delay,
                    selected,
                    frequency,
                    no_of_factors,
                    sparse = TRUE,
                    max_factor_lag_order = 10,
                    lag_estim_criterion = "BIC",
                    decorr_errors = TRUE,
                    ridge_penalty = 1e-6,
                    lasso_penalty = NULL,
                    max_iterations = 1000,
                    max_no_steps = NULL,
                    weights = NULL,
                    comp_null = 1e-15,
                    spca_conv_crit = 1e-4,
                    parallel = FALSE,
                    max_ar_lag_order = 5,
                    max_predictor_lag_order = 5,
                    jitter = 1e-8,
                    svd_method = "precise") {
  func_call <- match.call()
  
  # Misshandling
  
  # The following variables will be checked inside the twoStepSDFM function:
  #   selected, no_of_factors, max_factor_lag_order, decorr_errors,
  #   ridge_penalty, lasso_penalty, max_iterations, max_no_steps, weights, 
  #   comp_null, spca_conv_crit,
  
  # Misshandling of the data matrix
  if(!is.zoo(data) && !is.xts(data)){
    stop(paste0("data must be a time-series/zoo object"))
  }
  no_of_variables <- dim(data)[2]
  no_of_observations <- dim(data)[1]
  
  # Mishandling of variables_of_interest
  variables_of_interest <- checkPositiveSignedParameterVector(variables_of_interest, "variables_of_interest", length(variables_of_interest))
  variables_of_interest <- as.vector(variables_of_interest)
  if(length(variables_of_interest) != length(unique(variables_of_interest))){
    warning("variables_of_interest has non-unique entires. Only unique entries will be used going forward.")
    variables_of_interest <- unique(variables_of_interest)
    variables_of_interest <- checkPositiveSignedParameterVector(variables_of_interest, "variables_of_interest", length(variables_of_interest))
    variables_of_interest <- as.vector(variables_of_interest)
  }
  no_of_vois <- length(variables_of_interest)
  
  # Mishandling of delay
  if(is.null(delay)){
    delay <- matrix(rep(0, no_of_variables), ncol = 1)
  }else{
    delay <- checkPositiveSignedParameterVector(delay, "delay", no_of_variables)
  }
  if(any(-max_fcast_horizon >= delay[variables_of_interest])) {
    stop("`max_fcast_horizon` is too recent relative to the available missing data. Adjust delays or forecast horizon.")
  }
  
  # Check for NAs in the dataset outside the ragged edges
  na_ind <- FALSE
  for(col in 1:dim(data)[2]){
    na_ind <- any(is.na(data[1:(no_of_observations - delay[col]), col]))
    if(na_ind){
      stop(paste0("data has NA values outside the ragged edges.")) 
    }
  }
  
  
  # Check for observations in the dataset inside the ragged edges
  obs_ind <- FALSE
  for(col in 1:dim(data)[2]){
    if(delay[col] > 0){
      obs_ind <- !all(is.na(data[(no_of_observations - delay[col] + 1):no_of_observations, col]))
    }
    if(obs_ind){
      stop(paste0("data has observed values inside the ragged edges.")) 
    }
  }
  
  # Mishandling of frequency
  frequency <- checkPositiveSignedParameterVector(frequency, "frequency", no_of_variables)
  if (length(frequency) != no_of_variables || any(!(frequency %in% c(4, 12)))) {
    stop(paste0("frequency has non-conform values. Currently only values 4 (quarterly data) and 12 (monthly data) are supported."))
  }
  
  # Check for mishandling of the Variables of Interest
  if(any(frequency[variables_of_interest] != 4)){
    stop(paste0("Currently, only quarterly target variables are supported."))
  }
  
  # Mishandling max_fcast_horizon
  max_fcast_horizon <- checkPositiveSignedInteger(max_fcast_horizon, "max_fcast_horizon")
  
  # Mishandling of sparse
  sparse <- checkBoolean(sparse, "sparse")
  if(!sparse){
    warning(paste0("sparse is set to FALSE. A dense DFM is used to nowcast. All LARS-EN stopping criteria are ignored."))
  }
  
  # Mishandling max_ar_lag_order
  max_ar_lag_order <- checkPositiveSignedInteger(max_ar_lag_order , "max_ar_lag_order ")
  
  # Mishandling max_predictor_lag_order
  max_predictor_lag_order <- checkPositiveSignedInteger(max_predictor_lag_order , "max_predictor_lag_order ")
  if(max_predictor_lag_order == 0){
    stop(paste0("max_predictor_lag_order must be zero."))
  }
  
  if(month(time(data))[dim(data)[1]] %in% c(3, 6, 9, 12)){
    fcast_horizon <- 0
    target_variable_delay <- delay[variables_of_interest]
    quarterly_delay <- delay[which(frequency == 4)[-variables_of_interest]]
    effective_fcast_horizon <- max_fcast_horizon
  }else if(month(time(data))[dim(data)[1]] %in% c(1, 4, 7, 10)){
    fcast_horizon <- 2
    target_variable_delay <- delay[variables_of_interest] + 3
    quarterly_delay <- delay[which(frequency == 4)[-variables_of_interest]] + 3
    effective_fcast_horizon <- max_fcast_horizon - 1
  }else if(month(time(data))[dim(data)[1]] %in% c(2, 5, 8, 11)){
    fcast_horizon <- 1
    target_variable_delay <- delay[variables_of_interest] + 3
    quarterly_delay <- delay[which(frequency == 4)[-variables_of_interest]] + 3
    effective_fcast_horizon <- max_fcast_horizon - 1
  }
  
  if(sparse){
    SDFM_fit <- twoStepSDFM(data = data[, which(frequency == 12), drop = FALSE], delay = delay[which(frequency == 12)],
                            selected = selected, no_of_factors = no_of_factors,  
                            max_factor_lag_order = max_factor_lag_order, 
                            decorr_errors = decorr_errors, lag_estim_criterion = lag_estim_criterion, 
                            ridge_penalty = ridge_penalty,  lasso_penalty = lasso_penalty, 
                            max_iterations = max_iterations, max_no_steps = max_no_steps,
                            weights = weights, comp_null = comp_null, spca_conv_crit = spca_conv_crit,
                            parallel = parallel, fcast_horizon = fcast_horizon, 
                            jitter = jitter, svd_method = svd_method
    )
  }else{
    SDFM_fit <- twoStepDenseDFM(data = data[, which(frequency == 12), drop = FALSE], delay = delay[which(frequency == 12)],
                                no_of_factors = no_of_factors,  
                                max_factor_lag_order = max_factor_lag_order, 
                                decorr_errors = decorr_errors, lag_estim_criterion = lag_estim_criterion, 
                                comp_null = comp_null, parallel = parallel, 
                                fcast_horizon = fcast_horizon, jitter = jitter
    )
  }
  
  # Prepare the data-set for the forecasting wrapper
  factor_ts <- SDFM_fit$smoothed_factors
  column_names <- c(colnames(data)[which(frequency == 4)], paste0("Factor ", 1:no_of_factors))
  fcast_data <- merge.zoo(data[, which(frequency == 4)], factor_ts)
  colnames(fcast_data) <- column_names
  
  # If data does not start at the second month of the first quarter 
  #   available, add observations at the beginning of the panel: 
  #   This convention is necessary for the aggregation scheme inside
  #   the forecastWrapper for the quarterly data. (see Mariano, R. S., & Murasawa, 
  #   Y. (2003). A new coincident index of business cycles based on monthly and 
  #   quarterly series. Journal of applied Econometrics, 18(4), 427-443.)
  if(!(month(time(fcast_data))[1] %in% c(2, 5, 8, 11))){
    temp_times <- time(fcast_data)[1]
    temp_data <- coredata(fcast_data)
    if(month(time(fcast_data))[1] %in% c(1, 4, 7, 10)){
      temp_data <- rbind(matrix(NaN, 2, dim(temp_data)[2]), temp_data)
      temp_start <- as.Date(time(fcast_data)[1]) %m-% months(2)
      fcast_data <- as.zoo(ts(temp_data, start = c(year(temp_start), month(temp_start)), frequency = 12))
      colnames(fcast_data) <- column_names
      new_no_of_obs <- no_of_observations + 2
    }else if(month(time(fcast_data))[1] %in% c(3, 6, 9, 12)){
      temp_data <- rbind(matrix(NaN, 1, dim(temp_data)[2]), temp_data)
      temp_start <- as.Date(time(fcast_data)[1]) %m-% months(1)
      fcast_data <- as.zoo(ts(temp_data, start = c(year(temp_start), month(temp_start)), frequency = 12))
      colnames(fcast_data) <- column_names
      new_no_of_obs <- no_of_observations + 1
    }
  }
  
  # Split the data set into target variables, quarterly predictors and monthly predictors
  modified_data <- t(coredata(fcast_data))
  target_variables <- modified_data[variables_of_interest, , drop = FALSE]
  quarterly_predictor_ind <- delay[which(frequency == 4)][-variables_of_interest]
  if(length(quarterly_predictor_ind) == 0){
    quarterly_predictors <- NULL
  }else{
    quarterly_predictors <- modified_data[which(frequency == 4)[-variables_of_interest], , drop = FALSE] 
  }
  factors <- modified_data[(dim(modified_data)[1] - no_of_factors + 1):(dim(modified_data)[1]), , drop = FALSE]
  
  forecasts <- forecastWrapper(target_variables = target_variables, quarterly_predictors = quarterly_predictors,
                               factors = factors, target_variable_delay = target_variable_delay, 
                               quarterly_delay = quarterly_delay, lag_estim_criterion = lag_estim_criterion,
                               max_fcast_horizon = effective_fcast_horizon, max_ar_lag_order = max_ar_lag_order,
                               max_predictor_lag_order = max_predictor_lag_order, jitter = jitter)
  
  # Create nice result object
  result <- list()
  
  # Store the forecast results
  result[[1]] <- list()
  names(result)[1] <- "Forecasts"
  
  # Store forecasts together with the quarterly target series
  qtrly_series <- data[which(month(time(data)) %in% c(3, 6, 9, 12)), variables_of_interest, drop = FALSE]
  forecast_and_series <- matrix(NaN, dim(qtrly_series)[1] + max_fcast_horizon, 2 * length(variables_of_interest))
  colnames(forecast_and_series) <- paste0(1:(2 * length(variables_of_interest)))
  for(n in 1:length(variables_of_interest)){
    forecast_and_series[1:dim(qtrly_series)[1], 2*n - 1] <- qtrly_series[, n]
    forecast_and_series[(dim(qtrly_series)[1] - floor(delay[variables_of_interest[n]] / 3) + 1):dim(forecast_and_series)[1], 2*n] <-
      forecasts$`Avg. Point Forecast`[n, (dim(forecasts$`Avg. Point Forecast`)[2] - floor(delay[variables_of_interest[n]] / 3) - max_fcast_horizon + 1):dim(forecasts$`Avg. Point Forecast`)[2]]
    colnames(forecast_and_series)[2*n - 1] <- colnames(data)[variables_of_interest[n]]
    colnames(forecast_and_series)[2*n] <- paste0("Fcast ", colnames(data)[variables_of_interest[n]])
  }
  result$Forecasts <- as.zoo(ts(forecast_and_series, start = c(year(time(data)[1]), 
                                                               quarter(time(data)[1])), frequency = 4))
  
  result$`Single Predictor Forecasts` <- list()
  for(i in 1:length(variables_of_interest)){
    result$`Single Predictor Forecasts`[[i]] <- 
      as.zoo(ts(t(forecasts[[i]]), 
                end = c(year(time(result$Forecasts)[dim(result$Forecasts)[1]]),
                        quarter(time(result$Forecasts)[dim(result$Forecasts)[1]])),
                frequency = 4
      ))
    names(result$`Single Predictor Forecasts`)[i] <- names(forecasts)[i]
  }
  
  
  result$`SDFM Fit` <- SDFM_fit
  result$call <- func_call
  class(result) <- "SDFMnowcast"
  return(result)
}

#' @name print.SDFMnowcast
#' @title Generic print function for SDFMnowcast S3 objects
#' 
#' @param x `SDFMnowcast` object.
#' @param ... Additional parameters for the plotting functions.
#'
#' @return
#' No return value; Prints a summary to the console.
#' 
#' @author
#' Domenic Franjic
#' 
#' @export
print.SDFMnowcast <- function(x, ...) {
  print(tail(x$Forecasts, 10))
  invisible(x)
}


#' @name plot.SDFMnowcast
#' @title Generic plotting function for SDFMnowcast S3 objects
#' @param x `SDFMnowcast` object.
#' @param axis_text_size Numeric size of x- and y-axis labels. Prased to ggplot2 
#' `theme(..., text = element_text(size = axis_text_size))`.
#' @param ... Additional parameters for the plotting functions.
#' 
#' @return
#' A named list storing of `ggplot` objects:
#' \describe{
#'   \item{`Single Pred. Fcast Density Plots x`}{`patchwork` / `ggplot` objects
#'   graphing the distribution of forecasts generated by the predictors for
#'   each prediction (backcasts, nowcasts, forecasts) for each target,
#'   respectively. Altogether, there will be as many such objects as there are
#'   targets, with `x` replaced by the column name of the target.}
#' }
#' 
#' @author
#' Domenic Franjic
#' 
#' @export
plot.SDFMnowcast <- function(x,                         
                             axis_text_size = 20,
                             ...) {
  out_list <- list()
  
  # Single Predictor Density Plots
  absolute_fcast_date <- time(x$`SDFM Fit`$data)[dim(x$`SDFM Fit`$data)[1]]
  if(dim(x$`Single Predictor Forecasts`[[1]])[2] > 2){
    for(h in 1:(dim(x$Forecasts)[2] / 2)){
      current_single_pred_raw <- x$`Single Predictor Forecasts`[[h]]
      current_single_pred <- na.omit(t(coredata(current_single_pred_raw)))
      
      plot_list <- list()
      for(horizon in 1:dim(current_single_pred)[2]){
        data_df <- data.frame(
          Series = rownames(current_single_pred),
          Value = as.numeric(current_single_pred[, horizon])
        )
        
        data_df$Series <- factor(data_df$Series, levels = rownames(current_single_pred))
        relative_fcast_date <- as.yearqtr(time(current_single_pred_raw)[horizon])
        current_horizon <- 4 * (relative_fcast_date - as.yearqtr(absolute_fcast_date))
        max_density <- max(density(data_df$Value)$y)
        current_density_plot <- ggplot(data_df, aes(x = Value)) +
          geom_density(fill = "#88ccee", alpha = 0.6, color = "#332288") +
          geom_vline(xintercept = mean(data_df$Value), colour = "#882255",
                     lty = 1) +
          geom_text(label = "Mean", y = max_density * 0.7, 
                    x = mean(data_df$Value ) + 0.1 * sqrt(var(data_df$Value)),
                    colour = "#882255") +
          geom_vline(xintercept = median(data_df$Value), colour = "#117733",
                     lty = 2) +
          geom_text(label = "Median", y = max_density * 0.8, 
                    x = median(data_df$Value ) - 0.12 * sqrt(var(data_df$Value)),
                    colour = "#117733") + 
          geom_point(aes(y = 0), size = 2) +
          geom_text(aes(label = Series, y  = 0.00), nudge_y = max_density * 0.2,
                    size = 3.5, angle = 90, color = "black") +
          labs(title = paste0(ifelse(current_horizon >= 0,
                                     ifelse(current_horizon == 0,
                                            "Nowcast", paste0(current_horizon, "-step ahead Forecast")),
                                     paste0(-current_horizon, "-step back Backcast")),
                              " for ", relative_fcast_date),
               x = "Predicted Value", y = ""
          ) +
          theme_minimal() + 
          theme(text = element_text(size = axis_text_size))
        plot_list[[horizon]] <- current_density_plot
      }
      out_list[[h]] <- patchwork::wrap_plots(plot_list, ncol = 2)
      names(out_list)[h] <- paste0("Single Pred. Fcast Density Plots ", colnames(x$Forecasts)[2 * h - 1])
    }
  }
  
  return(out_list)
}

test_that("", {
  
  # Load data
  data("mixed_freq_factor_model")
  data_zoo <- scale(mixed_freq_factor_model$data)
  no_of_factors <- dim(mixed_freq_factor_model$factors)[2]
  no_of_mtly_vars <- sum(mixed_freq_factor_model$frequency == 12)
  delay <- mixed_freq_factor_model$delay
  frequency <- mixed_freq_factor_model$frequency
  max_ar_lag_order <- 5
  max_predictor_lag_order <- 5
  variables_of_interest <- 1:2
  max_fcast_horizon <- 4
  selected <- rep(round(no_of_mtly_vars * 0.5), no_of_factors)
  
  # Parsing zoo object
  expect_silent(nowcast(data = data_zoo, variables_of_interest = variables_of_interest, 
                        max_fcast_horizon = 0, delay = delay, selected = selected,
                        frequency = frequency, no_of_factors = no_of_factors))
  
})

---
title: "Introduction to TwoStepSDFM"
output: rmarkdown::html_vignette
bibliography: "../inst/REFERENCES.bib"
link-citations: true
vignette: >
 %\VignetteIndexEntry{Introduction to TwoStepSDFM}
 %\VignetteEngine{knitr::rmarkdown}
 %\VignetteEncoding{UTF-8}
---
 
 
```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE,
                      comment = "#>",
                      fig.width = 8,
                      fig.height = 5,
                      dev = "png",
                      dpi = 150,
                      out.width = "100%"
)
```

```{r setup, include = FALSE}
library(TwoStepSDFM)
```


# Overview

The `TwoStepSDFM` package provides a fast and memory-efficient implementation of the cross-validation, prediction[^pred_footnote], and two-step estimation procedure for mixed-frequency *sparse dynamic factor models* (SDFM) outlined in @franjic2024nowcasting. The routines in this package also directly handle heterogeneous publication lags of the data, which can lead to missing observations at the end of the observation period (so-called *ragged edges*).

We estimate the sparse loading matrix via *sparse principal components analysis* (SPCA) [@zou2006sparse]. The latent states are then filtered via a univariate representation of the *Kalman Filter and Smoother* (KFS) as described in @koopman2000fast.

Our approach is a direct generalisation of the classical two-step estimator for dense DFMs described in @Giannone2008Nowcasting and @Doz2011Two_step. Differing from the classical model, however, we allow regularisation to shrink some of the factor loadings towards or directly to zero. This is achieved by elastic-net type size constraints introduced to the PCA's least squares problem according to @zou2006sparse. We also explicitly account for cross-sectional correlation in the measurement error. For more details see @franjic2024nowcasting.

[^pred_footnote]: Note that, due to the mixed-frequency structure, prediction here includes backcasts, nowcasts, and forecasts of the target variable.

# Main Features

- **Fast Model Simulation**: The `simFM()` function provides a flexible framework to simulate mixed-frequency data with ragged edges from an approximate DFM.
- **Estimation of the Number of Factors**: The `noOfFactors()` function uses the @onatski2009testing procedure to estimate the number of factors efficiently while providing good finite sample performance.
- **Fast Model Estimation**: The `twoStepSDFM()` function provides a fast, memory-efficient, and convenient implementation of the two-step estimator outlined in @franjic2024nowcasting.
- **Fast Hyper-Parameter Cross-Validation**: The `crossVal()` function provides a fast and parallel cross-validation wrapper to retrieve the optimal hyper-parameters using time-series cross-validation [@rob2018forecasting] with random hyper-parameter search [@bergstra2012random].
- **Fast Model Prediction**: The `nowcast()` function is a highly convenient prediction function for backcasts, nowcasts, and forecasts of multiple targets. It automatically takes care of all issues arising with mixed-frequency data and ragged edges.
- **Compatibility**: All functions take advantage of `C++` for enhanced speed and memory-efficiency.

# Side Features

- **Fast dense DFM estimation and prediction**: The `nowcast()` function is also able to produce predictions of a dense DFM according to @Giannone2008Nowcasting. The function `twoStepDenseDFM()` additionally exposes an estimation procedure for the dense two-step estimator.

- **Fast SPCA**: `sparsePCA()` exposes the internal `C++`-backed SPCA routine in `R`. This provides access to a fast and memory-efficient SPCA estimation routine as implemented by @zou2020elnet in pure `R`.

- **Fast Kalman Filter and Smoother**: The `kalmanFilterSmoother()` function exposes the internal `C++`-backed KFS routine.

# Example Usage

Below, we will outline the usage of the main features of this package. To this end, we start by simulating what we refer to as a *pseudo mixed-frequency data vintage*.[^pmfdv] This vintage includes a set of stationary mixed-frequency variables, a vector indicating the variables' frequencies, and a vector indicating the variables' delays. Afterwards, we go over a step-by-step example on how to validate the hyper-parameters, estimate the model parameters, and predict using the data vintage.

[^pmfdv]: We use the term *pseudo* as real-world vintages often contain data that is not necessarily stationary. These variables often rely on an individual transformation code. As the package does not provide means of automatically rendering data stationary, we assume all the data to be stationary. Users are thus advised to transform their data separately prior to the analysis.

## Simulate Data

We start our exposé by simulating data from an approximate DFM with measurement equation 
$$
\boldsymbol{x}_t = \boldsymbol{\Lambda} \boldsymbol{f}_{t} + \boldsymbol{\xi}_t,\quad \boldsymbol{\xi}_t \sim \mathcal{N}(\boldsymbol{\mu}, \boldsymbol{\Sigma}_{\xi}),
$$
and transition equation
$$
\boldsymbol{f}_t = \sum_{p=1}^P\boldsymbol{\Phi}_p \boldsymbol{f}_{t-p} + \boldsymbol{\epsilon}_t,\quad \boldsymbol{\epsilon}_t \sim \mathcal{N}(\boldsymbol{0}, \boldsymbol{\Sigma}_{f}), 
$$
for $t = 1, \dots,T$. The vector $\boldsymbol{x}_t := (x_{n,t})_{n=1}^{N}$ represents the vector of all *monthly* variables, including those which will be *aggregated into a quarterly representation* (quarterfied) later. $\boldsymbol{\Lambda} := (\lambda_{n,r})_{n,r=1}^{N,R}$ is the factor loading matrix that distributes the dynamics of the $R$ factors, represented by $\boldsymbol{f}_t := (f_{r,t})_{r=1}^R$, onto the data. $\boldsymbol{\Phi}_p:=(\phi_{r,s,p})_{r,s=1}^{R,R}$ is the $p$th transition VAR coefficient matrix. The vectors $\boldsymbol{\xi}_t := (\xi_{n,t})_{n=1}^N$ and $\boldsymbol{\epsilon}_t := (\epsilon_{r,t})_{r=1}^R$ represent the measurement and transition error, respectively. $\boldsymbol{\mu} := (\mu_{n})_{n=1}^N$ and $\boldsymbol{\Sigma}_{\xi}:=(\sigma_{\xi;n,m})_{n,m=1}^{N,N}$ represent the measurement error mean and measurement error variance-covariance matrix, respectively, and $\boldsymbol{\Sigma}_{f}:=(\sigma_{f;r,s})_{r,s=1}^{R,R}$ represents the transition error variance-covariance matrix.[^trans_mean]

As our goal is to simulate mixed-frequency data, we will quarterfy five variables, i.e. 10% of all variables simulated below. We do so by using the geometric mean as described in @Mariano2003new_coincident:
$$
\boldsymbol{x}^*_{t}:= \frac{1}{3}\left(\boldsymbol{x}_{t} + 2\boldsymbol{x}_{t-1} + 3\boldsymbol{x}_{t-2} + 2\boldsymbol{x}_{t-3} + \boldsymbol{x}_{t-4}\right)
$$
for $t=3, 6, \dots,T - \mathrm{mod}(T,3)$. Note that this implies that the simulated data will always start in the first month of the corresponding quarter. The `simFM()` function will therefore check the `starting_date` when data gets quarterfied and turned into a `zoo` object.

Lastly, we also want to impose a ragged edge structure onto the data. The ragged edges are set via the `delay` parameter. Note that the `delay` parameter differs from the `delay` parameter of the other estimation functions below. In the case of `simFM()`, `delay` refers to the number of months for monthly data and the number of quarters for quarterly data that will be missing. For example, consider `delay <- c(1, 1)` and assume the variable with index `1` will be quarterfied. In that case, the variable with index `1` will be delayed by 1 quarter, i.e., it will be missing 3 observations at the end of the panel. The variable with index `2` will be delayed by 1 month, i.e., it will be missing 1 observation at the end of the panel. In all other instances, including the `delay` object returned by `simFM()`, `delay` represents the number of months since the most recent publication.

As a final remark, note that all RNG calls happen inside `C++`. Therefore, `set.seed()` will not affect the results of `simFM()`. Rather all random processes are governed by the `seed` argument provided to the function. The `set.seed()` call in the chunk below is only used for the reproducibility of the generation of the DFM parameters. 

[^trans_mean]: Note that the choice of a zero-mean transition error is deliberate. As we work with a stationary transition equation, for every transition error mean $\boldsymbol{\nu}$ it holds that $\mathbb{E}(\boldsymbol{x}_t)=\boldsymbol{\mu} + \boldsymbol{\Lambda}\left(\boldsymbol{I} - \sum_{p=1}^P\boldsymbol{\Phi}_p\right)^{-1}\boldsymbol{\nu}$. Thus, simulating data from a state-space with measurement error mean $\boldsymbol{\mu}$ and transition error $\boldsymbol{\nu}$ is equivalent to simulating data from a linear state space with measurement error $\boldsymbol{\mu}^* := \boldsymbol{\mu} + \boldsymbol{\Lambda}\left(\boldsymbol{I} - \sum_{p=1}^P\boldsymbol{\Phi}_p\right)^{-1} \boldsymbol{\nu}$. Furthermore, for the purpose of the analysis, we will scale the data and thus explicitly assume that the transition error is zero mean.

```{r simulation, eval = TRUE, message = FALSE}

seed <- 02102025
set.seed(seed)
no_of_obs <- 300
no_of_vars <- 50
no_of_factors <- 3
trans_error_var_cov <- diag(1, no_of_factors)
loading_matrix <- matrix(rnorm(no_of_vars * no_of_factors), no_of_vars, no_of_factors)
meas_error_mean <- rep(0, no_of_vars)
meas_error_var_cov <- diag(1, no_of_vars)
trans_var_coeff <- cbind(diag(0.5, no_of_factors), -diag(0.25, no_of_factors))
factor_lag_order <- 2
delay <- c(2, 1, floor(rexp(no_of_vars - 2, 1)))
quarterfy <- TRUE
quarterly_variable_ratio <- 0.1
corr <- TRUE
beta_param <- 2
burn_in <- 999
starting_date <- "1970-01-01"
rescale <- FALSE
check_stationarity <- TRUE
stationarity_check_threshold <- 1e-10
factor_model <- simFM(no_of_obs = no_of_obs, no_of_vars = no_of_vars,
                      no_of_factors = no_of_factors, loading_matrix = loading_matrix,
                      meas_error_mean = meas_error_mean, meas_error_var_cov = meas_error_var_cov,
                      trans_error_var_cov = trans_error_var_cov, trans_var_coeff = trans_var_coeff,
                      factor_lag_order = factor_lag_order, delay = delay, quarterfy = quarterfy,
                      quarterly_variable_ratio  = quarterly_variable_ratio, corr = corr,
                      beta_param = beta_param, seed = seed, burn_in = burn_in, 
                      starting_date = starting_date, rescale = rescale, 
                      check_stationarity = check_stationarity, 
                      stationarity_check_threshold = stationarity_check_threshold)
spca_plots <- plot(factor_model, axis_text_size = 10, legend_title_text_size = 5)
spca_plots$`Data Plots`
spca_plots$`Factor Time Series Plots`
spca_plots$`Loading Matrix Heatmap`
```

## Estimate the Number of Factors

In the next step, we need to estimate the number of factors. We do so by using the procedure described in @onatski2009testing.[^estim_no_fact] Very loosely, this test infers the number of factors by looking at the maximum slopes between the eigenvalues of a special data Gram matrix. For more information on the procedure, see the literature cited above. Note that as mentioned above, data should be standardised prior to the analysis. Also, only provide monthly observations to `noOfFactors()`, as otherwise results might get skewed.

The @onatski2009testing procedure is a repeated testing procedure with non-standard distributions of the slope coefficients of the eigenvalues. This distribution depends on the number of eigenvalues considered in the test. Therefore, a minimum and maximum number of factors must be provided. These values should be close enough such that the test has enough power, but also far enough apart such that we do not reject too early due to hitting the maximum number of factors. When working with this test, we found that using `max_no_factors - min_no_factors = 1` works quite well. 

We also need to set a confidence threshold as a stopping rule of the testing procedure. In general, for finite samples, the @onatski2009testing procedure works quite well and in our experience better than information-based criteria, especially when working with data simulated from an SDFM with a high degree of sparsity. However, we also found that in some cases, the testing procedure seems to underestimate the number of factors if the threshold is set too strictly. Users should therefore set `confidence_threshold` comparatively large if the cost of underestimating the number of factors, i.e., including too few factors, is large.

[^estim_no_fact]: We also provide the @bai2002determining information-criterion based method to determine the number of factors in our package. The ICs are computed automatically when calling `noOfFactors()`, and the results are provided along with the results of the @onatski2009testing testing procedure.

```{r no_of_fact_test, eval = TRUE, message = FALSE}
mtly_data <- scale(factor_model$data[, which(factor_model$frequency == 12)])
no_of_fact_test <- noOfFactors(mtly_data, min_no_factors = 1, max_no_factors = 7,
                                  confidence_threshold = 0.2)
print(no_of_fact_test)
```

## Cross-Validate the Hyper-Parameters

With an estimate of the number of factors at hand, we proceed to infer the hyper-parameters. Just as above, we standardise the data prior to the analysis. Here, however, we provide the full mixed-frequency data set, as the function handles the issues of the mixed-frequency structure internally.

Our cross-validation routine is a random hyper-parameter search [@bergstra2012random] time-series cross-validation [@rob2018forecasting] scheme. This means that the candidate values for the hyper-parameters are drawn randomly.[^seed_call_cross_val] Therefore, we do not provide a grid of parameter candidates but the bounds of the distribution governing the hyper-parameter candidates. For the global ridge penalty, candidates are drawn as $\exp(u)$, where $u$ is uniformly distributed between `min_ridge_penalty` and `max_ridge_penalty`.

For the factor-specific lasso penalties, it is necessary to first choose a stopping criterion. The internal SPCA problem, which is used to regularise the loading matrix, is solved via a LARS-EN type algorithm found in @zou2020elnet. This algorithm has the advantage of providing three different stopping criteria: `"selected"`, `"penalty"`, and `"steps"`. These criteria represent different conceptual approaches, but for cross-validation purposes, the choice between them is typically non-critical as they serve the same predictive goal. In the example below, we use `"selected"`.[^stopping_crit_detail] When using `"selected"`, the $\ell_1$ size constraints are expressed in terms of the number of non-zero loadings of the corresponding loading matrix column. Candidates are therefore drawn from a discrete uniform distribution over the natural numbers. The upper and lower bounds of this distribution are governed by the first and second entry of `min_max_penalty`, respectively.

[^seed_call_cross_val]: Even though these random candidates are not drawn within `C++`, the function still requires a `seed` parameter. This is done for reproducibility, as we store the provided `seed` inside the function call object of the return object of `crossVal()`.

[^stopping_crit_detail]: For more details on the different stopping criteria, see the help page of `crossVal()` and `sparsePCA()`.

```{r cross_val, eval = TRUE, message = FALSE}
no_of_mtly_vars <- sum(factor_model$frequency == 12)
all_data <- scale(factor_model$data)
cross_val_res <- crossVal(data = all_data, variable_of_interest = 1, fcast_horizon = 0,
                                 delay = factor_model$delay, frequency = factor_model$frequency, 
                                 no_of_factors = no_of_fact_test$test$no_of_factors, seed = seed, 
                                 min_ridge_penalty = 1e-6, max_ridge_penalty = 1e+5, cv_repetitions = 5,
                                 cv_size = 100, lasso_penalty_type = "selected", min_max_penalty = c(5, no_of_mtly_vars),
                                 verbose = FALSE)
print(cross_val_res)
cross_val_plot <- plot(cross_val_res)
cross_val_plot$`CV Results`
```

## Estimate the Parameters of the SDFM

Before turning to the main prediction routine, we want to focus on the estimation of the model parameters first. While written with prediction as main focus, our package also provides the means to estimate the model parameters. For an SDFM, this is handled by the `twoStepSDFM()` function below. Its usage is straightforward. Note, however, that the function only accepts monthly data.

```{r estim_sparse_dfm, eval = TRUE, message = FALSE}
mtly_data <- scale(factor_model$data[, which(factor_model$frequency == 12)])
mtly_delay <- factor_model$delay[which(factor_model$frequency == 12)]
inferred_no_of_selected <- cross_val_res$CV$`Min. CV`[3:(3 + no_of_fact_test$test$no_of_factors - 1)]
inferred_ridge_penalty <- cross_val_res$CV$`Min. CV`[1]
sdfm_fit <- twoStepSDFM(data = mtly_data, delay = mtly_delay, selected = inferred_no_of_selected, 
                        no_of_fact_test$test$no_of_factors, ridge_penalty = inferred_ridge_penalty)
print(sdfm_fit)
sdfm_fit_plot <- plot(sdfm_fit)
sdfm_fit_plot$`Factor Time Series Plots`
sdfm_fit_plot$`Loading Matrix Heatmap`
```

## Predict Target Variables

Finally, we turn to the nowcasting routine. Fundamentally, this routine is an unrestricted factor augmented MIDAS model as described in @marcellino2010factor. For each target, we use each other target, each additional quarterly predictor, and quarterfied factor, and predict the target via an individual (AR)DL model. The exact autoregressive and predictor lag order are inferred internally via the BIC for each equation. The final forecast is then given by the unweighted mean of all individual (AR)DL model forecasts.

After cross-validation, we are set to directly predict the target variables. Here, just as with `crossVal()`, we are also able to provide the complete mixed-frequency data set. Note that, in contrast to the cross-validation function, it is possible to predict multiple targets across multiple horizons at once. One should keep in mind, however, that the optimal hyper-parameters are retrieved for a specific target and horizon (in our case the nowcast of the first quarterly variable). Therefore, in a serious prediction exercise, one should always tune the hyper-parameters for each target and horizon specifically.[^nowcast_flex_info] 

To show off the abilities of this function, however, we still predict multiple targets and horizons. The first target is lagged by two quarters, while the second is lagged by only one. The maximum forecasting horizon is set to two. This means that for both targets we will forecast two steps ahead. For target one we will additionally compute a backcast and a nowcast. For target two only a nowcast will be computed. In general, `max_fcast_horizon` governs the number of step-ahead forecasts computed. nowcasts and backcasts are automatically computed depending on the delay of the targets. 

[^nowcast_flex_info]: We still wrote the `nowcast()` function in such a flexible way as it is highly useful for quick assessments especially at the beginning when working with new data.

```{r predict, eval = TRUE, message = FALSE}
all_data <- scale(factor_model$data)
inferred_no_of_selected <- cross_val_res$CV$`Min. CV`[3:(3 + no_of_fact_test$test$no_of_factors - 1)]
inferred_ridge_penalty <- cross_val_res$CV$`Min. CV`[1]
nc_results <- nowcast(data = all_data, variables_of_interest = c(1, 2), max_fcast_horizon = 2, 
                      delay = factor_model$delay, ridge_penalty = inferred_ridge_penalty, selected = inferred_no_of_selected, frequency = factor_model$frequency, no_of_factors = no_of_fact_test$test$no_of_factors)
nc_plot <- plot(nc_results)
nc_plot$`Single Pred. Fcast Density Plots Series 1`
nc_plot$`Single Pred. Fcast Density Plots Series 2`
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Forecaster.R
\name{forecastWrapper}
\alias{forecastWrapper}
\title{Internal forecasting wrapper function}
\usage{
forecastWrapper(
  target_variables,
  quarterly_predictors,
  factors,
  target_variable_delay,
  quarterly_delay,
  lag_estim_criterion,
  max_fcast_horizon,
  max_ar_lag_order,
  max_predictor_lag_order,
  jitter
)
}
\description{
This function is for internal use only and may change in future releases
without notice. Users should use \code{nowcast()} instead for a stable and
supported interface.
Helper function to check parameter vectors for positive signed integer values
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PlotterHelper.R
\name{plotMeasVarCovHeatmap}
\alias{plotMeasVarCovHeatmap}
\title{Helper for plotting measurement error var.cov. heatmap}
\usage{
plotMeasVarCovHeatmap(
  measurement_error_var_cov_df,
  series_names,
  axis_text_size,
  legend_title_text_size
)
}
\description{
Helper for plotting measurement error var.cov. heatmap
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CrossVal.R
\name{nowcastSpecificationHelper}
\alias{nowcastSpecificationHelper}
\title{Helper function to wrap the nowcasting routine}
\usage{
nowcastSpecificationHelper(
  cv_repetitions,
  no_of_factors,
  no_of_variables,
  no_of_observations,
  no_of_mtly_variables,
  lasso_penalty_type,
  data,
  variable_of_interest,
  fcast_horizon,
  delay,
  candidates,
  frequency,
  max_factor_lag_order,
  decorr_errors,
  lag_estim_criterion,
  max_iterations,
  comp_null,
  spca_conv_crit,
  max_ar_lag_order,
  max_predictor_lag_order,
  jitter,
  svd_method,
  weights
)
}
\description{
Helper function to wrap the nowcasting routine
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Helper.R
\name{checkPositiveSignedParameterVector}
\alias{checkPositiveSignedParameterVector}
\title{Helper function to check parameter vectors for positive signed integer values}
\usage{
checkPositiveSignedParameterVector(parameter, name, size)
}
\description{
Helper function to check parameter vectors for positive signed integer values
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EstimateFactors.R
\name{plot.NoOfFactorsFit}
\alias{plot.NoOfFactorsFit}
\title{Generic plotting function for NoOfFactorsFit S3 objects}
\usage{
\method{plot}{NoOfFactorsFit}(x, axis_text_size = 20, legend_title_text_size = 20, ...)
}
\arguments{
\item{x}{\code{NoOfFactorsFit} object.}

\item{axis_text_size}{Numeric size of x- and y-axis labels. Prased to ggplot2
\code{theme(..., text = element_text(size = axis_text_size))}.}

\item{legend_title_text_size}{Numeric size of x- and y-axis labels. Prased to
ggplot2
\code{theme(..., legend.title = element_text(size = legend_title_text_size))}.}

\item{...}{Additional parameters for the plotting functions.}
}
\value{
A named list of plot objects:
\describe{
\item{\verb{Eigen Value Plot}}{\code{ggplot} object showing a bar plot of the
eigenvalues of the complex data Gram matrix.}
}
}
\description{
Create diagnostic plots for an \code{NoOfFactorsFit} object,
}
\author{
Domenic Franjic
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TwoStepSDFM.R
\name{predict.SDFMFit}
\alias{predict.SDFMFit}
\title{Generic plotting function for SDFMFit S3 objects}
\usage{
\method{predict}{SDFMFit}(object, horizon = 0, ...)
}
\arguments{
\item{object}{\code{SDFMFit} object.}

\item{horizon}{Number of forecasting steps into the future. Must be smaller
than or equal to \code{x$factor_fcast_horizon}.}

\item{...}{Additional parameters for the prediction function.}
}
\value{
A named list of plot objects:
\describe{
\item{data}{Object containing the original data. The object inherits its
class from \code{object$data}: If data is provided as \code{zoo}, \code{data} will be a
\code{zoo} object. If \code{data} is provided as matrix, \code{data} will be a
(\code{no_of_factors}\eqn{\times}{x}\code{no_of_obs}) matrix.}
\item{data_missing_pred}{Object containing only the predictions of all
missing observations plus the forecasts. Inherits its class from
\code{object$data} as above.}
\item{data_imputed}{Object containing the observed data, predictions of
all missing observations plus the forecasts. Inherits its class from
\code{object$data} as above.}
}
}
\description{
Predict all missing observations due to ragged edges in the data set plus
horizon steps ahead.
}
\examples{
data(factor_model)
no_of_vars <- dim(factor_model$data)[2]
no_of_factors <- dim(factor_model$factors)[2]
sdfm_fit <- twoStepSDFM(data = factor_model$data, delay = factor_model$delay,
                        selected = rep(floor(0.5 * no_of_vars), no_of_factors),
                        no_of_factors = no_of_factors, fcast_horizon = 5)
dfm_fit <- twoStepDenseDFM(data = factor_model$data, delay = factor_model$delay, 
                           no_of_factors = no_of_factors, fcast_horizon = 5)
predict(sdfm_fit, horizon = 5)
predict(dfm_fit, horizon = 5)

}
\author{
Domenic Franjic
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CrossVal.R
\name{plot.SDFMcrossVal}
\alias{plot.SDFMcrossVal}
\title{Generic plotting function for SDFMcrossVal S3 objects}
\usage{
\method{plot}{SDFMcrossVal}(x, axis_text_size = 20, legend_title_text_size = 20, ...)
}
\arguments{
\item{x}{\code{SDFMcrossVal} object.}

\item{axis_text_size}{Numeric size of x- and y-axis labels. Prased to ggplot2
\code{theme(..., text = element_text(size = axis_text_size))}.}

\item{legend_title_text_size}{Numeric size of x- and y-axis labels. Prased to
ggplot2
\code{theme(..., legend.title = element_text(size = legend_title_text_size))}.}

\item{...}{Additional parameters for the plotting functions.}
}
\value{
A named list of \code{ggplot} objects:
\describe{
\item{\verb{CV Results}}{\code{ggplot} object of the cross-validation error against
the log Ridge penalty. The overall sparsity level of the loading matrix
induced by the lasso penalty is indicated by point shapes and colours.}
\item{\verb{BIC Results}}{\code{ggplot} object of the BIC against the log Ridge
penalty. The overall sparsity level of the loading matrix induced by the
lasso penalty is indicated by point shapes and colours.}
}
}
\description{
Generic plotting function for SDFMcrossVal S3 objects
}
\author{
Domenic Franjic
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Helper.R
\name{checkPositiveSignedInteger}
\alias{checkPositiveSignedInteger}
\title{Helper function to check unsigned integer function parameter}
\usage{
checkPositiveSignedInteger(parameter, name, bit_size = 64)
}
\description{
Helper function to check unsigned integer function parameter
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/KalmanFilterSmoother.R
\name{print.KFSFit}
\alias{print.KFSFit}
\title{Generic printing function for KFSFit S3 objects}
\usage{
\method{print}{KFSFit}(x, ...)
}
\arguments{
\item{x}{\code{KFSFit} object.}

\item{...}{Additional parameters.}
}
\value{
No return value, called for side effects.
}
\description{
Print a compact summary of a \code{KFSFit} object.
}
\author{
Domenic Franjic
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SimFM.r
\name{simFM}
\alias{simFM}
\title{Simulate Dynamic Factor Models.}
\usage{
simFM(
  no_of_obs,
  no_of_vars,
  no_of_factors,
  loading_matrix,
  meas_error_mean,
  meas_error_var_cov,
  trans_error_var_cov,
  trans_var_coeff,
  factor_lag_order,
  delay = NULL,
  quarterfy = FALSE,
  quarterly_variable_ratio = 0,
  corr = FALSE,
  beta_param = Inf,
  seed = 20022024,
  burn_in = 1000,
  rescale = TRUE,
  starting_date = NULL,
  check_stationarity = FALSE,
  stationarity_check_threshold = 1e-05,
  parallel = FALSE
)
}
\arguments{
\item{no_of_obs}{Integer number of observations.}

\item{no_of_vars}{Integer number of Variables.}

\item{no_of_factors}{Integer number of factors.}

\item{loading_matrix}{Numeric (\code{no_of_vars} \eqn{\times}{x} \code{no_of_factors})
loading matrix.}

\item{meas_error_mean}{Numeric vector of the means of the measurement errors.}

\item{meas_error_var_cov}{Numeric (\code{no_of_vars} \eqn{\times}{x}
\code{no_of_vars}) variance-covariance matrix of the measurement errors.}

\item{trans_error_var_cov}{Numeric (\code{no_of_factors} \eqn{\times}{x}
\code{no_of_factors}) variance-covariance matrix of the transition errors.}

\item{trans_var_coeff}{Either a list of length \code{max_factor_lag_order} with
each entry a numeric (\code{no_of_factors} \eqn{\times}{x} \code{no_of_factors}) VAR
coefficient matrix or a matrix of dimensions (\code{no_of_factors} \eqn{\times}{x}
(\code{no_of_factors * max_factor_lag_order})) holding the VAR coefficients of the
factor VAR process in each (\code{no_of_factors} \eqn{\times}{x} \code{no_of_factors})
block.}

\item{factor_lag_order}{Integer order of the VAR process in the transition
equation.}

\item{delay}{Integer vector of delays imposed onto the end of the data
(ragged edges).}

\item{quarterfy}{Logical, whether or not some of the data should be
aggregated to quarterly representations.}

\item{quarterly_variable_ratio}{Ratio of variables ought to be quarterfied.}

\item{corr}{Logical, whether or not the measurement error should be randomly
correlated inside the function using a random correlation matrix with
off-diagonal elements governed by a beta-distribution.}

\item{beta_param}{Parameter of the beta-distribution governing the
off-diagonal elements of the variance-covariance matrix of the measurement
error.}

\item{seed}{32-bit unsigned integer seed for all random processes inside the
function.}

\item{burn_in}{Integer burn-in period of the simulated data ought to be
discarded at the beginning of the sample.}

\item{rescale}{Logical, whether or not the variance of the measurement error
should be rescaled by the common component to equalise the
signal-to-noise ratio.}

\item{starting_date}{A date type object indicating the start of the dataset.
If NULL (default), the function returns matrices with observations along the
second dimension (i.e., time in columns). If specified, the function treats
the data as a time series and returns a \code{zoo} object.}

\item{check_stationarity}{Logical, whether or not the stationarity properties
of the factor VAR process should be checked.}

\item{stationarity_check_threshold}{Threshold of the stationarity check for
when to deem an eigenvalue numerically negative.}

\item{parallel}{Logical, make use of Eigen internal parallel matrix
operations.}
}
\value{
Returns a \code{SimulData} containing the following elements:
\describe{
\item{data}{If \code{starting_date} is provided, a \code{zoo} object, else,
a (\code{no_of_vars} \eqn{\times}{x} \code{no_of_obs}) numeric matrix holding the
simulated data.}
\item{factors}{If \code{starting_date} is provided, a \code{zoo} object, else a
(\code{no_of_factors} \eqn{\times}{x} \code{no_of_obs}) numeric matrix holding  the
simulated latent factors.}
\item{trans_var_coeff}{Numeric (\code{no_of_factors} \eqn{\times}{x}
(\code{no_of_factors} * \code{factor_lag_order})) factor VAR coefficient matrix.}
\item{loading_matrix}{Numeric factor loading matrix.}
\item{meas_error}{If \code{starting_date} is provided, a \code{zoo} object, else
a (\code{no_of_vars} \eqn{\times}{x} \code{no_of_obs}) numeric matrix holding the
fundamental measurement errors.}
\item{meas_error_var_cov}{Numeric measurement error
variance-covariance matrix.}
\item{trans_error_var_cov}{Numeric transition error
variance-covariance matrix.}
\item{frequency}{Integer vector of variable frequencies.}
\item{delay}{Integer vector of variable delays, measured as the number of
months since the latest available observation.}
}
}
\description{
Simulate data from a linear Gaussian state-space model (latent factor model),
with measurement equation
\deqn{
  \bm{x}_t = \bm{\Lambda} \bm{f}_{t} + \bm{\xi}_t,\quad \bm{\xi}_t \sim \mathcal{N}(\bm{\mu}, \bm{\Sigma}_{\xi}),
}
and transition equation
\deqn{
  \bm{f}_t = \sum_{p=1}^P\bm{\Phi}_p \bm{f}_{t-p} + \bm{\epsilon}_t,\quad \bm{\epsilon}_t \sim \mathcal{N}(\bm{0}, \bm{\Sigma}_{f}).
}
for t = 1, \dots, T, as is used in, among others,
\insertRef{franjic2024nowcasting}{TwoStepSDFM}.
}
\details{
The \code{delay} vector indicates the number of observations at the end of the
sample that will be set to \code{NA} for each variable. Here, \code{delay} refers to
the number of months for monthly data and the number of quarters for
quarterly data. For example, consider \code{delay <- c(1, 1)} and assume the
variable with index \code{1} will be quarterfied. In that case, the variable with
index \code{1} will be delayed by 1 quarter, i.e., it will be missing 3
observations at the end of the panel. The variable with index \code{2} will be
delayed by 1 month, i.e., it will be missing 1 observation at the end of the
panel. This convention differs from the \code{delay} object of the \code{SimulData}
class this function returns. There, \code{delay} represents the number of months
since the most recent publication. For monthly variables, these values
coincide, but for quarterly variables they are inherently different.

If \code{quarterfy = TRUE}, \code{floor(quarterly_variable_ratio * no_of_vars)}
variables will be aggregated to a quarterly representation using the
geometric mean according to
\insertRef{Mariano2003new_coincident}{TwoStepSDFM}.

If \code{corr = TRUE}, the matrix \code{meas_error_var_cov} is internally replaced by a
random variance-covariance matrix:
\eqn{\tilde{\bm{\Sigma}}:=\bm{S}\bm{R}\bm{S}}{new_meas_var_cov = diag_variance_sqrt * corr_matrix * diag_variance_sqrt},
where \eqn{\bm{S}}{diag_variance_sqrt} is a diagonal matrix with entries
equal to \code{sqrt(diag(meas_error_var_cov))} and \eqn{\bm{R}}{corr_matrix} is a
random correlation matrix. \eqn{\bm{R}}{corr_matrix} is drawn according to
\insertRef{lewandowski2009generating}{TwoStepSDFM} (see also
\url{https://stats.stackexchange.com/questions/2746/how-to-efficiently-generate-random-positive-semidefinite-correlation-matrices}).
The parameter \code{beta_param} governs the degree of cross-correlation of the
off-diagonal elements. For more information see the literature cited above.

The random draws of the fundamental error terms are drawn within the \verb{C++}
backend. Therefore, \code{seed} must be provided and \code{set.seed()} will not
guarantee reproduceability.
}
\examples{
seed <- 02102025
set.seed(seed)
no_of_obs <- 100
no_of_vars <- 50
no_of_factors <- 3
trans_error_var_cov <- diag(1, no_of_factors)
loading_matrix <- matrix(round(rnorm(no_of_vars * no_of_factors)), no_of_vars, no_of_factors)
meas_error_mean <- rep(0, no_of_vars)
meas_error_var_cov <- diag(1, no_of_vars)
trans_var_coeff <- cbind(diag(0.5, no_of_factors), -diag(0.25, no_of_factors))
factor_lag_order <- 2
delay <- c(floor(rexp(no_of_vars, 1)))
quarterfy <- FALSE
quarterly_variable_ratio  <- 0
corr <- TRUE
beta_param <- 2
burn_in <- 999
starting_date <- "1970-01-01"
rescale <- TRUE
check_stationarity <- TRUE
stationarity_check_threshold <- 1e-10
factor_model <- simFM(no_of_obs = no_of_obs, no_of_vars = no_of_vars,
                      no_of_factors = no_of_factors, loading_matrix = loading_matrix,
                      meas_error_mean = meas_error_mean, 
                      meas_error_var_cov = meas_error_var_cov,
                      trans_error_var_cov = trans_error_var_cov, 
                      trans_var_coeff = trans_var_coeff,
                      factor_lag_order = factor_lag_order, delay = delay, 
                      quarterfy = quarterfy, 
                      quarterly_variable_ratio  = quarterly_variable_ratio, corr = corr,
                      beta_param = beta_param, seed = seed, burn_in = burn_in, 
                      starting_date = starting_date, rescale = rescale, 
                      check_stationarity = check_stationarity,
                      stationarity_check_threshold = stationarity_check_threshold)
print(factor_model)
spca_plots <- plot(factor_model)
spca_plots$`Factor Time Series Plots`
spca_plots$`Loading Matrix Heatmap`
spca_plots$`Meas. Error Var.-Cov. Matrix Heatmap`
spca_plots$`Meas. Error Var.-Cov. Eigenvalue Plot`
spca_plots$`Data Var.-Cov. Matrix Heatmap`
spca_plots$`Data Var.-Cov. Eigenvalue Plot`

}
\references{
\insertRef{Mariano2003new_coincident}{TwoStepSDFM}

\insertRef{lewandowski2009generating}{TwoStepSDFM}

\insertRef{franjic2024nowcasting}{TwoStepSDFM}
}
\author{
Domenic Franjic
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PlotterHelper.R
\name{plotFactorEstimates}
\alias{plotFactorEstimates}
\title{Helper for plotting factor time series}
\usage{
plotFactorEstimates(
  factors,
  smoothed_state_variance,
  no_of_factors,
  axis_text_size
)
}
\description{
Helper for plotting factor time series
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/KalmanFilterSmoother.R
\name{kalmanFilterSmoother}
\alias{kalmanFilterSmoother}
\title{Univariate Representation of the Multivariate Kalman Filter and
Smoother}
\usage{
kalmanFilterSmoother(
  data,
  delay,
  no_of_factors,
  loading_matrix,
  meas_error_var_cov,
  trans_error_var_cov,
  trans_var_coeff,
  factor_lag_order,
  fcast_horizon = 0,
  decorr_errors = TRUE,
  comp_null = 1e-15,
  parallel = FALSE,
  jitter = 1e-08
)
}
\arguments{
\item{data}{Numeric (no_of_vars \eqn{\times}{x} no_of_obs) matrix of data or
zoo/xts object sampled at the same frequency.}

\item{delay}{Integer vector of variable delays.}

\item{no_of_factors}{Integer number of factors.}

\item{loading_matrix}{Numeric (no_of_vars \eqn{\times}{x} no_of_factors)
loading matrix.}

\item{meas_error_var_cov}{Numeric (no_of_factors \eqn{\times}{x}
no_of_factors) variance-covariance matrix of the measurement errors.}

\item{trans_error_var_cov}{Numeric (no_of_vars \eqn{\times}{x} no_of_vars)
variance-covariance matrix of the transition errors.}

\item{trans_var_coeff}{Either a list of length max_factor_lag_order with each
entry a numeric (no_of_factors \eqn{\times}{x} no_of_factors) VAR
coefficient matrix or a matrix of dimensions (no_of_factors x(no_of_factors
\itemize{
\item max_factor_lag_order)) holding the VAR coefficients of the factor VAR
process in each (no_of_factors \eqn{\times}{x} no_of_factors) block.
}}

\item{factor_lag_order}{Integer order of the VAR process in the state
equation.}

\item{fcast_horizon}{Integer number of additional Filter predictions into the
future.}

\item{decorr_errors}{Logical, whether or not the errors should be
decorrelated (should be \code{TRUE} if \code{meas_error_var_cov} is not diagonal).}

\item{comp_null}{Computational zero.}

\item{parallel}{Logical, whether or not to use Eigen's internal parallel
matrix operations.}

\item{jitter}{Numerical jitter for stability of internal solver algorithms.
The jitter is added to the diagonal entries of the variance-covariance matrix
of the measurement errors.}
}
\value{
An object of class \code{KFSFit} with components:
\describe{
\item{data}{Original data matrix.}
\item{smoothed_factors}{Object containing the smoothed factor estimates.
The object inherits its class from \code{data}: If \code{data} is provided as \code{zoo},
\code{smoothed_factors} will be a \code{zoo} object. If \code{data} is provided as
\code{matrix}, \code{smoothed_factors} will be a (\code{no_of_factors} \eqn{\times}{x}
\code{no_of_obs}) matrix.}
\item{smoothed_state_variance}{(\code{no_of_factors} \eqn{\times}{x}
(\code{no_of_factors * no_of_obs})) matrix, where each (\code{no_of_factors}
\eqn{\times}{x} \code{no_of_factors}) block represents the smoother uncertainty
at time point \eqn{t}{t}}
\item{factor_var_lag_order}{Integer order of the VAR process in the state
equation.}
\item{llt_success_code}{Integer indicating the status of the Cholesky
factorization: \code{0} = LLT succeeded, \code{-1} = LLT failed but LDLT succeeded,
\code{-2} = both failed and errors are treated as uncorrelated.}
}
}
\description{
Filter and smooth the latent states/factors of a linear Gaussian state-space
model, with measurement equation
\deqn{
  \bm{x}_t = \bm{\Lambda} \bm{f}_{t} + \bm{\xi}_t,\quad \bm{\xi}_t \sim \mathcal{N}(\bm{0}, \bm{\Sigma}_{\xi}),
}
and transition equation
\deqn{
  \bm{f}_t = \sum_{p=0}^P\bm{\Phi}_p \bm{f}_{t-p} + \bm{\epsilon}_t,\quad \bm{\epsilon}_t \sim \mathcal{N}(\bm{0}, \bm{\Sigma}_{f}).
}
for t = 1, \dots, T. For filtering and smoothing, the univariate
representation of the multivariate Kalman Filter and Smoother is implemented
according to \insertRef{koopman2000fast}{TwoStepSDFM}.
}
\details{
To implement the univariate representation of the Kalman Filter and Smoother,
the measurement error term has to be cross-sectionally uncorrelated. If
\code{meas_error_var_cov} is not diagonal, one should set \code{decorr_errors = TRUE} so
that the data can be decorrelated internally prior to filtering and smoothing.

When decorrelating, the function first adds \code{jitter} to the diagonal elements
of \code{meas_error_var_cov} and then tries to compute the Cholesky factor via
Eigen's standard LLT decomposition \insertCite{eigenweb}{TwoStepSDFM}. If the
initial decorrelation fails, it silently switches to Eigen's more robust,
but slower, LDLT decomposition with pivoting
\insertCite{eigenweb}{TwoStepSDFM}. If this also fails, it is likely that
\code{meas_error_var_cov} is not well-behaved. The analysis should be repeated
with a larger \code{jitter} or a more robust variance-covariance matrix
(estimator). The success of the internal Cholesky decomposition is reported
by \code{llt_success_code}.
}
\examples{
data(factor_model)
no_of_factors <- dim(factor_model$factors)[2]
factor_lag_order <- dim(factor_model$trans_var_coeff)[2] / no_of_factors
filter_fit <- kalmanFilterSmoother(data = factor_model$data, delay = factor_model$delay, 
                                   no_of_factors = no_of_factors, 
                                   loading_matrix = factor_model$loading_matrix, 
                                   meas_error_var_cov = factor_model$meas_error_var_cov, 
                                   trans_error_var_cov = factor_model$trans_error_var_cov,
                                   trans_var_coeff = factor_model$trans_var_coeff, 
                                   factor_lag_order = factor_lag_order, 
                                   fcast_horizon = 5, decorr_errors = TRUE,  
                                   comp_null = 1e-15, parallel = FALSE,  jitter = 1e-8)
print(filter_fit)
filter_plots <- plot(filter_fit)
filter_plots$`Factor Time Series Plots`

}
\references{
\insertRef{koopman2000fast}{TwoStepSDFM}

\insertRef{eigenweb}{TwoStepSDFM}
}
\author{
Domenic Franjic
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Helper.R
\name{checkParameterMatrix}
\alias{checkParameterMatrix}
\title{Helper function to check parameter matrices}
\usage{
checkParameterMatrix(parameter, name, no_of_rows, no_of_cols)
}
\description{
Helper function to check parameter matrices
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CrossVal.R
\name{print.SDFMcrossVal}
\alias{print.SDFMcrossVal}
\title{Generic print function for SDFMcrossVal S3 objects}
\usage{
\method{print}{SDFMcrossVal}(x, ...)
}
\arguments{
\item{x}{\code{SDFMcrossVal} object.}

\item{...}{Additional parameters for the plotting functions.}
}
\value{
No return value; Prints a summary to the console.
}
\description{
Generic print function for SDFMcrossVal S3 objects
}
\author{
Domenic Franjic
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PlotterHelper.R
\name{plotInformationCrit}
\alias{plotInformationCrit}
\title{Helper for plotting ICs for selecting the number of factors}
\usage{
plotInformationCrit(information_crit, no_of_factors, axis_text_size)
}
\description{
Helper for plotting ICs for selecting the number of factors
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CrossVal.R
\name{crossVal}
\alias{crossVal}
\title{Cross-validate SDFM Hyper-Parameters}
\usage{
crossVal(
  data,
  variable_of_interest,
  fcast_horizon,
  delay,
  frequency,
  no_of_factors,
  seed,
  min_ridge_penalty,
  max_ridge_penalty,
  cv_repetitions,
  cv_size,
  lasso_penalty_type,
  min_max_penalty,
  max_factor_lag_order = 10,
  lag_estim_criterion = "BIC",
  decorr_errors = TRUE,
  max_iterations = 1000,
  weights = NULL,
  comp_null = 1e-15,
  spca_conv_crit = 1e-04,
  parallel = FALSE,
  no_of_cores = 1,
  max_ar_lag_order = 5,
  max_predictor_lag_order = 5,
  jitter = 1e-08,
  svd_method = "precise",
  verbose = TRUE
)
}
\arguments{
\item{data}{Numeric (no_of_vars \eqn{\times}{x} no_of_obs) matrix of data or
zoo/xts object sampled at mixed frequencies (quarterly and monthly).}

\item{variable_of_interest}{Integer indicating the index of the target
variables.}

\item{fcast_horizon}{Integer value indicating the target forecasting horizon.}

\item{delay}{Integer vector of variable delays, measured as the number of
months since the latest available observation.}

\item{frequency}{Integer vector of frequencies of the variables in the data
set (currently supported: \code{12} for monthly and \code{4} for quarterly data).}

\item{no_of_factors}{Integer number of factors.}

\item{seed}{32-bit unsigned integer seed for all random processes inside the
function.}

\item{min_ridge_penalty}{Numeric lower bound for the sampled ridge penalty
coefficient candidates.}

\item{max_ridge_penalty}{Numeric upper bound for the sampled ridge penalty
coefficient candidates.}

\item{cv_repetitions}{Integer number of \code{fcast_horizon}-step-ahead
predictions computed for each candidate set.}

\item{cv_size}{Integer number of candidate sets.}

\item{lasso_penalty_type}{Character indicating the lasso penalty type.
If set to \code{"selected"}, the \eqn{\ell_1}{`l_1`}-size constraint will be
returned as number of non-zero elements of each column of the loading matrix.
If set to \code{"penalty"}, the lasso size constraint will be returned. If set
to \code{"steps"}, the number of LARS-EN steps will be returned.}

\item{min_max_penalty}{Vector of size two, where the first element indicates
the lower and the second element indicates the upper bound of the lasso
penalty equivalent. If \code{lasso_penalty_type} is set to \code{"selected"} or
\code{"steps"}, both elements must be strictly positive integers.}

\item{max_factor_lag_order}{Integer maximum order of the VAR process in the
transition equation.}

\item{lag_estim_criterion}{Information criterion used for the estimation of
the factor VAR order (\code{"BIC"} (default), \code{"AIC"}, \code{"HIC"}).}

\item{decorr_errors}{Logical, whether or not the errors should be
decorrelated.}

\item{max_iterations}{Integer maximum number of iterations of the SPCA
algorithm.}

\item{weights}{Numeric vector, weights for each variable weighing the
\eqn{\ell_1}{`l_1`} size constraint.}

\item{comp_null}{Numeric computational zero.}

\item{spca_conv_crit}{Numeric conversion criterion for the SPCA algorithm.}

\item{parallel}{Logical, whether or not to run the cross-validation loop in
parallel.}

\item{no_of_cores}{Integer number of cores to use when run in parallel.}

\item{max_ar_lag_order}{Integer maximum number of lags of the target variable
included in the final ARDL prediction routine.}

\item{max_predictor_lag_order}{Integer maximum number of lags of the
predictors included in the final ARDL prediction routine.}

\item{jitter}{Numerical jitter for stability of internal solver algorithms.
The jitter is added to the diagonal entries of the variance covariance matrix
of the measurement errors.}

\item{svd_method}{Either \code{"fast"} or \code{"precise"}. Option \code{"fast"} uses
Eigen's BDCSVD divide and conquer method for the computation of the singular
values. Option \code{"precise"} (default) implements the slower, but numerically
more stable JacobiSVD method.}

\item{verbose}{Logical, whether to print some progress tracking output to the
console.}
}
\value{
An object of class \code{SDFMcrossVal} with main components:
\describe{
\item{\code{CV}}{A list with components \code{`CV Results`} (matrix of all
cross-validation errors and corresponding hyper-parameter values) and
\code{`Min. CV`} (row of \verb{CV Results} with the minimum cross-validation
error).}
\item{\code{BIC}}{A list with components \verb{BIC Results} (matrix of all BIC values
and corresponding hyper-parameter values) and \verb{Min. BIC} (row of
\verb{BIC Results} with the minimum BIC).}
}
}
\description{
This function uses time series cross-validation
\insertCite{rob2018forecasting}{TwoStepSDFM} in combination with random
hyper-parameter search \insertCite{bergstra2012random}{TwoStepSDFM} to
validate the hyper-parameters of a sparse dynamic factor model as described
in \insertRef{franjic2024nowcasting}{TwoStepSDFM}
}
\details{
\code{fcast_horizon} should be set to the target prediction horizon, as
hyper-parameters can differ substantially between different horizons. For
nowcasting, use \code{fcast_horizon = 0}. For backcasting, \code{fcast_horizon} can be
set to a negative number indicating the step-back backcasting horizon.

Internally, candidates of the hyper-parameters are drawn randomly. However,
a regular dense DFM will always be considered by default. The ridge
penalty is drawn as \eqn{\exp(u)}{exp(u)}, where \eqn{u}{u} is uniformly
distributed between \code{min_ridge_penalty} and \code{max_ridge_penalty}. If
\code{lasso_penalty_type = "selected"}, the lasso penalty is drawn as a random
vector \eqn{\bm{v}}{v}, where each entry is uniformly distributed. If
\code{lasso_penalty_type = "steps"}, the lasso penalty is drawn as a random
value \eqn{v}{v} that is uniformly distributed. If
\code{lasso_penalty_type = "penalty"}, the lasso penalty is drawn as a random
vector \eqn{\exp(\bm{v})}{exp(v)}, where each entry of
\eqn{\bm{v}}{v} is uniformly distributed. In all three cases, the upper and
lower bounds of the uniform distributions governing the lasso penalties are
given by the first and second entry of \code{min_max_penalty}, respectively.

For medium to large data sets in combination with a medium to large
\code{cv_size}, it can be beneficial to set \code{parallel = TRUE}. This will enable
parallelisation via the doParallel, doSNOW, foreach, and parallel packages
in R. In this case, \code{no_of_cores} should be set to the number of physical
cores of the user's machine. It is not advisable to use the number of logical
cores, as this can considerably deteriorate performance.

This function serves as a direct wrapper to \code{\link{nowcast}}. For more
information on the additional function parameters, see the corresponding help
page.
}
\examples{
data(mixed_freq_factor_model)
no_of_vars <- dim(mixed_freq_factor_model$data)[2]
no_of_factors <- dim(mixed_freq_factor_model$factors)[2]
cv_results <- crossVal(data = mixed_freq_factor_model$data, variable_of_interest = 1, 
                       fcast_horizon = 0, delay = mixed_freq_factor_model$delay, 
                       frequency = mixed_freq_factor_model$frequency,
                       no_of_factors = no_of_factors, seed = 25032026,
                       min_ridge_penalty = 1e-5, max_ridge_penalty = 10, 
                       cv_repetitions = 1, cv_size = 50, lasso_penalty_type = "selected",
                       min_max_penalty = c(5, 45), verbose = FALSE)
print(cv_results)
cv_plots <- plot(cv_results)        
cv_plots$`CV Results`
cv_plots$`BIC Results` 

}
\references{
\insertRef{bergstra2012random}{TwoStepSDFM}

\insertRef{rob2018forecasting}{TwoStepSDFM}

\insertRef{franjic2024nowcasting}{TwoStepSDFM}
}
\seealso{
\code{\link{sparsePCA}}: Routine for fitting estimating a sparse factor
loading matrix.

\code{\link{kalmanFilterSmoother}}: Routine for filtering and smoothing
latent factors.

\code{\link{twoStepSDFM}}: Two-step estimation routine for a sparse dynamic
factor model.

\code{\link{twoStepDenseDFM}}: Two-step estimation routine for a dense
dynamic factor model.
}
\author{
Domenic Franjic
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Helper.R
\name{makeRaggedEdges}
\alias{makeRaggedEdges}
\title{Helper function to check positive double function parameter}
\usage{
makeRaggedEdges(data, delay)
}
\description{
Helper function to check positive double function parameter
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PlotterHelper.R
\name{plotLoadingHeatMap}
\alias{plotLoadingHeatMap}
\title{Helper for plotting loading matrix heat maps}
\usage{
plotLoadingHeatMap(
  loading_matrix_estim,
  series_names,
  no_of_factors,
  axis_text_size,
  legend_title_text_size
)
}
\description{
Helper for plotting loading matrix heat maps
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TwoStepSDFM.R
\name{twoStepSDFM}
\alias{twoStepSDFM}
\title{Two Step Sparse Dynamic Factor Model Estimator.}
\usage{
twoStepSDFM(
  data,
  delay,
  selected,
  no_of_factors,
  max_factor_lag_order = 10,
  lag_estim_criterion = "BIC",
  decorr_errors = TRUE,
  ridge_penalty = 1e-06,
  lasso_penalty = NULL,
  max_iterations = 1000,
  max_no_steps = NULL,
  weights = NULL,
  comp_null = 1e-15,
  spca_conv_crit = 1e-04,
  parallel = FALSE,
  fcast_horizon = 0,
  jitter = 1e-08,
  svd_method = "precise"
)
}
\arguments{
\item{data}{Numeric (no_of_vars \eqn{\times}{x} no_of_obs) matrix of data or
zoo/xts object sampled at the same frequency.}

\item{delay}{Integer vector of variable delays, measured as the number of
months since the latest available observation.}

\item{selected}{Integer vector of the number of selected variables for each
factor.}

\item{no_of_factors}{Integer number of factors.}

\item{max_factor_lag_order}{Integer maximum order of the VAR process in the
transition equation.}

\item{lag_estim_criterion}{Information criterion used for the estimation of
the factor VAR order (\code{"BIC"} (default), \code{"AIC"}, \code{"HIC"}).}

\item{decorr_errors}{Logical, whether or not the errors should be
decorrelated.}

\item{ridge_penalty}{Ridge penalty.}

\item{lasso_penalty}{Numeric vector, lasso penalties for each factor (set to
NULL to disable as stopping criterion).}

\item{max_iterations}{Integer maximum number of iterations.}

\item{max_no_steps}{Integer number of LARS steps (set to NULL to disable as
stopping criterion).}

\item{weights}{Numeric vector, weights for each variable weighing the
\eqn{\ell_1}{`l_1`} size constraint.}

\item{comp_null}{Numeric computational zero.}

\item{spca_conv_crit}{Conversion threshold for the SPCA algorithm.}

\item{parallel}{Logical, whether or not to use Eigen's internal parallel
matrix operations.}

\item{fcast_horizon}{Integer number of additional Filter predictions into the
future.}

\item{jitter}{Numerical jitter for stability of internal solver algorithms.
The jitter is added to the diagonal entries of the variance covariance matrix
of the measurement errors.}

\item{svd_method}{Either "fast" or "precise". Option "fast" uses Eigen's
BDCSVD divide and conquer method for the computation of the singular values.
Option "precise" (default) implements the slower, but numerically more stable
JacobiSVD method \insertCite{eigenweb}{TwoStepSDFM}.}
}
\value{
An object of class SDFMFit with main components:
#' \describe{
\item{data}{Original data object.}
\item{loading_matrix_estim}{Numeric matrix of estimated factor loadings.}
\item{smoothed_factors}{Object containing the SPCA factor estimates. The
object inherits its class from data: If data is provided as \code{zoo},
\code{factor_estim} will be a \code{zoo} object. If data is provided as matrix,
\code{factor_estim} will be a (\code{no_of_factors}\eqn{\times}{x}\code{no_of_obs}
matrix.}
\item{smoothed_state_variance}{(\code{no_of_factors}\eqn{\times}{x}(
\code{no_of_factors} * \code{no_of_obs})) matrix, where each (\code{no_of_factors}
\eqn{\times}{x}\code{no_of_factors}) block represents the smoother uncertainty
at time point\eqn{t}{t}.}
\item{factor_var_lag_order}{Integer order of the VAR process in the state
equation.}
\item{error_var_cov_cholesky_factor}{Numeric lower-triangular Cholesky
factor of the estimated measurement error variance–covariance matrix.}
\item{llt_success_code}{Integer indicating the status of the Cholesky
factorization: \code{0} = LLT succeeded, \code{-1} = LLT failed but LDLT succeeded,
\code{-2} = both failed and errors are treated as uncorrelated.}
}
}
\description{
Estimate a sparse dynamic factor model with measurement equation
\deqn{
  \bm{x}_t = \bm{\Lambda} \bm{f}_{t} + \bm{\xi}_t,\quad \bm{\xi}_t \sim \mathcal{N}(\bm{0}, \bm{\Sigma}_{\xi}),
}
and transition equation
\deqn{
  \bm{f}_t = \sum_{p=0}^P\bm{\Phi}_p \bm{f}_{t-p} + \bm{\epsilon}_t,\quad \bm{\epsilon}_t \sim \mathcal{N}(\bm{0}, \bm{\Sigma}_{f}).
}
using sparse principal components  analysis and the Kalman Filter and
Smoother according to \insertRef{franjic2024nowcasting}{TwoStepSDFM}.
}
\details{
The function performs a two-step estimation procedure for sparse dynamic
factor models as described in \insertRef{franjic2024nowcasting}{TwoStepSDFM}.
In the first step, the factor loading matrix is estimated using SPCA
\insertCite{zou2006sparse}{TwoStepSDFM}. This will shrink some of the
loadings towards or exactly to zero. In the second step the latent factors
are estimated using the univariate representation of the Kalman Filter and
Smoother \insertCite{koopman2000fast}{TwoStepSDFM}.

The function takes three stopping criteria for the SPCA algorithm:
\code{selected}, \code{lasso_penalty}, and \code{max_no_steps}. The argument \code{weights}
allows specifying weights for the \eqn{\ell_1}{l1} constraint.  \code{svd_method}
controls the decomposition method for internal SVDs. For a detailed
description of these arguments and the SPCA step, see
\code{\link{sparsePCA}}.

With respect to the univariate representation of the Kalman filter and
smoother, \code{decorr_errors} indicates whether the data should be decorrelated
internally prior to filtering and smoothing. \code{jitter} is added to the
diagonal elements of the measurement variance–covariance matrix. For more
details, see \code{\link{kalmanFilterSmoother}}.

For more information on the two-step estimation procedure see
\insertRef{franjic2024nowcasting}{TwoStepSDFM}.
}
\examples{
data(factor_model)
no_of_vars <- dim(factor_model$data)[2]
no_of_factors <- dim(factor_model$factors)[2]
sdfm_fit <- twoStepSDFM(data = factor_model$data, delay = factor_model$delay,
                        selected = rep(floor(0.5 * no_of_vars), no_of_factors),
                        no_of_factors = no_of_factors)
print(sdfm_fit)
sdfm_plots <- plot(sdfm_fit)
sdfm_plots$`Factor Time Series Plots`
sdfm_plots$`Loading Matrix Heatmap`
sdfm_plots$`Meas. Error Var.-Cov. Matrix Heatmap`
sdfm_plots$`Meas. Error Var.-Cov. Eigenvalue Plot`

}
\references{
\insertRef{koopman2000fast}{TwoStepSDFM}

\insertRef{zou2006sparse}{TwoStepSDFM}

\insertRef{eigenweb}{TwoStepSDFM}

\insertRef{franjic2024nowcasting}{TwoStepSDFM}
}
\seealso{
\code{\link{sparsePCA}}: Routine for fitting estimating a sparse factor
loading matrix.

\code{\link{kalmanFilterSmoother}}: Routine for filtering and smoothing
latent factors.

\code{\link{twoStepDenseDFM}}: Two-step estimation routine for a dense
dynamic factor model.
}
\author{
Domenic Franjic
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SimFM.r
\name{plot.SimulData}
\alias{plot.SimulData}
\title{Generic plotting function for SimulData S3 objects}
\usage{
\method{plot}{SimulData}(x, axis_text_size = 20, legend_title_text_size = 20, ...)
}
\arguments{
\item{x}{\code{SimulData} object.}

\item{axis_text_size}{Numeric size of x- and y-axis labels. Prased to ggplot2
\code{theme(..., text = element_text(size = axis_text_size))}.}

\item{legend_title_text_size}{Numeric size of x- and y-axis labels. Prased to
ggplot2
\code{theme(..., legend.title = element_text(size = legend_title_text_size))}.}

\item{...}{Additional parameters for the plotting functions.}
}
\value{
A named list of plot objects:
\describe{
\item{\verb{Factor Time Series Plots}}{\code{patchwork}/\code{ggplot} object showing the
simulated factors over time.}
\item{\verb{Loading Matrix Heatmap}}{\code{ggplot} object showing a heatmap of the
simulated factor loadings. Zeros are highlighted in black.}
\item{\verb{Meas. Error Var.-Cov. Matrix Heatmap}}{\code{ggplot} object showing a
heatmap of the measurement error  variance-covariance matrix.}
\item{\verb{Meas. Error Var.-Cov. Eigenvalue Plot}}{\code{ggplot}object showing a bar
plot of the eigenvalues of the measurement error variance-covariance
matrix.}
\item{\verb{Data Var.-Cov. Matrix Heatmap}}{\code{ggplot} object showing a heatmap of
the data variance-covariance matrix.}
\item{\verb{Data Var.-Cov. Eigenvalue Plot}}{\code{ggplot} object showing a bar plot
of the eigenvalues of the data variance-covariance matrix.}
}
}
\description{
Create diagnostic plots for an \code{SimulData} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PlotterHelper.R
\name{plotMeasVarCovEigenvalues}
\alias{plotMeasVarCovEigenvalues}
\title{Helper for plotting measurement error var.cov. eigenvalues}
\usage{
plotMeasVarCovEigenvalues(
  eigen_values,
  no_of_factors,
  axis_text_size,
  legend_title_text_size
)
}
\description{
Helper for plotting measurement error var.cov. eigenvalues
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Nowcast.R
\name{plot.SDFMnowcast}
\alias{plot.SDFMnowcast}
\title{Generic plotting function for SDFMnowcast S3 objects}
\usage{
\method{plot}{SDFMnowcast}(x, axis_text_size = 20, ...)
}
\arguments{
\item{x}{\code{SDFMnowcast} object.}

\item{axis_text_size}{Numeric size of x- and y-axis labels. Prased to ggplot2
\code{theme(..., text = element_text(size = axis_text_size))}.}

\item{...}{Additional parameters for the plotting functions.}
}
\value{
A named list storing of \code{ggplot} objects:
\describe{
\item{\verb{Single Pred. Fcast Density Plots x}}{\code{patchwork} / \code{ggplot} objects
graphing the distribution of forecasts generated by the predictors for
each prediction (backcasts, nowcasts, forecasts) for each target,
respectively. Altogether, there will be as many such objects as there are
targets, with \code{x} replaced by the column name of the target.}
}
}
\description{
Generic plotting function for SDFMnowcast S3 objects
}
\author{
Domenic Franjic
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Nowcast.R
\name{nowcast}
\alias{nowcast}
\title{Predict Mixed-Frequency Data via Dynamic Factor Models}
\usage{
nowcast(
  data,
  variables_of_interest,
  max_fcast_horizon,
  delay,
  selected,
  frequency,
  no_of_factors,
  sparse = TRUE,
  max_factor_lag_order = 10,
  lag_estim_criterion = "BIC",
  decorr_errors = TRUE,
  ridge_penalty = 1e-06,
  lasso_penalty = NULL,
  max_iterations = 1000,
  max_no_steps = NULL,
  weights = NULL,
  comp_null = 1e-15,
  spca_conv_crit = 1e-04,
  parallel = FALSE,
  max_ar_lag_order = 5,
  max_predictor_lag_order = 5,
  jitter = 1e-08,
  svd_method = "precise"
)
}
\arguments{
\item{data}{Numeric (no_of_vars \eqn{\times}{x} no_of_obs) matrix of data or
zoo/xts object sampled at mixed frequencies (quarterly and monthly).}

\item{variables_of_interest}{Integer vector indicating the index of all
target variables.}

\item{max_fcast_horizon}{Maximum forecasting horizon of all targets.}

\item{delay}{Integer vector of variable delays, measured as the number of
months since the latest available observation.}

\item{selected}{Integer vector of the number of selected variables for each
factor.}

\item{frequency}{Integer vector of frequencies of the variables in the data
set (currently supported: \code{12} for monthly and \code{4} for quarterly data).}

\item{no_of_factors}{Integer number of factors.}

\item{sparse}{Logical, if \code{TRUE} (default) a sparse DFM is used to estimate
the model parameters and latent factors (see \code{\link{twoStepSDFM}}).
Else, a dense DFM is used (see \code{\link{twoStepDenseDFM}}).}

\item{max_factor_lag_order}{Integer maximum order of the VAR process in the
transition equation.}

\item{lag_estim_criterion}{Information criterion used for the estimation of
the factor VAR order (\code{"BIC"} (default), \code{"AIC"}, \code{"HIC"}).}

\item{decorr_errors}{Logical, whether or not the errors should be
decorrelated.}

\item{ridge_penalty}{Numeric ridge penalty.}

\item{lasso_penalty}{Numeric vector, lasso penalties for each factor (set to
NULL to disable as stopping criterion).}

\item{max_iterations}{Integer maximum number of iterations of the SPCA
algorithm.}

\item{max_no_steps}{Integer number of LARS steps (set to NULL to disable as
stopping criterion).}

\item{weights}{Numeric vector, weights for each variable weighing the
\eqn{\ell_1}{`l_1`} size constraint.}

\item{comp_null}{Numeric computational zero.}

\item{spca_conv_crit}{Numeric conversion criterion for the SPCA algorithm.}

\item{parallel}{Logical, whether or not to use Eigen's internal parallel
matrix operations.}

\item{max_ar_lag_order}{Integer maximum number of lags of the target variable
included in the final ARDL prediction routine.}

\item{max_predictor_lag_order}{Integer maximum number of lags of the
predictors included in the final ARDL prediction routine.}

\item{jitter}{Numerical jitter for stability of internal solver algorithms.
The jitter is added to the diagonal entries of the variance covariance matrix
of the measurement errors.}

\item{svd_method}{Either \code{"fast"} or \code{"precise"}. Option \code{"fast"} uses
Eigen's BDCSVD divide and conquer method for the computation of the singular
values. Option \code{"precise"} (default) implements the slower, but numerically
more stable JacobiSVD method.}
}
\value{
The \code{nowcast} function returns named list containing the following objects:
\describe{
\item{Forecasts}{Numeric matrix of the target variables and their
respective backcasts, nowcasts, and/or forecasts.}
\item{SDFM Fit}{An \code{SDFMFit} object holding the estimates of the model
parameters and the latent factors (see \code{\link{twoStepSDFM}} or
\code{\link{twoStepDenseDFM}}).}
}
}
\description{
Backcast, nowcast, and forecast quarterly target variables via a sparse/dense
DFM using additional monthly data with ragged edges. Forecasts are produced
using all quarterly targets and a quarterly representation of latent monthly
factors \insertCite{Mariano2003new_coincident}{TwoStepSDFM}. Final
predictions are computed via equally weighted forecast averaging of ARDL
models \insertCite{marcellino2010factor}{TwoStepSDFM} for each of the targets
and quarterfied factors.
}
\details{
This function serves as a prediction wrapper for the
\code{\link{twoStepDenseDFM}} and \code{\link{twoStepSDFM}} functions. \code{data}
should be a mixed-frequency data set. Currently, only monthly and quarterly
data are supported. With respect to the quarterly data, the function expects
the realization of the quarterly observations to occur in the last month of
the quarter. Indicate quarterly and monthly variables via \code{frequency} by
setting the corresponding element of \code{frequency} to \code{4} for quarterly and to
\code{12} for monthly data.

This function is only able to compute predictions for quarterly variables.
To impute the ragged edges of the monthly observations, and potentially
compute additional predictions for the monthly variables, call \code{predict} on
the \code{SDFMFit} object returned by \code{\link{twoStepDenseDFM}} /
\code{\link{twoStepSDFM}} (see \code{\link{predict.SDFMFit}}).

\code{max_fcast_horizon} sets the maximum number of forecasts predicted starting
from the final observation of the data set. For each target, the number of
backcasts and whether or not a nowcast should be computed is determined
internally. This is done in such a way that every missing quarterly
observation of the targets is predicted.

\code{max_ar_lag_order} governs the maximum number of lags of the current target
used to predict said target in each ARDL model. \code{max_predictor_lag_order}
governs the maximum number of lags of each additional quarterly predictor,
including other potential targets and the aggregated factors, used to predict
any given target in each ARDL model. The actual number of lags is internally
estimated using the BIC. Setting \code{max_ar_lag_order = 0} disables the use of
target lags in its own prediction function.

\code{sparse} toggles between a sparse DFM and a dense DFM. If \code{sparse = FALSE},
all SPCA stopping criteria and other parameters passed to the sparse
estimation routine are ignored (for details on these parameters see
\code{\link{twoStepDenseDFM}}). Parameters governing the Kalman Filter and
Smoother are passed directly to \code{\link{twoStepDenseDFM}} /
\code{\link{twoStepSDFM}}. For details see the corresponding help pages.
}
\examples{
data(mixed_freq_factor_model)
no_of_vars <- dim(mixed_freq_factor_model$data)[2]
no_of_factors <- dim(mixed_freq_factor_model$factors)[2]
sparse_nowcast <- nowcast(data = mixed_freq_factor_model$data, variables_of_interest = c(1, 2),
                          max_fcast_horizon = 4, delay = mixed_freq_factor_model$delay,
                          selected = rep(floor(0.5 * no_of_vars), no_of_factors), 
                          frequency = mixed_freq_factor_model$frequency, 
                          no_of_factors = no_of_factors, sparse = TRUE)
print(sparse_nowcast)
dense_nowcast <- nowcast(data = mixed_freq_factor_model$data, variables_of_interest = c(1, 2),
                         max_fcast_horizon = 4, delay = mixed_freq_factor_model$delay,
                         selected = NULL, frequency = mixed_freq_factor_model$frequency, 
                         no_of_factors = no_of_factors, sparse = FALSE)
sparse_plots <- plot(sparse_nowcast)
sparse_plots$`Single Pred. Fcast Density Plots Series 1`

}
\references{
\insertRef{Mariano2003new_coincident}{TwoStepSDFM}

\insertRef{marcellino2010factor}{TwoStepSDFM}

\insertRef{franjic2024nowcasting}{TwoStepSDFM}
}
\seealso{
\code{\link{sparsePCA}}: Routine for fitting estimating a sparse factor
loading matrix.

\code{\link{kalmanFilterSmoother}}: Routine for filtering and smoothing
latent factors.

\code{\link{twoStepSDFM}}: Two-step estimation routine for a sparse dynamic
factor model.

\code{\link{twoStepDenseDFM}}: Two-step estimation routine for a dense
dynamic factor model.
}
\author{
Domenic Franjic
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{factor_model}
\alias{factor_model}
\title{Example factor model dataset}
\format{
A \code{SimulData} containing the following elements:
\describe{
\item{data}{If \code{starting_date} is provided, a \code{zoo} object, else,
a (\code{no_of_vars} \eqn{\times}{x} \code{no_of_obs}) numeric matrix holding the
simulated data.}
\item{factors}{If \code{starting_date} is provided, a \code{zoo} object, else a
(\code{no_of_factors} \eqn{\times}{x} \code{no_of_obs}) numeric matrix holding  the
simulated latent factors.}
\item{trans_var_coeff}{Numeric (\code{no_of_factors} \eqn{\times}{x}
(\code{no_of_factors} * \code{factor_lag_order})) factor VAR coefficient matrix.}
\item{loading_matrix}{Numeric factor loading matrix.}
\item{meas_error}{If \code{starting_date} is provided, a \code{zoo} object, else
a (\code{no_of_vars} \eqn{\times}{x} \code{no_of_obs}) numeric matrix holding the
fundamental measurement errors.}
\item{meas_error_var_cov}{Numeric measurement error
variance-covariance matrix.}
\item{trans_error_var_cov}{Numeric transition error
variance-covariance matrix.}
\item{frequency}{Integer vector of variable frequencies.}
\item{delay}{Integer vector of variable delays, measured as the number of
months since the latest available observation.}
}
}
\source{
Generated via \code{simFM()}. For details see \code{factor_model$call}.
}
\usage{
factor_model
}
\description{
This is a simulated factor model dataset for demonstration and testing
with \code{TwoStepSDFM}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EstimateFactors.R
\name{print.NoOfFactorsFit}
\alias{print.NoOfFactorsFit}
\title{Generic printing function for NoOfFactorsFit S3 objects}
\usage{
\method{print}{NoOfFactorsFit}(x, ...)
}
\arguments{
\item{x}{\code{NoOfFactorsFit} object.}

\item{...}{Additional parameters for the plotting functions.}
}
\value{
No return value; Prints a summary to the console.
}
\description{
Print a compact summary of an \code{NoOfFactorsFit} object.
}
\author{
Domenic Franjic
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EstimateFactors.R
\name{noOfFactors}
\alias{noOfFactors}
\title{Estimate the number of Factors}
\usage{
noOfFactors(
  data,
  min_no_factors = 1,
  max_no_factors = 7,
  confidence_threshold = 0.05
)
}
\arguments{
\item{data}{Numeric (no_of_vars \eqn{\times}{x} no_of_obs) matrix of data or
zoo/xts object sampled at the same frequency.}

\item{min_no_factors}{Integer minimum number of factors to be tested.}

\item{max_no_factors}{Integer maximum number of factors to be tested (should
be at most min_no_factors + 17).}

\item{confidence_threshold}{Numeric threshold value to stop the testing procedure.}
}
\value{
An object of class \code{NoOfFactorsFit} with components:
\describe{
\item{no_of_factors}{Integer estimated number of factors.}
\item{p_value}{Numeric \eqn{p}{p}-value of the final test.}
\item{confidence_threshold}{Numeric significance level used.}
\item{statistic}{Numeric test statistic value of the last test.}
\item{eigen_values}{Numeric vector of eigenvectors of the complex data Gram
matrix.}
}
}
\description{
Estimate the number of factors of a linear Gaussian latent factor model using
the eigenvalue slope test of \insertRef{onatski2009testing}{TwoStepSDFM} and
the infromation criterion based approach of
\insertRef{bai2002determining}{TwoStepSDFM}.
}
\details{
The \insertCite{onatski2009testing}{TwoStepSDFM} procedure splits the data
matrix along the time dimension into two equally sized (\code{no_of_vars}
\eqn{\times}{x} \code{cut_off}) sub-matrices \eqn{\bm{X}_{1/2}}{`data_fst_half`}
and \eqn{\bm{X}_{2/2}}{`data_snd_half`}. It then proceeds to build
\eqn{\tilde{\bm{X}} := \bm{X}_{1/2} + i\bm{X}_{2/2}}{`complex_data = data_fst_half + i * data_snd_half`},
where \eqn{i=\sqrt{-1}}{`i=sqrt(-1)`}. We then compute the eigenvalues of the
Gram matrix \eqn{\tilde{\bm{X}}
\tilde{\bm{X}}^{\dagger}}{\code{complex_data \%*\% Conj(t(complex_data))}}, where
\eqn{\tilde{\bm{X}}^{\dagger}}{`Conj(t(complex_data))`} represents the
adjoint. Finally, a test based on the computed eigenvalues is performed.
This test is an iterative testing procedure, starting by testing the null
that the true number of factors is \code{min_no_factors}. If the test is rejected
by comparison of the \eqn{p}{p}-value against \code{confidence_threshold}, we
test whether the true number of factors is \code{min_no_factors + 1} until we can
no longer reject at \code{confidence_threshold} or \code{max_no_factors} is reached.

As the distribution of the eigenvalues under the null is nonstandard
\insertCite{onatski2009testing}{TwoStepSDFM},
simulated critical values are used. They are retrieved from
\insertRef{onatski2009testing_supl}{TwoStepSDFM}. As the range of the
simulated critical values is limited, the minimum and maximum number of
potential factors is limited such that \code{max_no_factors} should be no more
than \code{min_no_factors + 17}. However, it is recommended to operate well below
this maximum as the test size decreases with
\code{max_no_factors - min_no_factors}.

The \insertCite{bai2002determining}{TwoStepSDFM} information criterion
determines the number of factors by minimising a BIC. Here, three different
penalty terms are provided. It is up to the user to determine the most
appropriate for the problem at hand. In general, however, the second
information criterion is used.
}
\examples{
data(factor_model)
no_of_factors_estim <- noOfFactors(data = factor_model$data, min_no_factors = 1, 
                                   max_no_factors = 5, confidence_threshold = 0.05)
print(no_of_factors_estim)
factor_estim_plots <- plot(no_of_factors_estim)
factor_estim_plots$`Eigen Value Plot Test Procedure`
factor_estim_plots$`IC plot for IC1`
factor_estim_plots$`IC plot for IC2`
factor_estim_plots$`IC plot for IC3`

}
\references{
\insertRef{onatski2009testing}{TwoStepSDFM}

\insertRef{onatski2009testing_supl}{TwoStepSDFM}
}
\author{
Domenic Franjic
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SparcePCA.R
\name{sparsePCA}
\alias{sparsePCA}
\title{Sparse Principal Components Analysis}
\usage{
sparsePCA(
  data,
  delay,
  selected,
  no_of_factors,
  ridge_penalty = 1e-06,
  lasso_penalty = NULL,
  max_iterations = 1000,
  weights = NULL,
  max_no_steps = NULL,
  comp_null = 1e-15,
  spca_conv_crit = 1e-04,
  parallel = FALSE,
  svd_method = "precise",
  normalise = TRUE,
  comp_var_expl = TRUE
)
}
\arguments{
\item{data}{Numeric (no_of_vars \eqn{\times}{x} no_of_obs) matrix of data or
zoo/xts object sampled at the same frequency.}

\item{delay}{Integer vector of variable delays, measured as the number of
months since the latest available observation.}

\item{selected}{Integer vector of the number of selected variables for each
factor.}

\item{no_of_factors}{Integer number of factors.}

\item{ridge_penalty}{Numeric ridge penalty.}

\item{lasso_penalty}{Numeric vector, lasso penalties for each factor (set to
NULL to disable as stopping criterion).}

\item{max_iterations}{Integer maximum number of iterations.}

\item{weights}{Numeric vector, weights for each variable weighing the
\eqn{\ell_1}{`l_1`} size constraint.}

\item{max_no_steps}{Integer number of LARS steps (set to NULL to disable as
stopping criterion).}

\item{comp_null}{Numeric computational zero.}

\item{spca_conv_crit}{Conversion threshold for the SPCA algorithm.}

\item{parallel}{Logical, whether or not to use Eigen's internal parallel
matrix operations.}

\item{svd_method}{Either "fast" or "precise". Option "fast" uses Eigen's
BDCSVD divide and conquer method for the computation of the singular values.
Option "precise" (default) implements the slower, but numerically more stable
JacobiSVD method.}

\item{normalise}{Logical, whether to normalise the loading matrix as in
\insertRef{zou2020elnet}{TwoStepSDFM}. Default is \code{TRUE}.}

\item{comp_var_expl}{Logical, whether to compute the relative variance
explained by each factor. Default is \verb{TRUE``.}}
}
\value{
An object of class \code{SPCAFit} with components:
\describe{
\item{data}{Original data matrix.}
\item{loading_matrix_estim}{Numeric matrix of estimated factor loadings.}
\item{factor_estim}{Object containing the SPCA factor estimates. The
object inherits its class from \code{data}: If \code{data} is provided as \code{zoo},
\code{factor_estim} will be a \code{zoo} object. If \code{data} is provided as  \code{matrix},
\code{factor_estim} will be a (\code{no_of_factors} \eqn{\times}{x} \code{no_of_obs})
matrix.}
\item{total_var_expl}{Numeric total variance explained.}
\item{pct_var_expl}{Numeric vector relative variance explained by each
factor.}
}
}
\description{
Estimate sparse sparse principal components via SPCA according to
\insertRef{zou2006sparse}{TwoStepSDFM}.
}
\details{
The function takes three stopping criteria: \code{selected}, \code{lasso_penalty}, and
\code{max_no_steps}. With \code{selected} the SPCA algorithm stops if each column of
the estimated loading matrix has the corresponding number of non-zero
loadings. This allows the user to directly control the degree of sparsity of
each factor loading. With \code{lasso_penalty}, the SPCA algorithm stops as soon
as the side-constraints of the inherent elastic-net problem are no longer
satisfied. With \code{max_no_steps}, the SPCA algorithm only takes that many LARS
steps for each factor loading's individual elastic-net problem before
stopping. If all criteria are provided, the first one satisfied will stop the
algorithm. For details see also \insertCite{zou2006sparse}{TwoStepSDFM} and
\insertCite{zou2020elnet}{TwoStepSDFM}.

Loosely, each SPCA algorithm iteration solves an elastic-net type problem for
each column of the loading matrix. One can extend this problem to the
adaptive elastic-net \insertCite{zou2009adaptive}{TwoStepSDFM}. The variable
\code{weights} lets the user provide weights for each observation. These weights
must be strictly greater than zero and are normalised internally to represent
relative weights. For more information on the computational implementation of
the weight extension in the context of SPCA see
\insertRef{zou2024general}{TwoStepSDFM}.

In each SPCA algorithm iteration, the function executes an SVD. To this end,
Eigen provides two alternatives \insertCite{eigenweb}{TwoStepSDFM}: Option
\code{precise} makes use of JacobiSVD. This method is numerically more stable, but
computationally costly, especially for medium to large matrices. Option
\code{fast} makes use of BDCSVD. This divide-and-conquer approach can lead to
significant performance gains with respect to large matrices. BDCSVD,
however, can be numerically unstable when Eigen is compiled with aggressive
speed optimisations. In the context of the \code{R}, this should be of no concern.
By default, \code{R} and most packages are compiled with "mild" \code{-O2} optimisation
and without any additional aggressive optimisation flags. Nonetheless, one
should checker whether both variants provide reasonably close results before
switching to \code{fast}. For more information see
\insertRef{eigenweb}{TwoStepSDFM}.
}
\examples{
data(factor_model)
set.seed(17032026)
no_of_factors <- 3
no_of_vars <- dim(factor_model$data)[2]
selected <- rep(floor(0.5 * no_of_vars), no_of_factors)
lasso_penalty <- exp(runif(no_of_factors, -10, 1))
max_no_steps <- 1000
spca_fit <- sparsePCA(data = factor_model$data, delay = factor_model$delay, 
                      selected = selected, no_of_factors = no_of_factors, 
                      ridge_penalty = 1e-2, lasso_penalty = lasso_penalty,
                      max_iterations = 1000, weights = NULL, 
                      max_no_steps = max_no_steps, comp_null = 1e-15,
                      spca_conv_crit = 1e-04, parallel = FALSE, 
                      svd_method = "precise", normalise = FALSE,
                      comp_var_expl = TRUE)
print(spca_fit)
spca_plots <- plot(spca_fit)
spca_plots$`Factor Time Series Plots`
spca_plots$`Loading Matrix Heatmap`
spca_plots$`Meas. Error Var.-Cov. Matrix Heatmap`
spca_plots$`Eigenvalue Plot`
spca_plots$`Variance Explained Chart`

}
\references{
\insertRef{zou2006sparse}{TwoStepSDFM}

\insertRef{zou2009adaptive}{TwoStepSDFM}

\insertRef{eigenweb}{TwoStepSDFM}

\insertRef{zou2020elnet}{TwoStepSDFM}

\insertRef{zou2024general}{TwoStepSDFM}
}
\author{
Domenic Franjic
}
